import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';

import { IBcsNgListOption, BcsNgListOption, ListOptionTypeValuesAndLabels } from '../../../bcs_ng/model/bcs-ng-list-option.model';
import { BcsNgListOptionService } from './bcs-ng-list-option.service';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';

@Component({
  selector: 'jhi-bcs-ng-list-option-update',
  templateUrl: './bcs-ng-list-option-update.component.html'
})
export class BcsNgListOptionUpdateComponent implements OnInit {
  listOption: IBcsNgListOption
  isSaving = false;
  typeValuesAndLabels = ListOptionTypeValuesAndLabels;
  typeValues = Object.keys(this.typeValuesAndLabels);
  selectedType: string;

  editForm = this.fb.group({
    id: [],
    name: [null, Validators.required],
    label: [null, Validators.required],
    type: [null, Validators.required],
  });

  constructor(
    protected bcsNgListOptionService: BcsNgListOptionService,
    protected activatedRoute: ActivatedRoute,
    protected activeModal: NgbActiveModal,
    protected eventManager: JhiEventManager,
    private fb: FormBuilder
  ) {}

  ngOnInit(): void {
    if (this.listOption) this.updateForm(this.listOption);
  }

  updateForm(bcsNgListOption: IBcsNgListOption): void {
    this.editForm.patchValue({
      id: bcsNgListOption.id,
      name: bcsNgListOption.name,
      label: bcsNgListOption.label,
      type: bcsNgListOption.type,
    });
  }

  cancel(): void {
    this.activeModal.dismiss();
  }

  save(): void {
    this.isSaving = true;
    const bcsNgListOption = this.createFromForm();
    if (bcsNgListOption.id !== undefined) {
      this.subscribeToSaveResponse(this.bcsNgListOptionService.update(bcsNgListOption));
    } else {
      this.subscribeToSaveResponse(this.bcsNgListOptionService.create(bcsNgListOption));
    }
  }

  focus(): void {
    const element = document.querySelectorAll('[aria-label="multiselect-search"]') as NodeListOf<HTMLInputElement>;
    if (element[0]) element[0].focus();
    this.editForm.get('type').markAsTouched();
  }

  private createFromForm(): IBcsNgListOption {
    return {
      ...new BcsNgListOption(),
      id: this.editForm.get(['id'])!.value ?? undefined,
      name: this.editForm.get(['name'])!.value,
      label: this.editForm.get(['label'])!.value,
      type: this.editForm.get(['type'])!.value,
    };
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IBcsNgListOption>>): void {
    result.subscribe(
      (res) => this.onSaveSuccess(res.body),
      (err) => this.onSaveError(err)
    );
  }

  protected onSaveSuccess(res: IBcsNgListOption): void {
    this.isSaving = false;
    this.eventManager.broadcast({ name: 'bcsNgListOptionListModification', content: res.id });
    this.activeModal.dismiss();
  }

  protected onSaveError(err: any): void {
    console.error(err);
    this.isSaving = false;
  }
}
