import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { SERVER_BCS_URL } from '../../../app.constants';
import { createRequestOption } from '../../../shared/util/request-util';
import { IBcsNgListOption } from '../../../bcs_ng/model/bcs-ng-list-option.model';
import * as moment from 'moment';
import { map } from 'rxjs/operators';

type EntityResponseType = HttpResponse<IBcsNgListOption>;
type EntityArrayResponseType = HttpResponse<IBcsNgListOption[]>;

@Injectable({ providedIn: 'root' })
export class BcsNgListOptionService {
  public resourceUrl = SERVER_BCS_URL + 'api/bcs-ng-list-options';

  constructor(protected http: HttpClient) {}

  create(bcsNgListOption: IBcsNgListOption): Observable<EntityResponseType> {
    return this.http.post<IBcsNgListOption>(this.resourceUrl, bcsNgListOption, { observe: 'response' });
  }

  update(bcsNgListOption: IBcsNgListOption): Observable<EntityResponseType> {
    return this.http.put<IBcsNgListOption>(this.resourceUrl, bcsNgListOption, { observe: 'response' });
  }

  find(id: string): Observable<EntityResponseType> {
    return this.http.get<IBcsNgListOption>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http.get<IBcsNgListOption[]>(this.resourceUrl, { params: options, observe: 'response' })
    .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));;
  }

  delete(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  erase(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/erase/${id}`, { observe: 'response' });
  }

  protected convertDateFromServer(res: EntityResponseType): EntityResponseType {
    if (res.body) {
      res.body.modifyBlockingDate = res.body.modifyBlockingDate ? moment(res.body.modifyBlockingDate) : undefined;
    }
    return res;
  }

  protected convertDateArrayFromServer(res: EntityArrayResponseType): EntityArrayResponseType {
    if (res.body) {
      res.body.forEach((option: IBcsNgListOption) => {
        option.modifyBlockingDate = option.modifyBlockingDate ? moment(option.modifyBlockingDate) : undefined;
      });
    }
    return res;
  }
}
