import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';

import { DependencyService } from './dependency.service';
import { Dependency, IDependency } from '../../model/dependency.model';

@Component({
  selector: 'jhi-dependency-update',
  templateUrl: './dependency-update.component.html'
})
export class DependencyUpdateComponent implements OnInit {
  isSaving = false;

  editForm = this.fb.group({
    id: [],
    timeGap: [],
    type: []
  });

  constructor(
    protected dependencyService: DependencyService,
    protected activatedRoute: ActivatedRoute,
    private fb: FormBuilder
  ) {}

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ dependency }) => {
      this.updateForm(dependency);
    });
  }

  updateForm(dependency: IDependency): void {
    this.editForm.patchValue({
      id: dependency.id,
      timeGapDuration: dependency.timeGapDuration,
      timeGapUnit: dependency.timeGapUnit,
      type: dependency.type
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const dependency = this.createFromForm();
    if (dependency.id !== undefined) {
      this.subscribeToSaveResponse(this.dependencyService.update(dependency));
    } else {
      this.subscribeToSaveResponse(this.dependencyService.create(dependency));
    }
  }

  private createFromForm(): IDependency {
    return {
      ...new Dependency(),
      id: this.editForm.get(['id'])!.value,
      timeGapDuration: this.editForm.get(['timeGapDuration'])!.value,
      timeGapUnit: this.editForm.get(['timeGapUnit'])!.value,
      type: this.editForm.get(['type'])!.value
    };
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IDependency>>): void {
    result.subscribe(
      () => this.onSaveSuccess(),
      () => this.onSaveError()
    );
  }

  protected onSaveSuccess(): void {
    this.isSaving = false;
    this.previousState();
  }

  protected onSaveError(): void {
    this.isSaving = false;
  }
}
