import { Component, OnInit, OnDestroy } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Subscription } from 'rxjs';
import { JhiEventManager } from 'ng-jhipster';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { DependencyService } from './dependency.service';
import { DependencyDeleteDialogComponent } from './dependency-delete-dialog.component';
import { IDependency } from '../../model/dependency.model';

@Component({
  selector: 'jhi-dependency',
  templateUrl: './dependency.component.html'
})
export class DependencyComponent implements OnInit, OnDestroy {
  dependencies?: IDependency[];
  eventSubscriber?: Subscription;

  constructor(protected dependencyService: DependencyService, protected eventManager: JhiEventManager, protected modalService: NgbModal) {}

  loadAll(): void {
    this.dependencyService.query().subscribe((res: HttpResponse<IDependency[]>) => (this.dependencies = res.body || []));
  }

  ngOnInit(): void {
    this.loadAll();
    this.registerChangeInDependencies();
  }

  ngOnDestroy(): void {
    if (this.eventSubscriber) {
      this.eventManager.destroy(this.eventSubscriber);
    }
  }

  trackId(index: number, item: IDependency): string {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
    return item.id!;
  }

  registerChangeInDependencies(): void {
    this.eventSubscriber = this.eventManager.subscribe('dependencyListModification', () => this.loadAll());
  }

  delete(dependency: IDependency): void {
    const modalRef = this.modalService.open(DependencyDeleteDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.dependency = dependency;
  }
}
