import { Injectable } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Resolve, ActivatedRouteSnapshot, Routes, Router } from '@angular/router';
import { Observable, of, EMPTY } from 'rxjs';
import { flatMap } from 'rxjs/operators';


import { DependencyService } from './dependency.service';
import { DependencyComponent } from './dependency.component';
import { DependencyDetailComponent } from './dependency-detail.component';
import { DependencyUpdateComponent } from './dependency-update.component';
import { Dependency, IDependency } from '../../model/dependency.model';
import { UserRouteAccessService } from 'src/app/core/auth/user-route-access-service';
import { AuthorityName } from 'src/app/shared/constants/authority.constants';

@Injectable({ providedIn: 'root' })
export class DependencyResolve implements Resolve<IDependency> {
  constructor(private service: DependencyService, private router: Router) {}

  resolve(route: ActivatedRouteSnapshot): Observable<IDependency> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        flatMap((dependency: HttpResponse<Dependency>) => {
          if (dependency.body) {
            return of(dependency.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new Dependency());
  }
}

export const dependencyRoute: Routes = [
  {
    path: '',
    component: DependencyComponent,
    data: {
      authorities: [AuthorityName.ADMIN_MOSS, AuthorityName.ADMIN_BCS],
      pageTitle: 'Dépendences BCS'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: ':id/view',
    component: DependencyDetailComponent,
    resolve: {
      dependency: DependencyResolve
    },
    data: {
      authorities: [AuthorityName.ADMIN_MOSS, AuthorityName.ADMIN_BCS],
      pageTitle: 'Dépendence BCS'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: 'new',
    component: DependencyUpdateComponent,
    resolve: {
      dependency: DependencyResolve
    },
    data: {
      authorities: [AuthorityName.ADMIN_MOSS, AuthorityName.ADMIN_BCS],
      pageTitle: 'Dépendence BCS'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: ':id/edit',
    component: DependencyUpdateComponent,
    resolve: {
      dependency: DependencyResolve
    },
    data: {
      authorities: [AuthorityName.ADMIN_MOSS, AuthorityName.ADMIN_BCS],
      pageTitle: 'Dépendence BCS'
    },
    canActivate: [UserRouteAccessService]
  }
];
