import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';
import { IDependency } from '../../model/dependency.model';
import { SERVER_BCS_URL } from 'src/app/app.constants';
import { createRequestOption } from 'src/app/shared/util/request-util';
import { map } from 'rxjs/operators';

type EntityResponseType = HttpResponse<IDependency>;
type EntityArrayResponseType = HttpResponse<IDependency[]>;

@Injectable({ providedIn: 'root' })
export class DependencyService {
  public resourceUrl = SERVER_BCS_URL + 'api/dependencies';

  constructor(protected http: HttpClient) {}

  create(dependency: IDependency): Observable<EntityResponseType> {
    return this.http.post<IDependency>(this.resourceUrl, dependency, { observe: 'response' });
  }

  update(dependency: IDependency): Observable<EntityResponseType> {
    return this.http.put<IDependency>(this.resourceUrl, dependency, { observe: 'response' });
  }

  saveMany(dependencies: IDependency[]): Observable<EntityArrayResponseType> {
    return this.http.put<IDependency[]>(this.resourceUrl + '/many', dependencies, { observe: 'response' });
  }

  find(id: string): Observable<EntityResponseType> {
    return this.http.get<IDependency>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(
    req?: any, queryStringOptions?: { predecessor: string; successsor: string; }): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    let url = this.resourceUrl;
    if (queryStringOptions)
      url += `?predecessor=${queryStringOptions.predecessor}&successor=${queryStringOptions.successsor}`;
    return this.http.get<IDependency[]>(url, { params: options, observe: 'response' });
  }

  delete(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }
}
