import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';

import { DisplayConfigurationService } from './display-configuration.service';
import { DisplayConfiguration, IDisplayConfiguration } from '../../model/display-configuration.model';

@Component({
  selector: 'jhi-display-configuration-update',
  templateUrl: './display-configuration-update.component.html'
})
export class DisplayConfigurationUpdateComponent implements OnInit {
  isSaving = false;

  editForm = this.fb.group({
    id: [],
    name: [],
    type: [],
    isCharter: [],
    properties: []
  });

  constructor(
    protected displayConfigurationService: DisplayConfigurationService,
    protected activatedRoute: ActivatedRoute,
    private fb: FormBuilder
  ) { }

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ displayConfiguration }) => {
      this.updateForm(displayConfiguration);
    });
  }

  updateForm(displayConfiguration: IDisplayConfiguration): void {
    this.editForm.patchValue({
      id: displayConfiguration.id,
      name: displayConfiguration.name,
      type: displayConfiguration.type,
      isCharter: displayConfiguration.isCharter,
      properties: displayConfiguration.properties
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const displayConfiguration = this.createFromForm();
    if (displayConfiguration.id !== undefined) {
      this.subscribeToSaveResponse(this.displayConfigurationService.update(displayConfiguration));
    } else {
      this.subscribeToSaveResponse(this.displayConfigurationService.create(displayConfiguration));
    }
  }

  private createFromForm(): IDisplayConfiguration {
    return {
      ...new DisplayConfiguration(),
      id: this.editForm.get(['id'])!.value,
      name: this.editForm.get(['name'])!.value,
      type: this.editForm.get(['type'])!.value,
      isCharter: this.editForm.get(['isCharter'])!.value,
      properties: this.editForm.get(['properties'])!.value
    };
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IDisplayConfiguration>>): void {
    result.subscribe(
      () => this.onSaveSuccess(),
      () => this.onSaveError()
    );
  }

  protected onSaveSuccess(): void {
    this.isSaving = false;
    this.previousState();
  }

  protected onSaveError(): void {
    this.isSaving = false;
  }
}
