import { Component, OnInit, OnDestroy } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Subscription } from 'rxjs';
import { JhiEventManager } from 'ng-jhipster';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { DisplayConfigurationService } from './display-configuration.service';
import { DisplayConfigurationDeleteDialogComponent } from './display-configuration-delete-dialog.component';
import { IDisplayConfiguration } from '../../model/display-configuration.model';

@Component({
  selector: 'jhi-display-configuration',
  templateUrl: './display-configuration.component.html'
})
export class DisplayConfigurationComponent implements OnInit, OnDestroy {
  displayConfigurations?: IDisplayConfiguration[];
  eventSubscriber?: Subscription;

  constructor(
    protected displayConfigurationService: DisplayConfigurationService,
    protected eventManager: JhiEventManager,
    protected modalService: NgbModal
  ) { }

  loadAll(): void {
    this.displayConfigurationService
      .query()
      .subscribe((res: HttpResponse<IDisplayConfiguration[]>) => (this.displayConfigurations = res.body || []));
  }

  ngOnInit(): void {
    this.loadAll();
    this.registerChangeInDisplayConfigurations();
  }

  ngOnDestroy(): void {
    if (this.eventSubscriber) {
      this.eventManager.destroy(this.eventSubscriber);
    }
  }

  trackId(index: number, item: IDisplayConfiguration): string {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
    return item.id!;
  }

  registerChangeInDisplayConfigurations(): void {
    this.eventSubscriber = this.eventManager.subscribe('displayConfigurationListModification', () => this.loadAll());
  }

  delete(displayConfiguration: IDisplayConfiguration): void {
    const modalRef = this.modalService.open(DisplayConfigurationDeleteDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.displayConfiguration = displayConfiguration;
  }
}
