import { Injectable } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Resolve, ActivatedRouteSnapshot, Routes, Router } from '@angular/router';
import { Observable, of, EMPTY } from 'rxjs';
import { flatMap } from 'rxjs/operators';

import { DisplayConfigurationService } from './display-configuration.service';
import { DisplayConfigurationComponent } from './display-configuration.component';
import { DisplayConfigurationDetailComponent } from './display-configuration-detail.component';
import { DisplayConfigurationUpdateComponent } from './display-configuration-update.component';
import { DisplayConfiguration, IDisplayConfiguration } from '../../model/display-configuration.model';
import { UserRouteAccessService } from 'src/app/core/auth/user-route-access-service';
import { AuthorityName } from 'src/app/shared/constants/authority.constants';

@Injectable({ providedIn: 'root' })
export class DisplayConfigurationResolve implements Resolve<IDisplayConfiguration> {
  constructor(private service: DisplayConfigurationService, private router: Router) { }

  resolve(route: ActivatedRouteSnapshot): Observable<IDisplayConfiguration> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        flatMap((displayConfiguration: HttpResponse<DisplayConfiguration>) => {
          if (displayConfiguration.body) {
            return of(displayConfiguration.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new DisplayConfiguration());
  }
}

export const displayConfigurationRoute: Routes = [
  {
    path: '',
    component: DisplayConfigurationComponent,
    data: {
      authorities: [AuthorityName.ADMIN_MOSS, AuthorityName.ADMIN_BCS],
      pageTitle: 'DisplayConfigurations'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: ':id/view',
    component: DisplayConfigurationDetailComponent,
    resolve: {
      displayConfiguration: DisplayConfigurationResolve
    },
    data: {
      authorities: [AuthorityName.ADMIN_MOSS, AuthorityName.ADMIN_BCS],
      pageTitle: 'DisplayConfigurations'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: 'new',
    component: DisplayConfigurationUpdateComponent,
    resolve: {
      displayConfiguration: DisplayConfigurationResolve
    },
    data: {
      authorities: [AuthorityName.ADMIN_MOSS, AuthorityName.ADMIN_BCS],
      pageTitle: 'DisplayConfigurations'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: ':id/edit',
    component: DisplayConfigurationUpdateComponent,
    resolve: {
      displayConfiguration: DisplayConfigurationResolve
    },
    data: {
      authorities: [AuthorityName.ADMIN_MOSS, AuthorityName.ADMIN_BCS],
      pageTitle: 'DisplayConfigurations'
    },
    canActivate: [UserRouteAccessService]
  }
];
