import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';
import { SERVER_BCS_URL } from 'src/app/app.constants';
import { IDisplayConfiguration } from '../../model/display-configuration.model';
import { createRequestOption } from 'src/app/shared/util/request-util';


type EntityResponseType = HttpResponse<IDisplayConfiguration>;
type EntityArrayResponseType = HttpResponse<IDisplayConfiguration[]>;

@Injectable({ providedIn: 'root' })
export class DisplayConfigurationService {
  public resourceUrl = SERVER_BCS_URL + 'api/display-configurations';

  constructor(protected http: HttpClient) { }

  create(displayConfiguration: IDisplayConfiguration): Observable<EntityResponseType> {
    return this.http.post<IDisplayConfiguration>(this.resourceUrl, displayConfiguration, { observe: 'response' });
  }

  update(displayConfiguration: IDisplayConfiguration): Observable<EntityResponseType> {
    return this.http.put<IDisplayConfiguration>(this.resourceUrl, displayConfiguration, { observe: 'response' });
  }

  find(id: string): Observable<EntityResponseType> {
    return this.http.get<IDisplayConfiguration>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http.get<IDisplayConfiguration[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }
}
