import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { Observable } from 'rxjs';

import { DisplayRuleService } from './display-rule.service';
import { IDisplayRule } from '../../model/display-rule.model';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { TaskableEntity } from '../../model/taskable-entity.model';
import { AccountService } from 'src/app/core/auth/account.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { JhiEventManager } from 'ng-jhipster';

@Component({
  selector: 'jhi-display-rule-update',
  templateUrl: './display-rule-update.component.html'
})
export class DisplayRuleUpdateComponent extends BaseComponent implements OnInit {
  rule: IDisplayRule;
  isCharter: boolean;
  isSaving = false;
  taskableEntities: TaskableEntity[] = [];

  constructor(protected displayRuleService: DisplayRuleService,
    public activeModal: NgbActiveModal,
    protected eventManager: JhiEventManager,
    protected accountService: AccountService) {
    super(accountService)
  }

  ngOnInit(): void {
    this.isCharter = this.rule?.isCharter ?? false;
  }

  save(displayRule: IDisplayRule): void {
    this.isSaving = true;
    displayRule.ownerId = this.userIdentity.id;
    displayRule.isCharter = this.isCharter;
    if (this.rule?.id !== undefined) {
      displayRule.id = this.rule.id;
      this.subscribeToSaveResponse(this.displayRuleService.update(displayRule));
    } else {
      displayRule.id = undefined;
      this.subscribeToSaveResponse(this.displayRuleService.create(displayRule));
    }
  }

  cancel(): void {
    this.activeModal.close();
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IDisplayRule>>): void {
    result.subscribe(
      () => this.onSaveSuccess(),
      () => this.onSaveError()
    );
  }

  protected onSaveSuccess(): void {
    this.eventManager.broadcast('displayRuleListModification');
    this.isSaving = false;
    this.activeModal.close();
  }

  protected onSaveError(): void {
    this.isSaving = false;
    this.activeModal.close();
  }
}
