import { Component, OnInit, OnDestroy, ViewChild } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Subscription } from 'rxjs';
import { JhiEventManager } from 'ng-jhipster';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { DisplayRuleService } from './display-rule.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { AccountService } from 'src/app/core/auth/account.service';
import { MatSort } from '@angular/material/sort';
import { MatPaginator } from '@angular/material/paginator';
import { MatTableDataSource } from '@angular/material/table';
import { TaskableEntityService } from '../taskable-entity/taskable-entity.service';
import { TaskableEntity } from '../../model/taskable-entity.model';
import { DialogDeleteComponent } from 'src/app/shared/dialog/delete/dialog-delete.component';
import { DialogEraseComponent } from 'src/app/shared/dialog/erase/dialog-erase.component';
import { DialogRebornComponent } from 'src/app/shared/dialog/reborn/dialog-reborn.component';
import { DisplayRuleUpdateComponent } from './display-rule-update.component';
import { IDisplayRule } from '../../model/display-rule.model';

@Component({
  selector: 'jhi-display-rule',
  templateUrl: './display-rule.component.html',
  styleUrls: ['../road-map/update/display-configuration-rules/display-configuration-rules-dialog.scss']
})
export class DisplayRuleComponent extends BaseComponent implements OnInit, OnDestroy {
  displayRules?: IDisplayRule[];
  eventSubscriber?: Subscription;
  taskableEntities: TaskableEntity[] = [];

  dataSource = new MatTableDataSource<IDisplayRule>();
  @ViewChild(MatPaginator, { static: true }) paginator!: MatPaginator;
  public pageSize = 10;
  @ViewChild(MatSort) matSorter!: MatSort;
  load: boolean = false;

  constructor(
    protected displayRuleService: DisplayRuleService,
    protected eventManager: JhiEventManager,
    protected modalService: NgbModal,
    protected accountService: AccountService,
    protected taskableEntityService: TaskableEntityService,
  ) { super(accountService) }

  loadMyDisplayRule(): void {
    this.load = true;
    this.displayRuleService.findByOwner(this.userIdentity?.id).subscribe((res: HttpResponse<IDisplayRule[]>) => {
      let displayRules = res.body || [];
      if (this.isAdminMOSS || this.isAdminBCS) {
        this.displayRuleService.findCharter().subscribe((resCharter: HttpResponse<IDisplayRule[]>) => {
          resCharter?.body.forEach(c => { if (!displayRules.find(r => r.id === c.id)) displayRules.push(c) })
          this.updateTable(displayRules);
        });
      } else this.updateTable(displayRules);
    });
    this.taskableEntityService.query().subscribe(res => this.taskableEntities = res.body || []);
  }

  updateTable(displayRules: IDisplayRule[]) {
    this.displayRules = displayRules;
    this.dataSource.data = this.displayRules;
    this.dataSource.sort = this.matSorter;
    if (!this.isScully) {
      this.dataSource.paginator = this.paginator;
      this.dataSource.paginator._intl.itemsPerPageLabel = "Items par page :";
    }
    this.load = false;
  }

  ngOnInit(): void {
    this.loadMyDisplayRule();
    this.registerChangeInDisplayRules();
  }

  ngOnDestroy(): void {
    if (this.eventSubscriber) {
      this.eventManager.destroy(this.eventSubscriber);
    }
  }

  registerChangeInDisplayRules(): void {
    this.eventSubscriber = this.eventManager.subscribe('displayRuleListModification', () => this.loadMyDisplayRule());
  }

  defineParamsDialog(modalRef, id) {
    modalRef.componentInstance.id = id;
    modalRef.componentInstance.entityMsg = "cette règle";
    modalRef.componentInstance.serviceEntity = this.displayRuleService;
    modalRef.componentInstance.eventManagerName = "displayRuleListModification";
  }

  update(displayRule: IDisplayRule | undefined): void {
    const modalRef = this.modalService.open(DisplayRuleUpdateComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.rule = displayRule;
    modalRef.componentInstance.taskableEntities = this.taskableEntities;
  }

  delete(displayRule: IDisplayRule): void {
    const modalRef = this.modalService.open(DialogDeleteComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, displayRule.id);
  }

  reborn(displayRule: IDisplayRule): void {
    const modalRef = this.modalService.open(DialogRebornComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, displayRule.id);
  }

  erase(displayRule: IDisplayRule): void {
    const modalRef = this.modalService.open(DialogEraseComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, displayRule.id);
  }

  applyFilter(event: Event): void {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();
    if (this.dataSource?.paginator) {
      this.dataSource.paginator.firstPage();
    }
  }

  getTaskableEntity(id: string) {
    return this.taskableEntities.find(t => t.id === id)?.name;
  }
}
