import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';
import { SERVER_BCS_URL } from 'src/app/app.constants';
import { createRequestOption } from 'src/app/shared/util/request-util';
import { IDisplayRule } from '../../model/display-rule.model';


type EntityResponseType = HttpResponse<IDisplayRule>;
type EntityArrayResponseType = HttpResponse<IDisplayRule[]>;

@Injectable({ providedIn: 'root' })
export class DisplayRuleService {
  public resourceUrl = SERVER_BCS_URL + 'api/display-rules';

  constructor(protected http: HttpClient) { }

  create(displayRule: IDisplayRule): Observable<EntityResponseType> {
    return this.http.post<IDisplayRule>(this.resourceUrl, displayRule, { observe: 'response' });
  }

  update(displayRule: IDisplayRule): Observable<EntityResponseType> {
    return this.http.put<IDisplayRule>(this.resourceUrl, displayRule, { observe: 'response' });
  }

  saveMany(displayRules: IDisplayRule[]) {
    return this.http.put<IDisplayRule[]>(this.resourceUrl + '/many', displayRules, { observe: 'response' });
  }

  findByOwner(id: string): Observable<EntityArrayResponseType> {
    return this.http.get<IDisplayRule[]>(`${this.resourceUrl}/byOwner/${id}`, { observe: 'response' });
  }

  findCharter(): Observable<EntityArrayResponseType> {
    return this.http.get<IDisplayRule[]>(this.resourceUrl + '/get/charter', { observe: 'response' });
  }

  find(id: number): Observable<EntityResponseType> {
    return this.http.get<IDisplayRule>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http.get<IDisplayRule[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  erase(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/erase/${id}`, { observe: 'response' });
  }
}
