import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';
import { SERVER_BCS_URL } from 'src/app/app.constants';
import { createRequestOption } from 'src/app/shared/util/request-util';
import { IDisplayTask } from '../../model/display-task.model';


type EntityResponseType = HttpResponse<IDisplayTask>;
type EntityArrayResponseType = HttpResponse<IDisplayTask[]>;

@Injectable({ providedIn: 'root' })
export class DisplayTaskService {
  public resourceUrl = SERVER_BCS_URL + 'api/display-tasks';

  constructor(protected http: HttpClient) { }

  create(displayTask: IDisplayTask): Observable<EntityResponseType> {
    return this.http.post<IDisplayTask>(this.resourceUrl, displayTask, { observe: 'response' });
  }

  saveMany(displayTasks: IDisplayTask[]): Observable<EntityArrayResponseType> {
    return this.http.post<IDisplayTask[]>(this.resourceUrl + '/many', displayTasks, { observe: 'response' });
  }

  update(displayTask: IDisplayTask): Observable<EntityResponseType> {
    return this.http.put<IDisplayTask>(this.resourceUrl, displayTask, { observe: 'response' });
  }

  reincarnateRecursively(idList: string[]): Observable<EntityResponseType> {
    return this.http.put<IDisplayTask>(this.resourceUrl + '/reincarnate/recursive', { idList }, { observe: 'response' });
  }

  find(id: string): Observable<EntityResponseType> {
    return this.http.get<IDisplayTask>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(req?: any, object?: string): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    let url = this.resourceUrl;
    if (object) url += `?object=${object}`;
    return this.http.get<IDisplayTask[]>(url, { params: options, observe: 'response' });
  }

  getRpsObjects(displayTaskId: string): Observable<string[]> {
    return this.http.get<string[]>(this.resourceUrl + `/rps/${displayTaskId}`);
  }

  deleteManyRecursively(idList: string[]): Observable<HttpResponse<{}>> {
    return this.http.post(`${this.resourceUrl}/delete/recursive`, idList, { observe: 'response' });
  }

  delete(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  deleteMany(idList: string[]): Observable<HttpResponse<{}>> {
    return this.http.post(`${this.resourceUrl}/many`, idList, { observe: 'response' });
  }

  eraseManyRecursively(idList: string[]): Observable<HttpResponse<{}>> {
    return this.http.post(`${this.resourceUrl}/erase/recursive`, idList, { observe: 'response' });
  }
}
