import { Component, OnInit } from '@angular/core';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';
import { BcsNgHistoryService } from 'src/app/bcs_ng/entity/history/bcs-ng-history.service';
import { Baseline } from 'src/app/bcs_ng/model/enumeration/baseline.enum';
import { IHistoric } from 'src/app/bcs_ng/model/historic.model';

@Component({
  selector: 'jhi-bcs-ng-historic-update',
  templateUrl: './bcs-ng-historic-update.component.html',
})
export class BcsNgHistoricUpdateComponent implements OnInit {
  id: string;
  historicToUpdate: IHistoric;
  fromHistoric: boolean = true;
  newBaselineValue: Baseline;

  noneToUpdate: boolean = false;

  Baseline = Baseline;

  valuesAndLabels = {
    'NONE': 'Aucune baseline',
    'RPS': 'RPS passée',
    'RPS_A_VENIR': 'RPS à venir',
    'VISIBLE_CLIENT': 'Visible client',
  }

  constructor(
    public activeModal: NgbActiveModal,
    protected historicService: BcsNgHistoryService,
    protected eventManager: JhiEventManager
  ) { }

  ngOnInit(): void {
    // Fetch history
    if (!this.historicToUpdate) {
      this.historicService.findLatestByEntity(this.id, undefined, true).subscribe(res => {
        this.historicToUpdate = res.body;
        if (!this.historicToUpdate) this.noneToUpdate = true;
        if (
          this.historicToUpdate &&
          this.newBaselineValue === Baseline.RPS &&
          this.historicToUpdate.baseline !== Baseline.RPS_A_VENIR &&
          this.historicToUpdate.baseline !== Baseline.RPS
        ) this.noneToUpdate = true;
        if (
          this.historicToUpdate &&
          this.newBaselineValue === Baseline.RPS_A_VENIR &&
          this.historicToUpdate.baseline !== Baseline.RPS_A_VENIR &&
          this.historicToUpdate.baseline === Baseline.RPS
        ) this.noneToUpdate = true;
      });
    }
  }

  disabled(): boolean {
    return (this.historicToUpdate?.baseline === this.newBaselineValue && !this.fromHistoric) || !this.Baseline || this.noneToUpdate;
  }

  save(): void {
    this.historicService.update({
      ...this.historicToUpdate,
      baseline: this.newBaselineValue,
    }).subscribe(
      () => {
        this.eventManager.broadcast('historyListModification');
        this.activeModal.close();
      },
      err => {
        if (err) console.error(err);
      }
    );
    return;
  }

  cancel(): void {
    this.activeModal.dismiss();
  }
}