import { HttpResponse } from '@angular/common/http';
import { ChangeDetectorRef, Component, EventEmitter, Input, OnInit, Output } from "@angular/core";
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';
import { Subscription } from 'rxjs';
import { AccountService } from 'src/app/core/auth/account.service';
import { DialogDeleteComponent } from 'src/app/shared/dialog/delete/dialog-delete.component';
import { DialogEraseComponent } from 'src/app/shared/dialog/erase/dialog-erase.component';
import { DialogRebornComponent } from 'src/app/shared/dialog/reborn/dialog-reborn.component';
import { Baseline } from '../../model/enumeration/baseline.enum';
import { CrudType } from '../../model/enumeration/crud-type';
import { IHistoric } from '../../model/historic.model';
import { BcsNgHistoricUpdateComponent } from './bcs-ng-historic-update.component';

import { BcsNgHistoryService } from './bcs-ng-history.service';

@Component({
  selector: 'jhi-bcs-ng-history',
  templateUrl: './bcs-ng-history.component.html',
  styleUrls: ['./bcs-ng-history.scss']
})
export class BcsNgHistoryComponent implements OnInit {
  @Input() id: string;
  @Input() inModal: boolean;
  @Input() milestoneEntity: boolean = false;
  @Input() isAllowedToUpdate: boolean = false;
  @Input() fromDetail: boolean = false;
  @Output() emitHistoryDetail: EventEmitter<any> = new EventEmitter<any>();

  data: IHistoric[] = [];
  eventSubscriber?: Subscription;
  selectedHistory?: string;

  Baseline = Baseline;
  CrudType = CrudType;

  constructor(
    protected historyService: BcsNgHistoryService,
    protected accountService: AccountService,
    protected changeDetectorRef: ChangeDetectorRef,
    protected eventManager: JhiEventManager,
    protected modalService: NgbModal
  ) { }

  ngOnInit(): void {
    this.loadHistories();
    this.registerChangeInHistory();
  }

  private loadHistories(): void {
    this.historyService.findByEntity(this.id).subscribe((res: HttpResponse<IHistoric[]>) => {
      this.onSuccess(res);
    });
  }

  private onSuccess(res: HttpResponse<IHistoric[]>) {
    this.data = res.body;
  }

  update(id: string, historic: IHistoric): void {
    const modalRef = this.modalService.open(BcsNgHistoricUpdateComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.id = id;
    modalRef.componentInstance.historicToUpdate = historic;
    modalRef.componentInstance.fromHistoric = true;
  }

  erase(change: IHistoric): void {
    const modalRef = this.modalService.open(DialogEraseComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, change.id);
  }

  delete(change: IHistoric): void {
    const modalRef = this.modalService.open(DialogDeleteComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, change.id);
  }

  reborn(change: IHistoric): void {
    const modalRef = this.modalService.open(DialogRebornComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, change.id);
  }

  registerChangeInHistory(): void {
    this.eventSubscriber = this.eventManager.subscribe('historyListModification', () => this.loadHistories());
  }

  showDelta(change: IHistoric): void {
    this.emitHistory(change);
  }

  emitHistory(history: IHistoric): void {
    this.emitHistoryDetail.emit({
      history,
      historyNow: this.data[0],
      historyPrevious: this.data[this.data?.findIndex(h => h.id === history.id) + 1]
    })
  }

  defineParamsDialog(modalRef, id) {
    modalRef.componentInstance.id = id;
    modalRef.componentInstance.entityMsg = "cet historique";
    modalRef.componentInstance.serviceEntity = this.historyService;
    modalRef.componentInstance.eventManagerName = "historyListModification";
  }

  get isAdminBCS(): boolean {
    return this.accountService.isAdminBCS;
  }
}