import { Injectable } from '@angular/core';
import { ResourceStoreConfig, SchedulerConfig } from '@bryntum/scheduler';
import zipcelx from 'zipcelx';
import { DisplayConfigBryntum } from './display-config';

@Injectable()
export class GeneralConfig {
    constructor(protected displayConfig: DisplayConfigBryntum) { }

    storeConfig: Partial<ResourceStoreConfig> = {
        tree: true,
        transformFlatData: true,
        syncDataOnLoad: false,
        reapplyFilterOnAdd: true,
        reapplyFilterOnUpdate: true,
    };

    customEventLayout = {
        layoutFn: (event: any) => { console.log("test layout!!!!!!") }
    }

    customEventLayout2 = {
        layoutFn(items) {
            let bottom = 50;
            console.log("test layout!!!!!!")
            items.forEach((item, i) => {
                // Confirmed events should be higher
                item.height = item.confirmed ? 60 : 30;

                // Position every event 10px further down
                item.top = i * 10;

                bottom = Math.max(bottom, item.top + item.height);
            });

            // Return row height in pixels
            return bottom;
        }
    };
    schedulerConfig: Partial<SchedulerConfig> = {
        rowHeight: 40,
        resourceMargin: 6,
        barMargin: 2,
        columns: [
            {
                text: 'Nom',
                field: 'name',
                width: 250,
                type: 'tree',
                expandedFolderIconCls: null,
                collapsedFolderIconCls: null,
                leafIconCls: null,
                editable: false,
                htmlEncode: false,
                editor: false,
                renderer: ({ value, record, cellElement }) => {
                    cellElement.style.color = record.get('fontColor');
                    cellElement.style.fontSize = record.get('fontSize') + 'px';
                    return value;
                }
            }
        ],
        startDate: new Date(2022, 8, 1),
        endDate: new Date(2025, 8, 1),
        eventStyle: 'rounded',
        resourceStore: this.storeConfig,
        eventStore: {
            reapplyFilterOnUpdate: true,
        },
        features: {
            tree: true,
            filterBar: true,
            regionResize: true,
            eventEdit: false,
            dependencies: {
                radius: 5,
                allowCreate: false,
                allowDropOnEventBar: true
            },
            timeRanges: {
                showCurrentTimeLine: false,
                showHeaderElements: false,
            },
            excelExporter: {
                // Choose the date format for date fields
                dateFormat: 'DD/MM/YYYY',
                zipcelx: zipcelx,
                exporterConfig: {
                    columns: [
                        { text: 'Nom', field: 'name', width: 560 }
                    ],
                    eventColumns: [
                        { text: 'Abréviation', field: 'name' },
                        { text: 'Début', field: 'startDate', width: 140 },
                        { text: 'Fin', field: 'endDate', width: 140 }
                    ]
                }
            },
            labels: {
                top: {
                    renderer: ({ eventRecord }): string => {
                        const fontSize: number = (eventRecord?.getData('fontSize') ?? 16) - 5;
                        if (eventRecord.getData('label') === 'top') return `<div style="color : ${eventRecord.getData('eventColor') ?? '#888'}; font-size: ${fontSize}px;">${eventRecord.name}</div>`;
                        else {
                            return `<div></div>`;
                        }
                    }
                },
                bottom: {
                    renderer: ({ eventRecord }): string => {
                        const fontSize: number = (eventRecord?.getData('fontSize') ?? 16) - 5;
                        if (eventRecord.getData('label') === 'bottom') return `<div style="color : ${eventRecord.getData('eventColor') ?? '#888'}; font-size: ${fontSize}px;">${eventRecord.name}</div>`;
                        else if (eventRecord.cls["b-sch-event-delay-arrow"]) {
                            return eventRecord.cls["b-sch-event-delay-arrow"] ? (eventRecord.name ? `Décalé par : ${eventRecord.name}` : `<div></div>`) : `<div></div>`;
                        }
                        else {
                            return `<div></div>`;
                        }
                    }
                },
                right: {
                    renderer: ({ eventRecord }): string => {
                        const fontSize: number = (eventRecord?.getData('fontSize') ?? 16) - 5;
                        if (eventRecord.getData('label') === 'right') return `<div style="color : ${eventRecord.getData('eventColor') ?? '#888'}; font-size: ${fontSize}px;">${eventRecord.name}</div>`;
                        else {
                            return `<div></div>`;
                        }
                    }
                },
                left: {
                    renderer: ({ eventRecord }): string => {
                        const fontSize: number = (eventRecord?.getData('fontSize') ?? 16) - 5;
                        if (eventRecord.getData('label') === 'left') return `<div style="color : ${eventRecord.getData('eventColor') ?? '#888'}; font-size: ${fontSize}px;">${eventRecord.name}</div>`;
                        else {
                            return `<div></div>`;
                        }
                    }
                },
            }
        },
        store: {
            tree: true,
            transformFlatData: true,
        },
        eventLayout: 'mixed',
        //eventLayout: this.customEventLayout,
        eventRenderer: ({ eventRecord, renderData }) => {
            if (eventRecord.cls["milestone-event"]) renderData.wrapperStyle = 'z-index: 20 !important;';
            if (eventRecord.cls["clone-event"]) renderData.wrapperStyle = 'z-index: 20 !important;';
            if (eventRecord.cls["operation-event"]) renderData.wrapperStyle = 'z-index: 10 !important;';
            if (eventRecord.cls["b-sch-event-delay-arrow"] || eventRecord.cls["b-sch-event-delay-arrow-rps"])
                renderData.style = `background-color: transparent`;
            return {
                name: eventRecord.name,
                eventDelayArrow: eventRecord.cls["b-sch-event-delay-arrow"],
                eventDelayArrowRps: eventRecord.cls["b-sch-event-delay-arrow-rps"],
                eventRecord: eventRecord,
            };
        },
        eventBodyTemplate: (data: any) => {
            const fontSize = data?.eventRecord?.getData('fontSize') ?? 16;
            const rowHeight = data?.eventRecord?.getData('rowHeight') ?? 40;
            let marginFactor: number = 0.33;
            if (rowHeight >= 60) marginFactor = 0.25;
            if (rowHeight < 60 && rowHeight >= 40) marginFactor = 0.33;
            if (rowHeight < 40 && rowHeight > 20) marginFactor = 0.42;
            if (rowHeight <= 20) marginFactor = 0.55;
            const eventHeight = Number(rowHeight) - marginFactor * Number(rowHeight);

            const eventColor = data.eventRecord.getData('eventColor');
            if (data.eventRecord.getData('offsetMultiColorEventConfig')) {
                const offset = this.displayConfig.calculateOffsetMultiColorEventConfig(data.eventRecord, data.eventRecord.getData('offsetMultiColorEventConfig'));

                let result = !data.eventRecord.getData('label') ? `<div class='default-display' style="color : ${eventColor}; z-index : 1; font-size: ${fontSize}px;"><span>${data.name}</span></div>`
                    : `<div class='default-display' style="color : ${eventColor}; z-index : 1; font-size: ${fontSize}px;"></div>`;

                offset?.forEach((o, i) => {
                    result += `<div class="nested" style="left : ${o.startOffset}px; width: ${o.widthOffset}px; background-color : ${o.color}; 
                    height: ${eventHeight}px; border-radius: 2em; opacity : 0.8"></div>`;
                })
                return result;
            }
            else if (data.eventRecord.getData('iconCls')) {
                const displayName = data.eventRecord.getData('label') ? '' : `<span style="font-size: ${fontSize}px;">${data.name}</span>`;
                return `<div class="d-flex align-items-center">
                    <i class="${data.eventRecord.getData('iconCls')}" style="color : ${eventColor};"></i>
                    ` + displayName +
                    `</div>`;
            }
            else if (data.eventRecord.getData('label')) {
                return `<div class="h-100 w-100" style="background-color : ${eventColor}50;  border-radius: 2em;"></div>`;
            }
            else if (eventColor) {
                return `<div class='default-display' 
                style="color : ${eventColor}; font-size: ${fontSize}px; background-color : ${eventColor}50;"><span>${data.name}</span></div>`;
            }
            else if (data.eventDelayArrow || data.eventDelayArrowRps) {
                const label = data.name ? `Décalé par : ${data.name}` : '';
                const rpsSuffix = data.eventDelayArrowRps ? '-rps' : '';
                const arrowColor = data.eventDelayArrowRps ?
                    data.eventRecord.getData('delayRpsArrowConf')?.bgColor :
                    data.eventRecord.getData('delayArrowConf')?.bgColor;
                return `<div class="b-sch-event-delay-arrow${rpsSuffix}">
                    <div class="b-sch-event-delay-arrow-body${rpsSuffix}" style="background-color : ${arrowColor}">${label}</div>
                    <i class="b-fa fa-solid fa-chevron-right b-sch-event-delay-arrow-head${rpsSuffix}" style="color : ${arrowColor}"></i>
                </div>`;
            }
            else {
                return `<div class='default-display' style="font-size: ${fontSize}px;"><span>${data.name}</span></div>`;
            }
        },
        viewPreset: 'year',
        subGridConfigs: {
            width: 500
        },
        maxZoomLevel: 11,
        createEventOnDblClick: false
    };
}