import { Injectable } from '@angular/core';
import { Model } from '@bryntum/scheduler';
import { JhiEventManager } from 'ng-jhipster';
import { IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';

/** Listeners for events outside Bryntum menus */
@Injectable()
export class BryntumListeners {
    recordStateMap: { [key: string]: IBryntumRecordState } = {};

    listeners = {
        afterDependencyCreateDrop: (event: any) => console.log(event),
        beforeDependencyEdit: (event: any) => {
            this.onUpdateDependency(event);
            return false;
        },
        beforeEventAdd: (_: any) => {
            return false;
        },
        beforeEventEdit: (_: any) => {
            return false;
        },
        beforeAssignmentSelectionChange: (_: any) => {
            return false;
        },
        collapseNode: (event: any) => this.onCollapseNode(event.record),
        expandNode: (event: any) => this.onExpandNode(event.record)
    }

    constructor(protected eventManager: JhiEventManager) { }

    onUpdateDependency(dependencyRecord: any): void {
        this.eventManager.broadcast({ name: 'editDependency', content: dependencyRecord });
    }

    expandNode(node: Model, expand: boolean = true,) {
        if (node.getData('id')) this.recordStateMap[node.getData('id')].resource.expanded = expand;
    }

    onCollapseNode(nodeRecord: Model): void {
        this.expandNode(nodeRecord, false);
    }

    onExpandNode(nodeRecord: Model): void {
        this.expandNode(nodeRecord);
    }
}