import { Injectable } from '@angular/core';
import { Model, Scheduler } from '@bryntum/scheduler';
import { JhiEventManager } from 'ng-jhipster';
import { IBryntumEventClone } from 'src/app/bcs_ng/model/bryntum/bryntum-event-clone.model';
import { IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';

@Injectable()
export class ConfigCellMenu {
    cellMenu = {
        items: {
            removeRow: false,
            cut: false,
            copy: false,
            update: {
                text: "Modifier localement",
                icon: 'b-fa far fa-edit',
                weight: 200,
                onItem: (cellRecord) => this.onUpdate(cellRecord),
            },
            format: {
                text: "Formatter le texte",
                icon: 'b-fa far fa-paragraph',
                weight: 200,
                onItem: (cellRecord) => this.onFormat(cellRecord),
            },
            hide: {
                text: "Cacher avec ses enfants",
                icon: 'b-fa far fa-eye',
                weight: 200,
                onItem: (cellRecord) => this.onHideRecord(cellRecord),
            },
            hideWithChildren: {
                text: "Cacher en gardant ses enfants",
                icon: 'b-fa far fa-eye',
                weight: 200,
                onItem: (cellRecord) => this.onHideRecordAndElevateChildren(cellRecord),
            },
            delete: {
                text: "Supprimer (avec ses enfants)",
                icon: 'b-fa far fa-trash',
                weight: 200,
                onItem: (cellRecord) => this.onRemoveRecord(cellRecord),
            },
        }
    }

    scheduler?: Scheduler;
    recordStateMap: { [key: string]: IBryntumRecordState } = {};
    initialRecordStateMap: { [key: string]: IBryntumRecordState } = {};
    clonesMap: { [key: string]: IBryntumEventClone } = {};

    constructor(protected eventManager: JhiEventManager) { }

    onHideRecord(cellRecord: any): void {
        this.hideRecord(cellRecord.record, true);
    }

    onHideRecordAndElevateChildren(cellRecord: any): void {
        const parentId = cellRecord.record.get('parentId');
        const childrenIds = cellRecord.record.children?.map(c => c.id);
        childrenIds?.forEach(cId => {
            const child = this.scheduler.resourceStore.getById(cId);
            child.set('parentId', parentId);
            this.recordStateMap[child.id].resource.parentId = parentId;
        });

        // Hide clone if it exists
        const clone = Object.values(this.clonesMap).find(c => c.eventId === cellRecord.record.id);
        if (clone) {
            const cloneRecord = this.scheduler.eventStore.getById(clone.id);
            this.scheduler.eventStore.remove(cloneRecord);
        }

        // We also have to change the parentId of hidden and deleted children
        Object.values(this.recordStateMap).forEach(record => {
            if (!record.resource.displayed && record.resource.parentId === cellRecord.record.id) {
                record.resource.parentId = parentId;
            }
        });

        this.recordStateMap[cellRecord.record.id].resource.hidden = true;
        this.recordStateMap[cellRecord.record.id].resource.displayed = false;
        cellRecord.record.set('displayed', false);
    }

    hideRecord(record: Model, firstIteration: boolean): void {
        if (this.recordStateMap[record.id].resource.hidden) return;

        if (firstIteration) this.recordStateMap[record.id].resource.hidden = true;
        this.recordStateMap[record.id].resource.displayed = false;

        // Hide clone if it exists
        const clone = Object.values(this.clonesMap).find(c => c.eventId === record.id);
        if (clone) {
            const cloneRecord = this.scheduler.eventStore.getById(clone.id);
            this.scheduler.eventStore.remove(cloneRecord);
        }

        record.set('displayed', false);
        record.allChildren.forEach(child => {
            this.hideRecord(child, false);
        });
    }

    onRemoveRecord(cellRecord: any): void {
        this.removeRecord(cellRecord.record);
    }

    removeRecord(record: Model): void {
        this.recordStateMap[record.id].resource.deleted = true;
        this.recordStateMap[record.id].resource.displayed = false;

        // Hide clone if it exists
        const clone = Object.values(this.clonesMap).find(c => c.eventId === record.id);
        if (clone) {
            const cloneRecord = this.scheduler.eventStore.getById(clone.id);
            this.scheduler.eventStore.remove(cloneRecord);
        }

        record.set('displayed', false);
        record.allChildren.forEach(child => {
            this.removeRecord(child);
        });
    }

    onUpdate(cellRecord: any) {
        this.eventManager.broadcast({ name: 'openUpdateModal', content: cellRecord });
    }

    onFormat(cellRecord: any) {
        this.eventManager.broadcast({ name: 'onFormat', content: cellRecord });
    }
}