import { Injectable } from '@angular/core';
import { Model, Scheduler } from '@bryntum/scheduler';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';
import { IBryntumDelay } from 'src/app/bcs_ng/model/bryntum/bryntum-delay.model';
import { IBryntumEventClone } from 'src/app/bcs_ng/model/bryntum/bryntum-event-clone.model';
import { IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';
import { DisplayConfTaskArrowProperties } from 'src/app/bcs_ng/model/bryntum/display-configuration/display-conf-task-arrow-properties';
import { DisplayConfTaskProperties } from 'src/app/bcs_ng/model/bryntum/display-configuration/display-conf-task-properties';
import { DisplayConfiguration } from 'src/app/bcs_ng/model/display-configuration.model';
import { DisplayTask, IDisplayTask } from 'src/app/bcs_ng/model/display-task.model';
import { DisplayConfigurationType } from 'src/app/bcs_ng/model/enumeration/display-configuration-type';
import { Utils } from 'src/app/bcs_ng/utils/util';
import { DisplayTaskService } from '../../../display-task/display-task.service';
import { DisplayConfigurationArrowDialogcomponent } from '../../update/display-configuration-arrow/display-configuration-arrow-dialog.component';
import { DisplayConfigurationDialogcomponent } from '../../update/display-configuration/display-configuration-dialog.component';
import { DisplayConfigBryntum } from '../display-config';

@Injectable()
export class ConfigEventMenu {
    eventMenu = {
        processItems: ({ items, eventRecord, assignmentRecord }) => {
            this.getNeighbours(eventRecord.data);
            this.getParents(this.recordStateMap[eventRecord.data.id]);
            items.editEvent = false;
            items.copyEvent = false;
            items.cutEvent = false;
            items.unassignEvent = false;
            items.deleteEvent = false;

            const isClone = this.clonesMap[eventRecord.id] !== undefined;
            const isDelay = this.delaysMap[eventRecord.id] !== undefined;

            if (!isDelay) {

                if (!isClone) {
                    items.displayConfiguration = {
                        text: "Paramètres d'affichage",
                        icon: 'fas fa-paint-brush',
                        weight: 510,
                        onItem: (event) => this.openDisplayConfiguration(event),
                    },
                        items.arrowDisplayConfiguration = {
                            text: "Paramètres des flèches de retard",
                            icon: 'b-fa fas fa-arrow-right',
                            weight: 510,
                            onItem: (event) => this.openArrowDisplayConfiguration(event),
                        }
                }
                // Dependencies
                if (!isClone) {
                    items.hideDependencies = {
                        text: "Cacher/afficher les liens de prédécesseurs",
                        icon: 'b-fa far fa-eye',
                        weight: 510,
                        onItem: () => this.onHideOrDisplayDependencies(eventRecord),
                    };
                }

                // Event display location
                items.move = {
                    text: 'Afficher dans ...',
                    icon: 'b-fa fas fa-arrows',
                    menu: []
                };

                if (!isClone) {
                    this.parents.forEach(p => {
                        items.move.menu.push({
                            text: p.resource.name ?? p.resource.object.name,
                            icon: 'b-fa fas fa-arrow-up',
                            weight: 510,
                            onItem: ({ eventRecord }) => this.onMoveEventToResource(eventRecord, p),
                            disabled: this.isCloneEventAssignedToResource(eventRecord.data.id, p.id)
                        });
                    });
                    this.neighbours.forEach((n, index) => {
                        items.move.menu.push({
                            text: n.name,
                            icon: 'b-fa fas fa-arrow-right',
                            weight: 510,
                            onItem: ({ eventRecord }) => this.onMoveEventToResource(eventRecord, n),
                            disabled: this.isCloneEventAssignedToResource(eventRecord.data.id, n.id),
                            cls: index == 0 ? 'b-separator' : null,
                        });

                    });

                } else {
                    items.move.menu.push({
                        text: "Supprimer ce clone",
                        icon: 'b-fa fas fa-rotate-left',
                        weight: 510,
                        onItem: ({ eventRecord }) => this.onRemoveCloneAssignement(eventRecord, assignmentRecord.data.resourceId)
                    });
                }
            }
        }
    };
    neighbours: DisplayTask[] = [];
    parents: IBryntumRecordState[] = [];
    scheduler?: Scheduler;
    recordStateMap: { [key: string]: IBryntumRecordState } = {};
    clonesMap: { [key: string]: IBryntumEventClone } = {};
    delaysMap: { [key: string]: IBryntumDelay } = {};

    defaultEventFontSize: number = 16;

    constructor(
        protected eventManager: JhiEventManager,
        protected displayTaskService: DisplayTaskService,
        protected modalService: NgbModal,
        protected displayConfigBryntum: DisplayConfigBryntum
    ) { }


    //----------------------------------------------------------------------------------------------------------------
    //--                                              Move event                                                    --
    //----------------------------------------------------------------------------------------------------------------

    onMoveEventToResource(eventRecord, neighbour) {
        const record = this.recordStateMap[eventRecord.data.id];
        record.event.locationId = this.addResourceId(record.event.locationId, neighbour.id);

        const cloneEvent = this.getClone(record) ?? this.createClone(record);
        eventRecord.set('cloneId', cloneEvent.id)
        this.scheduler.assignmentStore.assignEventToResource(cloneEvent.id as any, neighbour.id);

        const delaysToMove = Object.values(this.delaysMap).filter(delay => delay.eventId === eventRecord.data.id);
        delaysToMove.forEach(delayToMove => this.scheduler.assignmentStore.assignEventToResource(delayToMove.id as any, neighbour.id));

        this.displayConfigBryntum.applyConfigRulesByEvent(this.scheduler.eventStore.getById(cloneEvent.id));
    }

    onRemoveCloneAssignement(clone: Model, currentResourceId: string) {
        const cloneParentRecord = this.recordStateMap[clone.getData('eventId')];
        this.scheduler.eventStore.unassignEventFromResource(clone.getData('id'), currentResourceId);

        cloneParentRecord.event.locationId = this.removeResourceId(cloneParentRecord.event?.locationId, currentResourceId);
        if (cloneParentRecord.event.locationId?.length == 0) {
            delete this.clonesMap[clone.getData('id')];
            const cloneParent = this.scheduler.eventStore.getById(cloneParentRecord.id);
            cloneParent.set('cloneId', null);
        }

        // If there are associated delays, remove them from the parent or the neighbour
        const delaysToMove = Object.values(this.delaysMap).filter(delay => delay.eventId === cloneParentRecord.id);
        delaysToMove.forEach(delayToMove => this.scheduler.assignmentStore.unassignEventFromResource(delayToMove.id, currentResourceId));
    }

    getClone(record: IBryntumRecordState) {
        return Object.values(this.clonesMap).find(clone => clone.eventId === record.id)?.event;
    }

    createClone(record: IBryntumRecordState) {
        const event = this.scheduler.eventStore.getById(record.id)
        const cloneId = Utils.createUUID();
        const cloneEvent = {
            ...record.event,
            id: cloneId,
            startDate: event.getData('startDate'),
            endDate: event.getData('endDate'),
            name: event.getData('name'),
            eventColor: event.getData('eventColor'),
            label: event.getData('label'),
            iconCls: event.getData('iconCls'),
            offsetMultiColorEventConfig: event.getData('offsetMultiColorEventConfig'),
            eventId: record.id
        };

        this.clonesMap[cloneEvent.id] = { id: cloneEvent.id, eventId: record.id, event: cloneEvent };
        this.scheduler.eventStore.add(cloneEvent);
        return cloneEvent;
    }

    addResourceId(currentLocationId: string[], newLocationId: string): string[] {
        let locationsId: string[] = [];
        if (currentLocationId) locationsId = locationsId.concat(currentLocationId);
        if (!locationsId?.includes(newLocationId)) locationsId.push(newLocationId)
        return locationsId;
    }

    removeResourceId(currentLocationId: string[], deleteResourceId: string): string[] {
        if (currentLocationId) {
            return currentLocationId = currentLocationId.filter(l => l !== deleteResourceId);
        } else return null;
    }

    isCloneEventAssignedToResource(eventRecordId, resourceId): boolean {
        const clone = Object.values(this.clonesMap).find(clone => clone.eventId === eventRecordId);
        return this.scheduler.assignmentStore.isEventAssignedToResource(clone?.id, resourceId);
    }

    getNeighbours(data: IDisplayTask) {
        if (data?.parentId) {
            const neighboursBryntum = this.scheduler.resourceStore?.getChildren(this.scheduler.resourceStore.getById(data.parentId));
            this.neighbours = neighboursBryntum?.map(n => JSON.parse(n.toString())).filter(n => n.id !== data.id);
        }
    }
    getParents(data: IBryntumRecordState) {
        this.parents = [];
        this.findParents(data);
        this.parents = this.parents.reverse();
    }

    findParents(data: IBryntumRecordState) {
        if (data?.resource?.parentId) {
            const parent = this.recordStateMap[data.resource.parentId]
            if (parent) {
                this.parents.push(parent);
                this.findParents(parent);
            }
        }
    }

    //----------------------------------------------------------------------------------------------------------------
    //--                                                Dependencies                                                --
    //----------------------------------------------------------------------------------------------------------------

    onHideOrDisplayDependencies(event: any): void {
        this.eventManager.broadcast({ name: 'hideOrDisplayDependencies', content: event });
    }

    //----------------------------------------------------------------------------------------------------------------
    //--                                            Display Configuration                                           --
    //----------------------------------------------------------------------------------------------------------------

    openDisplayConfiguration(event: any) {
        const modalRef = this.modalService.open(DisplayConfigurationDialogcomponent, { size: 'lg', backdrop: 'static' });
        modalRef.componentInstance.scheduler = this.scheduler;
        modalRef.componentInstance.eventRecord = event?.eventRecord;
        modalRef.componentInstance.fontSize = event?.eventRecord?.get('fontSize') ?? this.defaultEventFontSize;
        modalRef.componentInstance.defaulFontSize = this.defaultEventFontSize;
        modalRef.result.then((res) => {
            if (res) {
                const event = this.scheduler.eventStore.getById(res.idEvent);
                const properties: DisplayConfTaskProperties = res.properties;
                const currentDisplayConfProperties = DisplayConfigBryntum.getDisplayConfTaskPropertiesFromBcs(this.recordStateMap[res.idEvent]?.event.displayConfig);
                if (currentDisplayConfProperties) {
                    this.recordStateMap[res.idEvent].event.displayConfig.properties = this.displayConfigBryntum.updateDisplayConfTaskProperties(currentDisplayConfProperties, properties.bgColor, properties.shape, properties.position, properties.multiColorsEventConfigs)
                }
                else {
                    this.recordStateMap[res.idEvent].event.displayConfig = this.displayConfigBryntum.createNewDisplayConfiguration(undefined, 'overridedDisplayConfig', DisplayConfigurationType.TASK, JSON.stringify(properties));
                }
                event.set('displayConfig', this.recordStateMap[res.idEvent].event.displayConfig);

                this.displayConfigBryntum.applyConfigRulesByEvent(event);
            }
        });
    }

    openArrowDisplayConfiguration(event: any) {
        const modalRef = this.modalService.open(DisplayConfigurationArrowDialogcomponent, { size: 'lg', backdrop: 'static' });
        modalRef.componentInstance.scheduler = this.scheduler;
        modalRef.componentInstance.eventRecord = event?.eventRecord;
        modalRef.result.then((res) => {
            if (res) {
                const event = this.scheduler.eventStore.getById(res.idEvent);
                const delayArrowConf: DisplayConfTaskArrowProperties = res?.delayArrowConf;
                const delayRpsArrowConf: DisplayConfTaskArrowProperties = res?.delayRpsArrowConf;

                if (delayArrowConf || delayRpsArrowConf) {
                    if (delayArrowConf) {
                        const arrowAssociated = this.scheduler.eventStore.find(e => {
                            return e.getData('displayTask')?.id == res.idEvent &&
                                e.getData('cls') == 'b-sch-event-delay-arrow'
                        }, true)
                        if (arrowAssociated) arrowAssociated.set('delayArrowConf', delayArrowConf);
                    }
                    if (delayRpsArrowConf) {
                        const arrowAssociated = this.scheduler.eventStore.find(e => {
                            return e.getData('displayTask')?.id == res.idEvent &&
                                e.getData('cls') == 'b-sch-event-delay-arrow-rps'
                        }, true)
                        if (arrowAssociated) arrowAssociated.set('delayRpsArrowConf', delayRpsArrowConf);
                    }

                    const currentDisplayConfProperties = DisplayConfigBryntum.getDisplayConfTaskPropertiesFromBcs(this.recordStateMap[res.idEvent]?.event.displayConfig)
                    if (currentDisplayConfProperties) {
                        this.recordStateMap[res.idEvent].event.displayConfig.properties = this.displayConfigBryntum.updateArrowDisplayConfTaskProperties(currentDisplayConfProperties, delayArrowConf, delayRpsArrowConf);
                    } else {
                        const properties = { delayArrowConf: delayArrowConf, delayRpsArrowConf: delayRpsArrowConf }
                        this.recordStateMap[res.idEvent].event.displayConfig = this.displayConfigBryntum.createNewDisplayConfiguration(undefined, 'overridedDisplayConfig', DisplayConfigurationType.TASK, JSON.stringify(properties));
                    }
                    event.set('displayConfig', this.recordStateMap[res.idEvent].event.displayConfig);
                }

            }
        });
    }
}
