import { Injectable } from '@angular/core';
import { MessageDialog, Model, Scheduler } from '@bryntum/scheduler';
import { JhiEventManager } from 'ng-jhipster';
import { IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';

@Injectable()
export class ReorderConfig {
    scheduler: Scheduler;
    recordStateMap: { [key: string]: IBryntumRecordState } = {};

    constructor(protected eventManager: JhiEventManager) {}

    rowReorder = {
        dropOnLeaf: true,
        listeners: {
            gridRowBeforeDropFinalize: async ({ context }) => {
                const independantMilestone = context?.parent?.data?.independantMilestone;
                if (independantMilestone) {
                    return false;
                }
                const result = await MessageDialog.confirm({
                    title: 'Confirmation',
                    message: 'Etes-vous sûr de vouloir déplacer cette tâche ?'
                });

                // Return true to accept the drop or false to reject it
                return result === MessageDialog.okButton;
            }, gridRowDrop: (event: any) => {
                this.saveParentIndex(event);
                this.eventManager.broadcast('resourceMoved');
            }
        }
    }

    saveParentIndex(event: any) {
        this.scheduler.resourceStore.traverse((n: Model) => {
            this.recordStateMap[n.getData('id')].resource.parentIndex = n.parentIndex;
            if (n.getData('parentId') !== this.recordStateMap[n.getData('id')].resource.parentId) {
                this.recordStateMap[n.getData('id')].resource.parentId = n.getData('parentId');
            }
        });
    }

}