import { EventModel, Model, Scheduler } from '@bryntum/scheduler';
import { IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { Utils } from 'src/app/bcs_ng/utils/util';

export class SchedulerStoresHandler {

    scheduler: Scheduler;
    initialRecordStateMap: { [key: string]: IBryntumRecordState } = {};
    constructor() { }

    /** Make adjustments in the scheduler after it was fed data */
    setStoresData() {
        this.scheduler.resourceStore.removeFilter('displayed');

        this.scheduler.resourceStore.traverse((resource: Model) => {
            const typeId = resource.getData('typeId');

            // Set event dates
            if (!TaskableEntityTypeService.isMilestone(typeId) && !TaskableEntityTypeService.isGenericTask(typeId)) {
                const startDates = [], endDates = [];
                resource.allChildren?.forEach(child => {
                    const events = this.scheduler.assignmentStore.getEventsForResource(child.id);
                    if (events && events.length > 0) {
                        if (Utils.isDateValid(events[0]?.getData('startDate'))) startDates.push(events[0]?.getData('startDate'));
                        if (Utils.isDateValid(events[0]?.getData('endDate'))) endDates.push(events[0]?.getData('endDate'));
                    }
                });

                if (!TaskableEntityTypeService.isMilestone(typeId) && startDates.length > 0 && endDates.length > 0) {
                    const min = startDates?.reduce(function (a, b) { return a < b ? a : b });
                    const max = endDates?.reduce(function (a, b) { return a > b ? a : b });
                    let name;
                    if (TaskableEntityTypeService.isTaskFromBmcs(typeId))
                        name = resource.getData('name') ?? resource.getData('object')?.name;
                    else
                        name = resource.getData('shortName') ?? resource.getData('object')?.shortName;
                    const id = resource.getData('id');
                    const eventToUpdate = this.scheduler.eventStore.getById(id);
                    if (eventToUpdate) {
                        eventToUpdate.set('startDate', new Date(min));
                        eventToUpdate.set('endDate', new Date(max));
                    }
                }
            }
        });

        this.scheduler.eventStore.traverse((event: Model) => {
            if (event.getData('eventId')) {
                const parentCloneEvent = this.scheduler.eventStore.getById(event.getData('eventId'));
                const startDate = new Date(new Date(parentCloneEvent.getData('startDate')).toDateString());
                const endDate = new Date(new Date(parentCloneEvent.getData('endDate')).toDateString());
                event.set('startDate', startDate);
                event.set('endDate', endDate);
            } else {
                const startDate = new Date(new Date(event.getData('startDate')).toDateString());
                const endDate = new Date(new Date(event.getData('endDate')).toDateString());
                event.set('startDate', startDate);
                event.set('endDate', endDate);
            }
        });

        this.scheduler.resourceStore.filter({ id: 'displayed', property: 'displayed', value: true });
    }


    static getChildEventByEvent(scheduler: Scheduler, event: Model): Model[] {
        let childrenEvents: Model[] = [];
        const resourceAssigned = scheduler.assignmentStore.getResourcesForEvent(event as EventModel)[0];
        resourceAssigned.allChildren.forEach(c => {
            childrenEvents.push(scheduler.assignmentStore.getEventsForResource(c.id)[0] as Model);
        });
        return childrenEvents;
    }
}