import { OnInit } from '@angular/core';
import { Component } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { StatistiqueService } from 'src/app/admin/statistique/statistique.service';
import { StorageService } from 'src/app/blocks/storage/storage-service';
import { BmcsBaseComponent } from 'src/app/bmcs/shared/bmcs-base-component';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { AccountService } from 'src/app/core/auth/account.service';
import { routePath } from 'src/app/routes/path-routing.constant';

@Component({
  selector: 'jhi-road-map-bcs-detail',
  templateUrl: './road-map-bcs-detail.component.html',
  styleUrls: ['./bryntum/bryntum.scss'],
})
export class RoadMapBcsDetailComponent extends BmcsBaseComponent implements OnInit {
  id: string;
  mode: string = '';

  onLoad: boolean = false;

  constructor(
    private route: ActivatedRoute,
    protected router: Router,
    private storageService: StorageService,
    protected accountService: AccountService,
    protected bmcsBaseService: BmcsBaseService,
    protected statistiqueService: StatistiqueService,
  ) { super(accountService, statistiqueService, bmcsBaseService) }

  async ngOnInit(): Promise<void> {
    this.onLoad = true;
    await super.ngOnInit();
    this.setData();
    this.onLoad = false;
  }

  private setData(): void {
    this.id = this.route.snapshot.paramMap.get('id');
    const storageParams = this.storageService.getParamsMap();
    const mode = storageParams?.get('mode') ?? 'view';
    this.mode = mode;
  }

  switchMode(): void {
    if (this.mode === 'view') {
      this.router.navigate([routePath.BCS_ROADMAP + '/' + this.id], { queryParams: { 'mode': 'edit' }, queryParamsHandling: "preserve" });
      this.mode = 'edit';
    } else {
      this.router.navigate([routePath.BCS_ROADMAP + '/' + this.id], { queryParams: { 'mode': 'view' }, queryParamsHandling: "preserve" });
      this.mode = 'view';
    }
  }

  navigateToCloneView(cloneId: string): void {
    this.router.navigate([routePath.BCS_ROADMAP + '/' + cloneId], { queryParams: { 'mode': 'view' } });
    this.mode = 'view';
    this.id = cloneId;
  }
}