import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Subscription } from 'rxjs';
import { JhiEventManager } from 'ng-jhipster';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { RoadMapBcsService } from './road-map.service';
import { DomaineType, IDomaine } from 'src/app/shared/model/entity/domaine.model';
import { DomaineService } from 'src/app/entities/domaine/domaine.service';
import { DialogDeleteComponent } from 'src/app/shared/dialog/delete/dialog-delete.component';
import { DialogRebornComponent } from 'src/app/shared/dialog/reborn/dialog-reborn.component';
import { DialogEraseComponent } from 'src/app/shared/dialog/erase/dialog-erase.component';
import { IRoadMapBcs, RoadMapBcs } from '../../model/road-map-bcs.model';
import { RoadMapFieldsDialogComponent } from './update/road-map-fields/road-map-fields-dialog.component';
import { Router } from '@angular/router';
import { CruddomaineService } from 'src/app/entities/group-users/cruddomaine.service';
import { AccountService } from 'src/app/core/auth/account.service';
import { ICruddomaine } from 'src/app/shared/model/entity/cruddomaine.model';
import { IRoadMapCrud } from '../../shared/dto/model/roadMap-crud.dto';
import { RoadMapBcsDTO } from '../../shared/dto/roadMapBcs.dto';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { Baseline } from '../../model/enumeration/baseline.enum';
import { RoadMapBcsDomainHistoricComponent } from './view/domain-history/road-map-bcs-domain-historic.component';
import { StorageService, StoreKey } from 'src/app/blocks/storage/storage-service';
import { routePath } from 'src/app/routes/path-routing.constant';
import { StatistiqueService } from 'src/app/admin/statistique/statistique.service';
import { BmcsBaseComponent } from 'src/app/bmcs/shared/bmcs-base-component';

@Component({
  selector: 'jhi-road-map',
  templateUrl: './road-map.component.html',
  styleUrls: ['../bcs-ng.scss']
})
export class RoadMapBcsComponent extends BmcsBaseComponent implements OnInit {
  roadMaps?: IRoadMapBcs[];
  roadMapsCrud?: IRoadMapCrud[];
  domaines: IDomaine[] = [];
  domainesWithRoadMap: Map<IDomaine, IRoadMapCrud[]> = new Map<IDomaine, IRoadMapCrud[]>();
  specialitesWithRoadMap: Map<IDomaine, IRoadMapCrud[]> = new Map<IDomaine, IRoadMapCrud[]>();

  crudDomaines: [ICruddomaine[]];
  onLoad: boolean = false;

  eventSubscriber?: Subscription;

  baseline: Baseline;
  Baseline = Baseline;

  constructor(
    protected roadMapService: RoadMapBcsService,
    protected domaineService: DomaineService,
    protected crudDomaineService: CruddomaineService,
    protected eventManager: JhiEventManager,
    protected modalService: NgbModal,
    protected router: Router,
    protected accountService: AccountService,
    protected roadMapBcsDTO: RoadMapBcsDTO,
    protected bmcsBaseService: BmcsBaseService,
    protected statistiqueService: StatistiqueService,
    protected storageService: StorageService
  ) {
    super(accountService, statistiqueService, bmcsBaseService);
  }

  async ngOnInit() {
    this.onLoad = true;
    await super.ngOnInit();
    const bl = this.storageService.get(StoreKey.BCS_BASELINE) as Baseline;
    if (bl) this.baseline = bl;
    else this.baseline = this.isUserMoss ? Baseline.NONE : Baseline.VISIBLE_CLIENT;
    this.loadAll();
    this.registerChangeInRoadMap();
  }

  async loadAll() {
    await this.loadDomaines();
    this.domaines?.forEach(d => {
      d.type == DomaineType.DOMAINE ? this.domainesWithRoadMap.set(d, []) : this.specialitesWithRoadMap.set(d, []);
    })
    this.loadRoadMaps();
  }

  async loadRoadMaps() {
    this.storageService.set(StoreKey.BCS_BASELINE, this.baseline);
    this.roadMapService.query(undefined, this.baseline).subscribe(async (res: HttpResponse<IRoadMapBcs[]>) => {
      this.roadMaps = res.body || [];
      this.crudDomaines = await this.loadCrudDomaine();
      this.fillRoadMapMaps();
      this.onLoad = false;
    });
  }

  async loadCrudDomaine(): Promise<[ICruddomaine[]]> {
    return new Promise(resolve => {
      this.crudDomaineService.queryMyne({ where: this.accountService.userGroup }).subscribe(
        (res: HttpResponse<[ICruddomaine[]]>) => {
          resolve(res.body || [[]]);
        });
    });
  }

  loadDomaines() {
    this.domaines = this.getDomaines();
  }

  fillRoadMapMaps(): void {
    this.roadMapsCrud = this.roadMapBcsDTO.convertToCrud(this.roadMaps, this.domaines, this.crudDomaines);
    const newDomainesWithRoadMap = new Map<IDomaine, IRoadMapCrud[]>();
    const newSpecialitesWithRoadMaps = new Map<IDomaine, IRoadMapCrud[]>();
    this.domaines?.forEach(d => {
      d.type == DomaineType.DOMAINE ? newDomainesWithRoadMap.set(d, []) : newSpecialitesWithRoadMaps.set(d, []);
    })
    this.roadMapsCrud.forEach(rm => {
      rm.domaines?.forEach(domaine => {
        if (domaine?.type === DomaineType.DOMAINE) {
          const roadMaps = newDomainesWithRoadMap.get(domaine);
          roadMaps?.push(rm);
          newDomainesWithRoadMap.set(domaine, roadMaps);
        } else {
          const roadMaps = newSpecialitesWithRoadMaps.get(domaine);
          roadMaps?.push(rm);
          newSpecialitesWithRoadMaps.set(domaine, roadMaps);
        }
      });
    });
    this.domainesWithRoadMap = newDomainesWithRoadMap;
    this.specialitesWithRoadMap = newSpecialitesWithRoadMaps;
  }

  registerChangeInRoadMap(): void {
    this.eventSubscriber = this.eventManager.subscribe('roadMapBcsListModification', () => {
      this.domainesWithRoadMap.clear();
      this.specialitesWithRoadMap.clear();
      this.loadRoadMaps();
    });
  }

  defineParamsDialog(modalRef, id) {
    modalRef.componentInstance.id = id;
    modalRef.componentInstance.entityMsg = "cette feuille de route";
    modalRef.componentInstance.serviceEntity = this.roadMapService;
    modalRef.componentInstance.eventManagerName = "roadMapBcsListModification";
  }

  delete(roadMap: IRoadMapBcs): void {
    const modalRef = this.modalService.open(DialogDeleteComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, roadMap.id);
  }

  reborn(roadMap: IRoadMapBcs): void {
    const modalRef = this.modalService.open(DialogRebornComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, roadMap.id);
  }

  erase(roadMap: IRoadMapBcs): void {
    const modalRef = this.modalService.open(DialogEraseComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, roadMap.id);
  }

  navigateToRoadMapDetail(id: string, mode: string): void {
    this.router.navigate([routePath.BCS_ROADMAP + '/' + id], { queryParams: { mode } });
  }

  openRoadMapModal(): void {
    const modalRef = this.modalService.open(RoadMapFieldsDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.domains = this.domaines;
    modalRef.result.then((res: { title: string, domains: IDomaine[] }) => {
      this.saveAndNavigate(res.title, res.domains);
    }).catch((err) => {
      if (err) console.error(err);
    });
  }

  private saveAndNavigate(title: string, domains: IDomaine[]): void {
    const roadMap = new RoadMapBcs();
    roadMap.name = title;
    roadMap.domainesId = JSON.stringify(domains.map(d => d.id));
    this.roadMapService.create(roadMap).subscribe(res => {
      this.navigateToRoadMapDetail(res.body.id, 'edit');
    });
  }

  updateDomainHistorics(domaine: IDomaine, newBaselineValue: Baseline, specialty: boolean): void {
    let roadMaps: IRoadMapBcs[] = [];
    if (specialty) roadMaps = this.specialitesWithRoadMap.get(domaine);
    else roadMaps = this.domainesWithRoadMap.get(domaine);
    const modalRef = this.modalService.open(RoadMapBcsDomainHistoricComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.domaine = domaine;
    modalRef.componentInstance.roadMaps = roadMaps;
    modalRef.componentInstance.newBaselineValue = newBaselineValue;
    modalRef.result.then(() => {
      window.location.reload();
    }, err => {
      if (err) console.error(err);
    });
  }

  isAllowToUpdate(crud: ICruddomaine[]) { return crud?.some(crudDomaine => crudDomaine?.update); }

  isAllowToDeleteOrReborn(crud: ICruddomaine[]) { return crud?.some(crudDomaine => crudDomaine?.deleted); }

  isAllowToErase(crud: ICruddomaine[]) { return crud?.some(crudDomaine => crudDomaine?.erase); }

  isUserReferentMoss(domaine: IDomaine) {
    return this.isAdminBCS || this.crudDomaines?.some(crud => crud.some(cd => cd.domaine.id === domaine.id));
  }
}
