import { AfterContentChecked, Component, OnInit } from '@angular/core';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';

import { Utils } from 'src/app/bcs_ng/utils/util';
import { ITask } from 'src/app/bcs_ng/model/task/Itask.model';
import { TaskableEntity } from 'src/app/bcs_ng/model/taskable-entity.model';
import { TaskableEntityService } from '../../../taskable-entity/taskable-entity.service';
import { OperationService } from '../../../table/operation/operation.service';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { BryntumRecordState, IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';
import { IDependency } from 'src/app/bcs_ng/model/dependency.model';
import { IOperation, Operation } from 'src/app/bcs_ng/model/operation.model';
import { VersionedSystemService } from 'src/app/bmcs/entity/versioned-system/versioned-system.service';
import { IVersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { HttpResponse } from '@angular/common/http';
import { IDeployedCenter } from 'src/app/bmcs/model/deployed-center.model';
import { DeployedCenterService } from 'src/app/bmcs/entity/deployed-center/deployed-center.service';
import { IDomaine } from 'src/app/shared/model/entity/domaine.model';
import { AccountService } from 'src/app/core/auth/account.service';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { ChangeDetectorRef } from '@angular/core';
import { BaseComponent } from 'src/app/shared/base.component';

@Component({
  selector: 'jhi-add-taskable-entity',
  templateUrl: './add-taskable-entity.component.html',
  styleUrls: ['../../bryntum/bryntum.scss']
})
export class AddTaskableEntityComponent extends BaseComponent implements OnInit, AfterContentChecked {
  taskableEntities: TaskableEntity[] = [];
  taskableEntitySelected: TaskableEntity;

  tasks: ITask[];

  records: IBryntumRecordState[] = [];
  dependencies: IDependency[] = [];

  displayColumnsGenericTask: string[] = ['select', 'name', 'shortName', 'startDate', 'endDate'];
  displayColumnsOperation: string[] = ['select', 'manual_id', 'versionedSystem', 'deployedCenter', 'name', 'startDate', 'endDate'];
  displayColumnsMilestone: string[] = ['select', 'versionedSystem', 'deployedCenter', 'operation', 'name', 'shortName', 'date'];

  operations: IOperation[] = [];
  versionedSystems: IVersionedSystem[] = [];
  deployedCenters: IDeployedCenter[] = [];
  domains: IDomaine[] = [];

  onLoad: boolean = false;

  constructor(
    protected accountService: AccountService,
    protected bmcsBaseService: BmcsBaseService,
    public activeModal: NgbActiveModal,
    public taskableEntityService: TaskableEntityService,
    public operationService: OperationService,
    public versionedSystemService: VersionedSystemService,
    public deployedCenterService: DeployedCenterService,
    private cdRef: ChangeDetectorRef,
  ) { super(accountService) }

  ngOnInit(): void {
    this.cdRef.detach();
    this.taskableEntityService.query().subscribe(res => {
      this.taskableEntities = res.body || [];
    });
    this.loadSVandCD();
  }

  ngAfterContentChecked(): void {
    this.cdRef.detectChanges();
  }

  /** -------------------- Init --------------------  */
  loadSVandCD(): void {
    // this.onLoad = true;
    this.versionedSystems = this.bmcsBaseService.getVersionedSystems;
    this.deployedCenters = this.bmcsBaseService.getDeployedCenters;

    this.operationService.query().subscribe((res: HttpResponse<IOperation[]>) => {
      this.operations = res.body || [];
      this.operations.forEach(o => {
        o.versionedSystem = this.bmcsBaseService.findVersionedSystem(o.versionedSystemId);
        o.deployedCenter = this.bmcsBaseService.findDeployedCenter(o.deployedCenterId);
      });
      this.onLoad = false;
    }, (err) => {
      this.onLoad = false;
      console.error(err);
    });
  }

  /** -------------------- Modal actions --------------------  */
  changeTaskableEntity() {
    this.tasks = [];
  }

  async onChangeSelectItem(selected: ITask[]) {
    this.tasks = selected;
  }

  async emitNewElement() {
    await this.buildRecordStates();
    this.activeModal.close({
      records: this.records,
      dependencies: this.dependencies,
    });
  }

  cancel(): void {
    this.activeModal.dismiss();
  }

  /** -------------------- Record states --------------------  */
  buildRecordState(id: string, taskObject: ITask, parentId: string, typeId: string): IBryntumRecordState {
    const resource = {
      id,
      name: null,
      hidden: false,
      deleted: false,
      displayed: true,
      typeId,
      parentId,
      parentIndex: undefined,
      readOnly: false,
      createdBy: null,
      object: taskObject,
      rpsObjects: [],
      expanded: false,
      cls: '',
      independantMilestone: false,
      fontSize: null,
      rowHeight: null,
      fontColor: null,
    };

    let cls = '';
    if (TaskableEntityTypeService.isOperation(this.taskableEntitySelected.id)) cls = 'operation-event';
    if (TaskableEntityTypeService.isMilestone(this.taskableEntitySelected.id)) cls = 'milestone-event';

    const event = {
      id,
      name: null,
      locationId: null,
      startDate: undefined,
      endDate: undefined,
      readOnly: true,
      parentId,
      object: taskObject,
      type: this.taskableEntities.find(t => t.id === typeId),
      hidePredecessors: false,
      cls,
      delayEvent: false,
    };
    const recordState = new BryntumRecordState(id, false, false, resource, event);
    return recordState;
  }

  buildRecordStates() {
    for (const t of this.tasks) {
      const id = Utils.createUUID();
      if (this.isDomaine(this.taskableEntitySelected.id)) {
        this.createDomaineRecordState(t, id, null);
      }
      else if (this.isSystem(this.taskableEntitySelected.id)) {
        this.createSystemRecordState(t, id, null);
      }
      else if (this.isVersionedSystem(this.taskableEntitySelected.id)) {
        let sv = this.getSV(t.id);
        if (sv) this.createVersionedSystemRecordState(sv, id, null);
      }
      else if (this.isDeployedCenter(this.taskableEntitySelected.id)) {
        this.createDeployedCenterRecordStateMap(t, id, null);
      }
      else if (this.isOperation(this.taskableEntitySelected.id)) {
        this.createOperationRecordState(t, id, null);
      }
      else if (this.isMilestone(this.taskableEntitySelected.id)) {
        this.createMilestonesRecordState(t, id, null);
      } else {
        this.createGenericTaskRecordState(t, id, null);
      }
    }
  }

  private createDomaineRecordState(domaine: IDomaine, id: string, idParent: string) {
    const recordState = this.buildRecordState(id, domaine, idParent, TaskableEntityTypeService.getIdDomaine());
    this.records.push(recordState);
    for (const sys of domaine.bmcsSystems) {
      const idSys = Utils.createUUID();
      this.createSystemRecordState(sys, idSys, id);
    }
  }

  private createSystemRecordState(system: ITask, id: string, idParent: string) {
    const recordState = this.buildRecordState(id, system, idParent, TaskableEntityTypeService.getIdSystem());
    this.records.push(recordState);
    for (const sv of system.versionedSystems) {
      const idSV = Utils.createUUID();
      this.createVersionedSystemRecordState(sv, idSV, id);
    }
  }

  private createVersionedSystemRecordState(sv: IVersionedSystem, id: string, idParent: string) {
    let temp_DiplayTaskTaskId_CdId: Map<string, string> = new Map<string, string>();
    let operations: IOperation[] = this.getOperationsByVS(sv.id.toLowerCase());
    if (operations.length > 0) {
      const recordState = this.buildRecordState(id, sv, idParent, TaskableEntityTypeService.getIdVersionedSystem());
      this.records.push(recordState);
      for (const o of operations) {
        let idCD = null;
        if (o.deployedCenterId) {
          if (!temp_DiplayTaskTaskId_CdId.has(o.deployedCenterId)) {
            idCD = Utils.createUUID();
            let cd = this.getCD(o.deployedCenterId);
            this.createDeployedCenterRecordStateMap(cd, idCD, id, true);
            temp_DiplayTaskTaskId_CdId.set(o.deployedCenterId, idCD);
          } else {
            idCD = temp_DiplayTaskTaskId_CdId.get(o.deployedCenterId);
          }
        }
        const idOperation = Utils.createUUID();
        this.createOperationRecordState(o, idOperation, idCD ?? id);
      }
    }
  }

  private createDeployedCenterRecordStateMap(cd: IDeployedCenter, id: string, idParent: string, isFromSV: boolean = false) {
    const recordState = this.buildRecordState(id, cd, idParent, TaskableEntityTypeService.getIdDeployedCenter());
    this.records.push(recordState);
    if (!isFromSV) {
      let operations: IOperation[] = this.getOperationsByDC(cd.id.toLowerCase());
      for (const o of operations) {
        const idOperation = Utils.createUUID();
        this.createOperationRecordState(o, idOperation, id);
      }
    }
  }

  private createOperationRecordState(t: ITask, idOperation: string, idParent: string): Operation {
    const ope = new Operation();
    ope.milestones = t.milestones;
    const recordState = this.buildRecordState(idOperation, t, idParent, TaskableEntityTypeService.getIdOperation());
    this.records.push(recordState);
    for (const child of t?.milestones) {
      if (!child.deleted) {
        const idMilestone = Utils.createUUID();
        this.createMilestonesRecordState(child, idMilestone, idOperation);
      }
    }
    return ope;
  }

  private createMilestonesRecordState(t: ITask, id: string, idParent: string) {
    const recordState = this.buildRecordState(id, t, idParent, TaskableEntityTypeService.getIdMilestone());
    this.records.push(recordState);
    t.predecessorDependencies.forEach(dep => this.dependencies.push(dep));
    t.successorDependencies.forEach(dep => this.dependencies.push(dep));
  }

  private createGenericTaskRecordState(t: ITask, id: string, idParent: string) {
    const recordState = this.buildRecordState(id, t, idParent, TaskableEntityTypeService.getIdGenericTask());
    this.records.push(recordState);
  }

  /** -------------------- Utils --------------------  */
  private getOperationsByVS(idVersionedSystem: string): IOperation[] {
    return this.operations.filter(o => o.versionedSystemId === idVersionedSystem && !o.deleted);
  }

  private getOperationsByDC(idDeployedCenter: string): IOperation[] {
    return this.operations.filter(o => o.deployedCenterId === idDeployedCenter && !o.deleted);
  }

  getSV(id: string): IVersionedSystem {
    return this.versionedSystems.find(sv => sv.id === id.toUpperCase());
  }

  getCD(id: string): IDeployedCenter {
    return this.deployedCenters.find(cd => cd.id === id.toUpperCase());
  }

  isMilestone(id: string) {
    return TaskableEntityTypeService.isMilestone(id);
  }

  isOperation(id: string) {
    return TaskableEntityTypeService.isOperation(id);
  }

  isGenericTask(id: string) {
    return TaskableEntityTypeService.isGenericTask(id);
  }

  isDeployedVersionedSystem(id: string) {
    return TaskableEntityTypeService.isDeployedVersionedSystem(id);
  }

  isVersionedSystem(id: string) {
    return TaskableEntityTypeService.isVersionedSystem(id);
  }

  isSystem(id: string) {
    return TaskableEntityTypeService.isSystem(id);
  }

  isDeployedCenter(id: string) {
    return TaskableEntityTypeService.isDeployedCenter(id);
  }

  isDomaine(id: string) {
    return TaskableEntityTypeService.isDomaine(id);
  }
}
