import { Component, OnInit } from '@angular/core';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';
import { IBryntumResource } from 'src/app/bcs_ng/model/bryntum/bryntum-resource.model';
import { SortData } from '../../bryntum/sort-data';

@Component({
  templateUrl: './road-map-deleted-elements-dialog.component.html',
  styleUrls: ['../../bryntum/bryntum.scss']
})
export class RoadMapDeletedElementsDialogComponent implements OnInit {
  deletedRecords: IBryntumRecordState[] = [];
  rootElements: IBryntumResource[] = [];
  deletedElementsMap: { [key: string]: IBryntumResource[] } = {};
  toReincarnate: { [key: string]: boolean } = {};
  toErase: { [key: string]: boolean } = {};
  initialToErase: { [key: string]: boolean } = {};

  deletedEntitiesInfo = 'Vous ne pouvez pas agir sur cet élément car son entité sous-jacente a été supprimée. ' +
    'Un administrateur peut la restaurer.';

  constructor(public activeModal: NgbActiveModal,) { }

  ngOnInit(): void {
    const deletedElements = this.deletedRecords.map(record => {
      return {
        ...record.resource,
        name: record.resource.name ?? record.resource.object.name,
      }
    });

    // Elements without parent or whose parent has not been deleted
    const rootElements = deletedElements.filter(element => {
      return !element.parentId || !deletedElements.some(potentialParent => potentialParent.id === element.parentId);
    });
    rootElements.forEach(element => {
      this.deletedElementsMap[element.id] = deletedElements.filter(potentialChild => potentialChild.parentId === element.id);
    });

    this.toReincarnate = rootElements.reduce((prev, curr) => {
      prev[curr.id] = false;
      return prev;
    }, {});
    this.toErase = this.deletedRecords.reduce((prev, curr) => {
      prev[curr.id] = curr.toErase;
      return prev;
    }, {});
    this.initialToErase = this.deletedRecords.reduce((prev, curr) => {
      prev[curr.id] = curr.toErase;
      return prev;
    }, {});
    this.rootElements = rootElements;
  }

  onChangeCheckboxValue(id: string, event: any): void {
    this.toReincarnate[id] = event.checked;
    if (event.checked) this.toErase[id] = false;
  }

  onChangeEraseCheckboxValue(id: string, event: any): void {
    this.toErase[id] = event.checked;
    if (event.checked) this.toReincarnate[id] = false;
  }

  save(): void {
    const toRedisplayList = this.deletedRecords.filter(e => this.toReincarnate[e.id]).map(record => record.resource);
    const orderedToRedisplayList = SortData.sortData(toRedisplayList);
    const toEraseList = this.deletedRecords.filter(e => this.toErase[e.id] !== this.initialToErase[e.id]).map(record => record.resource);
    this.activeModal.close({
      toRedisplay: orderedToRedisplayList,
      toErase: toEraseList,
    });
  }

  cancel(): void {
    this.activeModal.dismiss();
  }

  getResourceName(resource: IBryntumResource): string {
    return resource.name?.length ? resource.name : resource.object?.name;
  }

  getResourceChildNb(resource: IBryntumResource): string {
    return resource.name?.length ? resource.name : resource.object?.name;
  }
}