import { Component } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { IBryntumOverridableData } from 'src/app/bcs_ng/model/bryntum/bryntum-overridable-data.model';
import { DisplayTask, IDisplayTask } from 'src/app/bcs_ng/model/display-task.model';
import { TimeUnit, TimeUnitLabels } from 'src/app/bcs_ng/model/enumeration/time-unit.enum';
import { IDependency } from 'src/app/bcs_ng/model/dependency.model';
import { DependencyService } from '../../../dependency/dependency.service';
import { DisplayTaskService } from '../../../display-task/display-task.service';

@Component({
  templateUrl: './task-dependency-update.component.html'
})
export class TaskDependencyUpdateComponent {
  predecessorId: string = '';
  successorId: string = '';
  predecessor: IDisplayTask;
  successor: IDisplayTask;
  predecessorName: string = '';
  successorName: string = '';

  dependency: IDependency;

  editForm = this.fb.group({
    timeGapDuration: [0, [Validators.pattern("^[0-9]{1,3}$")]],
    timeGapUnit: [TimeUnit.MONTHS]
  });

  warningMessage = 'Attention, la valeur de délai est spécifique à cette feuille de route. ' +
    'Il s\'agit de la valeur des données de la base calendaire. ' +
    'La modifier impactera toutes les feuilles de route contenant une dépendance entre ces deux mêmes jalons.';

  timeUnitLabels = TimeUnitLabels;
  timeUnitValues = Object.keys(this.timeUnitLabels);

  constructor(
    public activeModal: NgbActiveModal,
    protected displayTaskService: DisplayTaskService,
    protected dependecyService: DependencyService,
    private fb: FormBuilder,
  ) {}

  ngOnInit(): void {
    if (this.predecessorId && this.successorId) this.fetchData();
  }

  fetchData(): void {
    // Fetch predecessor
    this.displayTaskService.find(this.predecessorId).subscribe(res1 => {
      this.predecessor = res1.body;
      const overridden = this.getOverridenData(this.predecessor);
      this.predecessorName = overridden.name;

      // Fetch successor
      this.displayTaskService.find(this.successorId).subscribe(res2 => {
        this.successor = res2.body;
        const overridden = this.getOverridenData(this.successor);
        this.successorName = overridden.name;

        // Fetch dependency
        this.dependecyService.query(
          undefined,
          {
            predecessor: this.predecessor.object.id,
            successsor: this.successor.object.id
          }
        )
        .subscribe(res3 => {
          if (res3.body?.length) this.dependency = res3.body[0];
          this.updateForm(this.dependency);
        });
      });
    });
  }

  updateForm(dependency: IDependency): void {
    this.editForm.patchValue({
      timeGapDuration: dependency.timeGapDuration,
      timeGapunit: dependency.timeGapUnit,
    });
  }

  save(): void {
    const timeGapDuration = this.editForm.get('timeGapDuration').value;
    const timeGapUnit = this.editForm.get('timeGapUnit').value;
    this.activeModal.close({id: this.dependency.id, timeGapDuration: Number(timeGapDuration), timeGapUnit});
  }

  cancel(): void {
    this.activeModal.dismiss();
  }

  private getOverridenData(displayTask: IDisplayTask): IBryntumOverridableData {
    const emptyDisplayTask = new DisplayTask();
      emptyDisplayTask.name = displayTask.name;
      emptyDisplayTask.object = displayTask.object;
      emptyDisplayTask.type = displayTask.type;
      return emptyDisplayTask.getOverridableData;
  }
}