import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Model, Scheduler } from '@bryntum/scheduler';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { DisplayConfTaskArrowProperties } from 'src/app/bcs_ng/model/bryntum/display-configuration/display-conf-task-arrow-properties';
import { DisplayConfigBryntum } from '../../bryntum/display-config';
import { SchedulerStoresHandler } from '../../bryntum/scheduler-stores';

@Component({
  templateUrl: './display-configuration-arrow-dialog.component.html',
  styleUrls: ['../../bryntum/bryntum.scss', './display-configuration-arrow-dialog.scss', '../display-configuration-rules/display-configuration-rules-dialog.scss'],
  providers: [SchedulerStoresHandler]
})
export class DisplayConfigurationArrowDialogcomponent implements OnInit {

  scheduler: Scheduler;
  eventRecord: Model;
  childEvents: Model[];
  delayArrowsettingForm = this.fb.group({
    bgColor: [null],
    hidden: [false],
    position: []
  });
  delayRpsArrowsettingForm = this.fb.group({
    bgColor: [null],
    hidden: [false],
    position: [null]
  });

  constructor(
    public activeModal: NgbActiveModal,
    private fb: FormBuilder,
    protected schedulerStoresHandler: SchedulerStoresHandler
  ) { }

  // ----------------------------------- Component lifecycle -----------------------------------
  ngOnInit(): void {
    let properties = DisplayConfigBryntum.getCurrentDisplayConfTaskProperties(this.eventRecord);
    if (properties?.delayArrowConf) this.updateForm(this.delayArrowsettingForm, properties?.delayArrowConf);
    if (properties?.delayRpsArrowConf) this.updateForm(this.delayRpsArrowsettingForm, properties?.delayRpsArrowConf);
  }

  cancel(): void {
    this.activeModal.close(null);
  }

  delayArrowsettingFromForm(): DisplayConfTaskArrowProperties {
    return {
      ... new DisplayConfTaskArrowProperties(),
      hidden: this.delayArrowsettingForm.controls['hidden']?.value,
      bgColor: this.delayArrowsettingForm.controls['bgColor']?.value,
      position: this.delayArrowsettingForm.controls['position']?.value,
    }
  }
  delayRpsArrowsettingFromForm(): DisplayConfTaskArrowProperties {
    return {
      ... new DisplayConfTaskArrowProperties(),
      hidden: this.delayRpsArrowsettingForm.controls['hidden']?.value,
      bgColor: this.delayRpsArrowsettingForm.controls['bgColor']?.value,
      position: this.delayRpsArrowsettingForm.controls['position']?.value,
    }
  }

  save(): void {
    this.activeModal.close({
      idEvent: this.eventRecord.getData('id'),
      delayArrowConf: this.delayArrowsettingFromForm(),
      delayRpsArrowConf: this.delayRpsArrowsettingFromForm()
    });
  }


  updateForm(form: FormGroup, properties: DisplayConfTaskArrowProperties) {
    form.controls['bgColor'].setValue(properties?.bgColor);
    form.controls['hidden'].setValue(properties?.hidden);
    form.controls['position'].setValue(properties?.position);
  }

  setDefaultConfigColor() {
    this.delayArrowsettingForm.controls['bgColor'].setValue(null);
  }

}