import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { Scheduler } from '@bryntum/scheduler';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { BcsNgListOptionService } from 'src/app/bcs_ng/entity/bcs-ng-list-option/bcs-ng-list-option.service';
import { IBcsNgListOption } from 'src/app/bcs_ng/model/bcs-ng-list-option.model';
import { DisplayRule } from 'src/app/bcs_ng/model/display-rule.model';
import { TaskableEntity } from 'src/app/bcs_ng/model/taskable-entity.model';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { Utils } from 'src/app/bcs_ng/utils/util';
import { MultiColorsEventConfigRule } from '../../../../../model/bryntum/display-configuration/multi-colors-event-config';

@Component({
  selector: 'jhi-display-configuration-card',
  templateUrl: './display-configuration-rules-card.component.html',
  styleUrls: ['../../../bryntum/bryntum.scss', '../display-configuration-rules-dialog.scss'],
})
export class DisplayConfigurationRulesCardcomponent implements OnInit {

  scheduler: Scheduler;
  @Input() rule: DisplayRule;
  @Input() isFromMyDisplayRule = false;

  @Output() newRule = new EventEmitter<DisplayRule>();
  @Output() deleteRule = new EventEmitter<string>();

  ruleForm = this.fb.group({
    id: [null],
    name: [null, [Validators.required]],
    taskableEntityId: [null, [Validators.required]],
    bgColor: [null],
    shape: [],
    position: ['undefined', [Validators.required]],
    column: [],
    ruleType: [],
    value: []
  });
  @Input() taskableEntities: TaskableEntity[] = [];
  taskableEntitySelected: TaskableEntity;
  colorType: string = undefined;

  @Input() edit: boolean = false;

  multiColorsSelected: MultiColorsEventConfigRule;
  milestoneType: IBcsNgListOption[] = [];

  constructor(
    public activeModal: NgbActiveModal,
    private fb: FormBuilder,
    public bcsNgListOptionService: BcsNgListOptionService
  ) { }

  ngOnInit(): void {
    this.bcsNgListOptionService.query().subscribe(res => this.milestoneType = (res.body || []).filter(o => o.type === 'MILESTONE_TYPE'));
    if (!this.rule) this.setNewMultiColorsEventConfigRule();
  }

  ngOnChanges() {
    if (this.rule) this.updateForm(this.rule);
  }

  save(): void {
    this.newRule.emit(this.convertFromForm());
    this.ruleForm.reset();
  }

  removeRule(cancel = false) {
    cancel ? this.deleteRule.emit(null) : this.deleteRule.emit(this.rule.id);
    this.edit = false;
  }

  updateForm(rule: DisplayRule): void {
    this.taskableEntitySelected = this.taskableEntities.find(t => t.id === rule.taskableEntityId)
    this.ruleForm.patchValue({
      id: rule.id,
      name: rule.name,
      taskableEntityId: rule.taskableEntityId,
      bgColor: rule.bgColor,
      shape: rule.shape,
      position: rule.position ?? 'undefined',
      column: rule.column,
      ruleType: rule.ruleType,
      value: rule.value,
      multiColorsEventConfigsRules: rule?.multiColorsEventConfigsRules
    });
    if (rule?.multiColorsEventConfigsRules?.color) {
      this.colorType = 'multiColor';
      this.multiColorsSelected = rule?.multiColorsEventConfigsRules;
    } else {
      this.colorType = 'color';
      this.setNewMultiColorsEventConfigRule();
    }
  }

  convertFromForm(): DisplayRule {
    return {
      ... new DisplayRule(),
      id: this.rule?.id ?? Utils.createUUID(),
      name: this.ruleForm.get(['name'])!.value,
      taskableEntityId: this.ruleForm.get(['taskableEntityId'])!.value,
      bgColor: this.ruleForm.get(['bgColor'])!.value,
      shape: this.ruleForm.get(['shape'])!.value,
      position: this.ruleForm.get(['position'])!.value != 'undefined' ? this.ruleForm.get(['position'])!.value : null,
      column: this.ruleForm.get(['column'])!.value,
      ruleType: this.ruleForm.get(['ruleType'])!.value,
      value: this.ruleForm.get(['value'])!.value,
      multiColorsEventConfigsRules: this.multiColorsSelected?.milestoneStartTypeId ? this.multiColorsSelected : this.newMultiColorsEventConfigRule()
    };
  }

  onChangeRule() {
    if (this.ruleForm.get(['column'])!.value ||
      this.ruleForm.get(['ruleType'])!.value ||
      this.ruleForm.get(['value'])!.value) {
      this.ruleForm.controls['column'].setValidators([Validators.required]);
      this.ruleForm.controls['ruleType'].setValidators([Validators.required]);
      this.ruleForm.controls['value'].setValidators([Validators.required]);
      this.ruleForm.controls['column'].updateValueAndValidity();
      this.ruleForm.controls['ruleType'].updateValueAndValidity();
      this.ruleForm.controls['value'].updateValueAndValidity();
    }
  }

  setDefaultConfigPosition() {
    this.ruleForm.controls['position'].setValue('undefined');
  }

  setDefaultConfigColor() {
    this.ruleForm.controls['bgColor'].setValue(undefined);
  }

  setDefaultConfigShape() {
    this.ruleForm.controls['shape'].setValue(undefined);
  }

  onEdit() {
    this.edit = true;
  }

  newMultiColorsEventConfigRule() {
    return { ...new MultiColorsEventConfigRule() };
  }

  setNewMultiColorsEventConfigRule() {
    this.multiColorsSelected = this.newMultiColorsEventConfigRule();
  }

  isMilestoneSelected() {
    return TaskableEntityTypeService.isMilestone(this.ruleForm.get(['taskableEntityId'])!.value);
  }

  isOperationSelected() {
    return TaskableEntityTypeService.isOperation(this.ruleForm.get(['taskableEntityId'])!.value);
  }

  cancel(): void {
    this.activeModal.dismiss();
  }

}