import { Component, OnInit } from '@angular/core';
import { Scheduler } from '@bryntum/scheduler';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { DisplayRule } from 'src/app/bcs_ng/model/display-rule.model';
import { TaskableEntity } from 'src/app/bcs_ng/model/taskable-entity.model';
import { TaskableEntityService } from '../../../taskable-entity/taskable-entity.service';

@Component({
  templateUrl: './display-configuration-rules-dialog.component.html',
  styleUrls: ['../../bryntum/bryntum.scss', './display-configuration-rules-dialog.scss'],
})
export class DisplayConfigurationRulesDialogcomponent implements OnInit {

  scheduler: Scheduler;
  rules: DisplayRule[] = [];
  taskableEntities: TaskableEntity[] = [];
  displayNewRuleForm: boolean = false;

  constructor(
    public activeModal: NgbActiveModal,
    private taskableEntityService: TaskableEntityService
  ) { }

  ngOnInit(): void {
    this.loadTaskableEntities();
    if (this.rules.length == 0) this.displayNewRuleForm = true;
  }

  loadTaskableEntities() {
    this.taskableEntityService.query().subscribe(res => {
      this.taskableEntities = res.body || [];
    });
  }

  onDisplayNewRuleForm() {
    this.displayNewRuleForm = true;
  }

  addRule(newRule: DisplayRule) {
    this.rules.push(newRule);
    this.displayNewRuleForm = false;
  }

  updateRule(updateRule: DisplayRule, index: string) {
    this.rules[index] = updateRule;
  }

  deleteRule(index: number) {
    if (index != null) {
      this.rules.splice(index, 1);
    } else this.displayNewRuleForm = false;
  }

  // ----------------------------------- Modal actions -----------------------------------
  cancel(): void {
    this.activeModal.dismiss();
  }

  save(): void {
    this.activeModal.close({ rules: this.rules });
  }
}