import { Component, OnInit } from "@angular/core";
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { IBryntumEvent } from 'src/app/bcs_ng/model/bryntum/bryntum-event.model';
import { IBryntumResource } from 'src/app/bcs_ng/model/bryntum/bryntum-resource.model';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { SortData } from '../../bryntum/sort-data';

@Component({
  templateUrl: './road-map-hidden-elements-dialog.component.html',
  styleUrls: ['../../bryntum/bryntum.scss']
})
export class RoadMapHiddenElementsDialogComponent implements OnInit {

  hiddenElements: IBryntumResource[] = [];

  hiddenDomains: IBryntumResource[] = [];
  hiddenSystems: IBryntumResource[] = [];
  hiddenVersionedSystems: IBryntumResource[] = [];
  hiddenDeployedCenters: IBryntumResource[] = [];
  hiddenOperations: IBryntumResource[] = [];
  hiddenMilestones: IBryntumResource[] = [];
  hiddenGenericTasks: IBryntumResource[] = [];
  hiddenIndependantTasks: IBryntumResource[] = [];

  disconnectedElements: IBryntumEvent[] = [];

  toRedisplay: { [key: string]: boolean } = {}
  toReconnect: { [key: string]: boolean } = {};

  constructor(public activeModal: NgbActiveModal) { }

  ngOnInit(): void {
    this.hiddenDomains = this.hiddenElements.filter(e => TaskableEntityTypeService.isDomaine(e.typeId));
    this.hiddenSystems = this.hiddenElements.filter(e => TaskableEntityTypeService.isSystem(e.typeId));
    this.hiddenVersionedSystems = this.hiddenElements.filter(e => TaskableEntityTypeService.isVersionedSystem(e.typeId));
    this.hiddenDeployedCenters = this.hiddenElements.filter(e => TaskableEntityTypeService.isDeployedCenter(e.typeId));
    this.hiddenOperations = this.hiddenElements.filter(e => TaskableEntityTypeService.isOperation(e.typeId));
    this.hiddenMilestones = this.hiddenElements.filter(e => TaskableEntityTypeService.isMilestone(e.typeId));
    this.hiddenGenericTasks = this.hiddenElements.filter(e => TaskableEntityTypeService.isGenericTask(e.typeId));
    this.hiddenIndependantTasks = this.hiddenElements.filter(e => !e.object);
    this.toRedisplay = this.hiddenElements.reduce((prev, curr) => {
      prev[curr.id] = false;
      return prev;
    }, {});
  }

  onChangeResourceCheckboxValue(id: string, event: any): void {
    this.toRedisplay[id] = event.checked;
  }

  onChangeEventCheckboxValue(id: string, event: any): void {
    this.toReconnect[id] = event.checked;
  }

  onChangeAllResourcesCheckboxValue(resourcesList: IBryntumResource[]): void {
    const allChecked = this.allResourcesChecked(resourcesList);
    resourcesList.forEach(resource => this.toRedisplay[resource.id] = !allChecked);
  }

  onChangeAllEventsCheckboxValues(eventsList: IBryntumEvent[]): void {
    const allChecked = this.allEventsChecked(eventsList);
    eventsList.forEach(event => this.toReconnect[event.id] = !allChecked);
  }

  allResourcesChecked(resourcesList: IBryntumResource[]): boolean {
    if (!resourcesList) return false;
    return !resourcesList.some(resource => !this.toRedisplay[resource.id]);
  }

  allEventsChecked(eventsList: IBryntumEvent[]): boolean {
    if (!eventsList) return false;
    return eventsList.some(event => this.toReconnect[event.id]);
  }

  getResourceName(resource: IBryntumResource): string {
    return resource.name?.length ? resource.name : resource.object?.name;
  }

  save(): void {
    const toRedisplayList = this.hiddenElements.filter(e => this.toRedisplay[e.id]);
    const orderedToRedisplayList = SortData.sortData(toRedisplayList);
    const toReconnect = this.disconnectedElements.filter(e => this.toReconnect[e.id]);
    this.activeModal.close({
      toRedisplay: orderedToRedisplayList,
      toReconnect,
    });
  }

  cancel(): void {
    this.activeModal.dismiss();
  }
}