import { Component, EventEmitter, OnDestroy, OnInit, Output, ViewChild } from '@angular/core';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { Subscription } from 'rxjs';

import { RoadMapBcsService } from '../road-map.service';
import { BryntumSchedulerComponent } from '@bryntum/scheduler-angular';
import { AddTaskableEntityComponent } from './add-taskable-entity/add-taskable-entity.component';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { LocaleManager, Scheduler } from '@bryntum/scheduler';
import { IDomaine } from 'src/app/shared/model/entity/domaine.model';
import { DomaineService } from 'src/app/entities/domaine/domaine.service';
import { ConvertData } from '../bryntum/convert-data';
import { ActivatedRoute, Router } from '@angular/router';
import { SortData } from '../bryntum/sort-data';
import { JhiEventManager } from 'ng-jhipster';
import { RoadMapHiddenElementsDialogComponent } from './hidden-elements/road-map-hidden-elements-dialog.component';
import { IRoadMapBcs, RoadMapMessage } from 'src/app/bcs_ng/model/road-map-bcs.model';
import { IDisplayTask } from 'src/app/bcs_ng/model/display-task.model';
import { IBryntumRecordState } from 'src/app/bcs_ng/model/bryntum/bryntum-record-state.model';
import { DisplayTaskService } from '../../display-task/display-task.service';
import { IBryntumResource } from 'src/app/bcs_ng/model/bryntum/bryntum-resource.model';
import { IBryntumEvent } from 'src/app/bcs_ng/model/bryntum/bryntum-event.model';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { RoadMapDeletedElementsDialogComponent } from './deleted-elements/road-map-deleted-elements-dialog.component';
import { DisplayTaskUpdateDialogcomponent } from './display-task-update/display-task-update-dialog.component';
import { DisplayConfigurationService } from '../../display-configuration/display-configuration.service';
import { DisplayConfigBryntum } from '../bryntum/display-config';
import { Utils } from 'src/app/bcs_ng/utils/util';
import { localeFr } from 'src/app/bcs_ng/utils/custom.locale.Fr';
import { DisplayConfigurationType } from 'src/app/bcs_ng/model/enumeration/display-configuration-type';
import { BryntumListeners } from '../bryntum/listeners';
import { TaskDependencyUpdateComponent } from './dependency-update/task-dependency-update.component';
import { GeneralConfig } from '../bryntum/config';
import { DependencyService } from '../../dependency/dependency.service';
import { IDependency } from 'src/app/bcs_ng/model/dependency.model';
import { BryntumUtils } from 'src/app/bcs_ng/utils/bryntumUtils';
import { TimeUnit } from 'src/app/bcs_ng/model/enumeration/time-unit.enum';
import { SchedulerStoresHandler } from '../bryntum/scheduler-stores';
import { ReorderConfig } from '../bryntum/reorder-config';
import { ConfigEventMenu } from '../bryntum/menu_config/event_menu_config';
import { TimeAxisMenu } from '../bryntum/menu_config/time_axis_menu';
import { ConfigCellMenu } from '../bryntum/menu_config/cell_menu_config';
import { scheduleMenu } from '../bryntum/menu_config/menu_config';
import { IBryntumDelay } from 'src/app/bcs_ng/model/bryntum/bryntum-delay.model';
import { BaseComponent } from 'src/app/shared/base.component';
import { AccountService } from 'src/app/core/auth/account.service';
import { GroupUsersService } from 'src/app/entities/group-users/group-users.service';
import { AfterTaskUpdateHandler } from '../bryntum/after-task-update';
import { DisplayConfigurationRulesDialogcomponent } from './display-configuration-rules/display-configuration-rules-dialog.component';
import { CrudType } from 'src/app/bcs_ng/model/enumeration/crud-type';
import { IBryntumEventClone } from 'src/app/bcs_ng/model/bryntum/bryntum-event-clone.model';
import { ITask } from 'src/app/bcs_ng/model/task/Itask.model';
import { UpdateIndependantTaskComponent } from './independant-task-update/update-independent-task.component';
import { IBryntumData } from 'src/app/bcs_ng/model/bryntum/bryntum-data.model';
import { RoadMapFieldsDialogComponent } from './road-map-fields/road-map-fields-dialog.component';
import { ApplyDisplayConfigurationPreferencesDialogcomponent } from './user-display-configuration-preferences/apply/apply-display-configuration-preferences-dialog.component';
import { SaveDisplayConfigurationPreferencesDialogcomponent } from './user-display-configuration-preferences/save/save-display-configuration-preferences-dialog.component';
import { FormatTaskCellComponent } from './format-task-cell/format-task-cell.component';
import { DisplayConfTaskProperties } from 'src/app/bcs_ng/model/bryntum/display-configuration/display-conf-task-properties';
import { DisplayConfiguration } from 'src/app/bcs_ng/model/display-configuration.model';
import { DisplayRule } from 'src/app/bcs_ng/model/display-rule.model';
import { DisplayConfRoadMapProperties, FilterArrowMode, TimeRangeMode } from 'src/app/bcs_ng/model/bryntum/display-configuration/display-conf-road-map-properties';
import { routePath } from 'src/app/routes/path-routing.constant';

@Component({
  selector: 'jhi-road-map-bcs-update',
  templateUrl: './road-map-bcs-update.component.html',
  styleUrls: ['../bryntum/bryntum.scss', '../../bcs-ng.scss'],
  providers: [
    ConfigEventMenu, GeneralConfig, ConfigCellMenu, DisplayConfigBryntum, TimeAxisMenu, BryntumListeners,
    BryntumUtils, SchedulerStoresHandler, ReorderConfig, AfterTaskUpdateHandler
  ]
})
export class RoadMapBcsUpdateComponent extends BaseComponent implements OnInit, OnDestroy {
  @Output() switchMode: EventEmitter<any> = new EventEmitter<any>();
  @Output() navigateToClone: EventEmitter<any> = new EventEmitter<any>();

  // Main variables
  roadMap?: IRoadMapBcs;
  domaines: IDomaine[] = [];
  domainesSelected: IDomaine[] = [];
  onLoad = true;
  isSaving = false;
  eventSubscriber?: Subscription;

  // Intermediate level
  milestones: ITask[] = [];
  recordStateMap: { [key: string]: IBryntumRecordState } = {};
  initialRecordStateMap: { [key: string]: IBryntumRecordState } = {};
  clonesMap: { [key: string]: IBryntumEventClone } = {};
  dependencyStateMap: { [key: string]: IDependency } = {};
  initialDependencyStateMap: { [key: string]: IDependency } = {};
  delaysMap: { [key: string]: IBryntumDelay } = {};

  // Bryntum
  @ViewChild('scheduler') schedulerComponent!: BryntumSchedulerComponent;
  scheduler: Scheduler;
  schedulerConfig;
  listenersScheduler;
  eventMenu;
  scheduleMenu = scheduleMenu;
  timeAxisHeaderMenu;
  cellMenu;
  listeners;
  rowReorder;

  resourceFontSize: number;
  eventFontSize: number;
  resourceFontColor: string = '';
  timeRangeStartDate: Date = null;
  timeRangeMode: TimeRangeMode = TimeRangeMode.NONE;
  TimeRangeMode = TimeRangeMode;
  filterDelayArrow: FilterArrowMode = FilterArrowMode.CUSTOM;
  filterRpsDelayArrow: FilterArrowMode = FilterArrowMode.CUSTOM;
  FilterArrowMode = FilterArrowMode;

  dropdownSettingsDomaine = {
    singleSelection: false,
    idField: 'id',
    textField: 'domaine',
    selectAllText: 'Tout sélectionner',
    unSelectAllText: 'Tout désélectionner',
    itemsShowLimit: 10,
    allowSearchFilter: true,
    searchPlaceholderText: 'Rechercher'
  };

  levelsResource: number[];

  dependencyEdit = {
    editorConfig: {
      items: {
        // Custom label for the type field
        typeField: {
          label: 'Kind'
        }
      },
      bbar: {
        items: {
          // Hiding save button
          saveButton: {
            hidden: true
          }
        }
      }
    }
  };

  messageRemovedOperations: string;
  messageAddedOperations: string;
  messageRemovedMilestones: string;
  messageAddedMilestones: string;

  showFormattingArea: boolean = false;

  constructor(
    protected roadMapService: RoadMapBcsService,
    protected displayTaskService: DisplayTaskService,
    protected displayConfigurationService: DisplayConfigurationService,
    protected dependencyService: DependencyService,
    protected domaineService: DomaineService,
    protected modalService: NgbModal,
    protected activatedRoute: ActivatedRoute,
    protected router: Router,
    protected generalConfig: GeneralConfig,
    protected configEventMenu: ConfigEventMenu,
    protected displayConfigBryntum: DisplayConfigBryntum,
    protected timeAxisMenu: TimeAxisMenu,
    protected configCellMenu: ConfigCellMenu,
    protected bryntumListeners: BryntumListeners,
    public bryntumUtils: BryntumUtils,
    public reorderConfg: ReorderConfig,
    protected eventManager: JhiEventManager,
    protected schedulerStoresHandler: SchedulerStoresHandler,
    protected afterTaskUpdateHandler: AfterTaskUpdateHandler,
    protected accountService: AccountService,
    protected groupUsersService: GroupUsersService
  ) {
    super(accountService)
  }

  //------------------------------------Component lifecycle------------------------------------
  ngOnInit(): void {
    this.onLoad = true;
    this.initConfig(); // Custom menus
    this.activatedRoute.paramMap.subscribe(params => {
      const id = params.get("id");
      this.roadMapService.find(id).subscribe(async res => {
        this.roadMap = await this.roadMapService.completeDisplayTasks(res.body);
        this.roadMap.domainesId = JSON.parse(this.roadMap.domainesId);
        const messages: RoadMapMessage = this.roadMap?.message ? JSON.parse(this.roadMap?.message) : null;
        if (messages) {
          this.messageAddedOperations = 'Opération(s) ajoutée(s) automatiquement : ' + (messages?.addedOperations ?? 'Aucune');
          this.messageRemovedOperations = 'Opération(s) supprimée(s) automatiquement : ' + (messages?.removedOperations ?? 'Aucune');
          this.messageAddedMilestones = 'Jalon(s) ajouté(s) automatiquement : ' + (messages?.addedMilestones ?? 'Aucun');
          this.messageRemovedMilestones = 'Jalon(s) supprimé(s) automatiquement : ' + (messages?.removedMilestones ?? 'Aucun');
        }
        this.loadDomains();
        this.initRecordStateMap(SortData.sortData(this.roadMap?.tasks));
        this.initClonesMap(this.recordStateMap);
        this.initDependencyStateMap(this.roadMap?.tasks);
        this.initDelaysMap(this.recordStateMap, this.dependencyStateMap);
        this.setStateMapsToAllConfigs();
        this.initScheduler(this.recordStateMap, this.clonesMap, this.delaysMap, this.dependencyStateMap);
        this.displayConfigBryntum.applyProperties(this.roadMap?.displayConfig);
        if (this.roadMap?.displayConfig) {
          const properties: DisplayConfRoadMapProperties = JSON.parse(this.roadMap?.displayConfig.properties);
          this.resourceFontSize = properties.resourceFontSize ?? 16;
          this.resourceFontColor = properties.resourceFontColor ?? '#474444';
          this.eventFontSize = properties.eventFontSize ?? 16;
          this.configEventMenu.defaultEventFontSize = this.eventFontSize;
          this.timeRangeStartDate = properties.timeRangeStartDate;
          this.timeRangeMode = properties.timeRangeMode ?? TimeRangeMode.NONE;
          this.filterDelayArrow = properties.filterDelayArrow ?? FilterArrowMode.CUSTOM;
          this.filterRpsDelayArrow = properties.filterRpsDelayArrow ?? FilterArrowMode.CUSTOM;
        }
        if (this.timeRangeMode !== TimeRangeMode.NONE) {
          const timeRange = this.bryntumUtils.buildTimeRange(this.timeRangeStartDate, this.timeRangeMode);
          this.scheduler.project.timeRangeStore.add(timeRange);
        }

        this.schedulerStoresHandler.setStoresData();
        this.onLoad = false;
      });
    });
  }

  ngOnDestroy(): void {
    this.eventManager.destroy(this.eventSubscriber);
  }

  ngAfterViewInit() {
    this.scheduler = this.schedulerComponent.instance;
    this.setSchedulerToAllConfigs();
    LocaleManager.registerLocale('Fr', {
      desc: 'Français', locale: localeFr
    });
    LocaleManager.applyLocale('Fr');
  }

  //------------------------------------Component lifecycle helpers------------------------------------
  loadDomains(): void {
    this.domaineService.queryAllActive().subscribe(
      (res) => {
        this.domaines = res.body || [];
        this.domainesSelected = this.domaines.filter(d => this.roadMap.domainesId.includes(d.id));
      },
      (err) => this.onSaveError(err)
    );
  }

  initRecordStateMap(displayTasks: IDisplayTask[]): void {
    this.initialRecordStateMap = ConvertData.displayTasksToRecordsMap(SortData.sortData(displayTasks), undefined, true);
    this.recordStateMap = ConvertData.displayTasksToRecordsMap(SortData.sortData(displayTasks), undefined, true);
  }

  initClonesMap(recordsMap: { [key: string]: IBryntumRecordState }): void {
    this.clonesMap = ConvertData.recordsToClonesMap(recordsMap);
  }

  initDependencyStateMap(displayTasks: IDisplayTask[]): void {
    this.dependencyStateMap = ConvertData.displayTasksToDependenciesMap(displayTasks);
    this.initialDependencyStateMap = ConvertData.displayTasksToDependenciesMap(displayTasks);
  }

  initDelaysMap(
    recordsMap: { [key: string]: IBryntumRecordState },
    dependenciesMap: { [key: string]: IDependency }
  ): void {
    this.delaysMap = ConvertData.recordsMapToDelaysMap(recordsMap, dependenciesMap);
  }

  initScheduler(
    recordsMap: { [key: string]: IBryntumRecordState },
    clonesMap: { [key: string]: IBryntumEventClone },
    delaysMap: { [key: string]: IBryntumDelay },
    dependenciesMap: { [key: string]: IDependency },
  ): void {
    const { resources, events, assignments, dependencies } = ConvertData.recordsToMainData(recordsMap, dependenciesMap, this.milestones, true);
    const { cloneEvents, cloneAssignments } = ConvertData.clonesMapToCloneEvents(clonesMap);
    const { delayEvents, delayAssignments } = ConvertData.recordsMapToDelayEvents(recordsMap, delaysMap);
    this.addToScheduler({ resources, events, assignments, dependencies, cloneEvents, cloneAssignments, delayEvents, delayAssignments });
    this.scheduler.resourceStore?.filter({ id: 'displayed', property: 'displayed', value: true });
    this.scheduler.resourceStore?.filter({ id: 'deleted', property: 'deleted', value: false });
  }

  initConfig() {
    this.schedulerConfig = this.generalConfig.schedulerConfig;
    this.eventMenu = this.configEventMenu.eventMenu;
    this.cellMenu = this.configCellMenu.cellMenu;
    this.timeAxisHeaderMenu = this.timeAxisMenu.timeAxisHeaderMenu;
    this.listeners = this.bryntumListeners.listeners;
    this.rowReorder = this.reorderConfg.rowReorder;
    this.registerChange();
  }

  setSchedulerToAllConfigs() {
    this.configEventMenu.scheduler = this.scheduler;
    this.configCellMenu.scheduler = this.scheduler;
    this.displayConfigBryntum.scheduler = this.scheduler;
    this.bryntumUtils.scheduler = this.scheduler;
    this.schedulerStoresHandler.scheduler = this.scheduler;
    this.reorderConfg.scheduler = this.scheduler;
    this.afterTaskUpdateHandler.scheduler = this.scheduler;
  }

  setStateMapsToAllConfigs() {
    this.reorderConfg.recordStateMap = this.recordStateMap;
    this.configEventMenu.recordStateMap = this.recordStateMap;
    this.configEventMenu.clonesMap = this.clonesMap;
    this.configEventMenu.delaysMap = this.delaysMap;
    this.configCellMenu.recordStateMap = this.recordStateMap;
    this.configCellMenu.initialRecordStateMap = this.initialRecordStateMap;
    this.configCellMenu.clonesMap = this.clonesMap;
    this.bryntumListeners.recordStateMap = this.recordStateMap;
    this.afterTaskUpdateHandler.recordStateMap = this.recordStateMap;
    this.afterTaskUpdateHandler.clonesMap = this.clonesMap;
    this.afterTaskUpdateHandler.dependencyStateMap = this.dependencyStateMap;
    this.schedulerStoresHandler.initialRecordStateMap = this.initialRecordStateMap;
    this.displayConfigBryntum.recordStateMap = this.recordStateMap;
  }

  registerChange() {
    this.eventSubscriber = this.eventManager.subscribe('openUpdateModal', (cellRecord) => {
      this.updateDisplayTask(cellRecord.content.cellData.id);
    });
    this.eventSubscriber.add(this.eventManager.subscribe('editDependency', (event) => {
      const predecessorId = event.content.dependencyRecord.data.from;
      const successorId = event.content.dependencyRecord.data.to;
      this.updateDependency(predecessorId, successorId);
    }));
    this.eventSubscriber.add(this.eventManager.subscribe('hideOrDisplayDependencies', (event) => {
      const id = event.content.data.id;
      const hidden = this.recordStateMap[id].event.hidePredecessors;
      if (hidden) {
        this.reconnectWithPredecessors(id);
      } else {
        this.disconnectFromPredecessors(id);
      }
    }));
    this.eventSubscriber.add(this.eventManager.subscribe('resourceMoved', () => {
      // TODO : this is temporarily commented, because the dates of independant parent tasks are not correctly updated
      this.schedulerStoresHandler.setStoresData();
    }));
    this.eventSubscriber.add(this.eventManager.subscribe('onFormat', (cellRecord => {
      this.formatTaskCell(cellRecord.content.cellData.id);
    })));
  }

  //------------------------------------Actions on roadmap------------------------------------
  controlRoadmapFields(): boolean {
    return this.onLoad || (this.roadMap?.name && this.roadMap?.name?.length <= 128 && (this.domaines?.length == 0 || this.domainesSelected?.length > 0));
  }

  saveAs(): void {
    const modalRef = this.modalService.open(RoadMapFieldsDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.domains = this.domaines;
    modalRef.result.then((res: { title: string, domains: IDomaine[] }) => {
      this.save(false, res);
    });
  }

  save(stayOnEdit: boolean, cloneData?: { title: string, domains: IDomaine[] }): void {
    this.isSaving = true;
    this.roadMap.domainesId = JSON.stringify(this.domainesSelected.map(d => d.id));
    if (this.isAdminBCS) this.saveRoadMap(stayOnEdit, cloneData);
    else {
      this.groupUsersService.isAllowToCreateInBcsByDomaines(this.domainesSelected, CrudType.CREATE).subscribe(
        () => this.saveRoadMap(stayOnEdit),
        err => this.onSaveError(err)
      );
    }
  }

  private saveRoadMap(stayOnEdit: boolean, cloneData?: { title: string, domains: IDomaine[] }) {
    const toSubscribe = this.saveDisplayConfig();
    toSubscribe.subscribe((res) => {
      this.roadMap.displayConfig = res?.body;
      const newOrUpdated = Object.values(this.recordStateMap).filter(record => this.recordIsToSave(record));
      const displayTasks = ConvertData.recordsToDisplayTasks(newOrUpdated, this.roadMap.id);
      this.roadMap.tasks = displayTasks;
      this.roadMapService.update(this.roadMap).subscribe(res => {
        this.roadMap = res?.body;

        // Handle records to erase
        const recordsToErase = Object.values(this.recordStateMap).filter(record => record.toErase);
        if (recordsToErase.length) {
          this.displayTaskService
            .deleteMany(recordsToErase.map(record => record.id))
            .subscribe(() => { }, err => this.onSaveError(err));
        }

        if (cloneData) { // If we clicked on "Save as"
          const roadMapClone = this.roadMapService.cloneRoadMap(this.roadMap, cloneData.domains, cloneData.title);
          const cloneDisplayTasks = ConvertData.recordsToDisplayTasks(Object.values(this.recordStateMap), roadMapClone.id)
            .filter(dt => !recordsToErase.some(r => r.id === dt.id)); // We must not include DTs that are erased
          const cloneDisplayTaskIdsMap: { [key: string]: string } = {};
          cloneDisplayTasks.forEach(dt => {
            const newId = Utils.createUUID();
            cloneDisplayTaskIdsMap[dt.id] = newId;
          });
          cloneDisplayTasks.forEach(dt => {
            dt.id = cloneDisplayTaskIdsMap[dt.id];
            let additionalProperties = JSON.parse(dt.additionalProperties)
            if (additionalProperties?.resourcesId) {
              const cloneResourceId = [];
              additionalProperties?.resourcesId.forEach(r => {
                cloneResourceId.push(cloneDisplayTaskIdsMap[r])
              });
              additionalProperties.resourcesId = cloneResourceId;
              dt.additionalProperties = JSON.stringify(additionalProperties);
            }
            if (dt.parentId) dt.parentId = cloneDisplayTaskIdsMap[dt.parentId];
            if (dt.displayConfig) dt.displayConfig.id = Utils.createUUID();
            if (dt.displayConfig?.properties) {
              const properties: DisplayConfTaskProperties = JSON.parse(dt.displayConfig?.properties);
              if (properties.multiColorsEventConfigs) {
                properties.multiColorsEventConfigs.forEach(mcec => {
                  mcec.startOffsetEventId = cloneDisplayTaskIdsMap[mcec.startOffsetEventId];
                  mcec.endOffsetEventId = cloneDisplayTaskIdsMap[mcec.endOffsetEventId];
                });
                dt.displayConfig.properties = JSON.stringify(properties);
              }
            }
            if (dt.displayConfig?.displayRules) {
              const taskCloneDisplayRules: DisplayRule[] = [];
              dt.displayConfig?.displayRules?.forEach(rule => {
                taskCloneDisplayRules.push({
                  ...rule,
                  id: Utils.createUUID(),
                })
              })
              dt.displayConfig.displayRules = taskCloneDisplayRules;
            }
          })
          roadMapClone.tasks = cloneDisplayTasks;
          this.displayConfigurationService.create(roadMapClone.displayConfig).subscribe(() => {
            this.roadMapService.create(roadMapClone).subscribe(() => {
              this.isSaving = false;
              this.navigateToView(stayOnEdit, roadMapClone.id);
            })
          });
        } else {
          this.isSaving = false;
          if (stayOnEdit) {
            Object.values(this.recordStateMap).filter(r => r.toErase).forEach(r => {
              delete this.recordStateMap[r.id];
            });
          }
          this.navigateToView(stayOnEdit);
        }
      })
    },
      (err) => console.error(err));
  }

  protected onSaveError(err): void {
    this.isSaving = false;
    console.error(err);
  }

  saveDisplayConfig() {
    const displayConfId = this.roadMap?.displayConfig?.id ?? Utils.createUUID();
    const displayConfig = this.displayConfigBryntum.createNewDisplayConfiguration(displayConfId, 'BaseDisplayConfig',
      DisplayConfigurationType.ROADMAP,
      this.displayConfigBryntum.getProperties(this.resourceFontSize, this.eventFontSize, this.resourceFontColor, this.timeRangeStartDate, this.timeRangeMode, this.filterDelayArrow, this.filterRpsDelayArrow));

    return this.roadMap?.displayConfig?.id ?
      this.displayConfigurationService.update(displayConfig) :
      this.displayConfigurationService.create(displayConfig);
  }

  navigateToView(stayOnEdit?: boolean, cloneId?: string) {
    if (stayOnEdit) {
      this.router.navigate([routePath.BCS_ROADMAP + '/' + this.roadMap.id], { queryParamsHandling: "preserve" });
    } else if (cloneId) {
      this.navigateToClone.emit(cloneId);
    } else {
      this.switchMode.emit();
    }
  }

  // ----------------------------------Add or update tasks----------------------------------
  addTaskableEntity(): void {
    const modalRef = this.modalService.open(AddTaskableEntityComponent, { windowClass: 'xxlModal', backdrop: 'static' });
    modalRef.result.then((result: { records: IBryntumRecordState[], dependencies: IDependency[] }) => {
      const tempRecordStateMap: { [key: string]: IBryntumRecordState } = {};
      const combinedMap = {};
      result.records.forEach(record => {
        if (TaskableEntityTypeService.isMilestone(record.resource.typeId) && record.resource.object) {
          // If another task is based upon the same object and its date is overridden, the new task must take the owerridden value
          const duplicate = Object.values(this.recordStateMap).find(r => r.resource.object?.id === record.resource.object.id);
          if (duplicate && duplicate.event.startDate) {
            record.event.startDate = duplicate.event.startDate;
            record.event.endDate = duplicate.event.endDate;
            record.resource.cls = 'overridden-task';
          }
        }
        tempRecordStateMap[record.id] = record;
        combinedMap[record.id] = record;
      });
      Object.keys(tempRecordStateMap).forEach(recordId => combinedMap[recordId] = tempRecordStateMap[recordId]);
      const newMilestones = Object
        .values(tempRecordStateMap)
        .filter(record => {
          return TaskableEntityTypeService.isMilestone(record.resource.typeId) &&
            !this.milestones.some(m => m.id === record.resource.object.id)
        })
        .map(record => record.resource.object);
      result.dependencies.forEach(dep => {
        if (!this.dependencyStateMap[dep.id]) {
          this.dependencyStateMap[dep.id] = dep;
        }
      });

      const mainData = ConvertData.recordsToMainData(tempRecordStateMap, this.dependencyStateMap, this.milestones);
      this.initDelaysMap(combinedMap, this.dependencyStateMap);
      const delaysData = ConvertData.recordsMapToDelayEvents(combinedMap, this.delaysMap);
      this.addToScheduler({ ...mainData, ...delaysData, cloneEvents: [], cloneAssignments: [] });
      result.records.forEach(record => this.recordStateMap[record.id] = record);
      this.schedulerStoresHandler.setStoresData();
      this.displayConfigBryntum.applyConfigRules();
      this.levelsResource = BryntumUtils.getLevelsOfNode(this.scheduler.resourceStore);
    }).catch(err => {
      if (err) console.error(err)
    });
  }

  addIndependantDisplayTask(): void {
    const modalRef = this.modalService.open(UpdateIndependantTaskComponent, { size: 'lg', backdrop: 'static' });
    modalRef.result.then((result: IBryntumRecordState) => {
      this.recordStateMap[result.id] = result;
      this.addToScheduler({
        resources: result.resource,
        events: result.event,
        assignments: { resourceId: result.id, eventId: result.id, readOnly: true },
        dependencies: [],
        cloneEvents: [],
        cloneAssignments: [],
        delayEvents: [],
        delayAssignments: [],
      })
    }).catch(err => {
      if (err) console.error(err);
    });
  }

  updateDisplayTask(id: string): void {
    if (this.recordStateMap[id].resource.object) this.updateDisplayTaskWithObject(id);
    else this.updateIndependantDisplayTask(id);
  }

  updateDisplayTaskWithObject(id: string): void {
    const modalRef = this.modalService.open(DisplayTaskUpdateDialogcomponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.record = this.recordStateMap[id];
    modalRef.componentInstance.eventRecords = Object
      .values(this.recordStateMap)
      .filter(record => TaskableEntityTypeService.isMilestone(record.resource.typeId));
    modalRef.result.then((result: {
      newRecordValue: IBryntumRecordState,
      milestoneDateModified: boolean,
      overridingDateRemoved: boolean,
    }) => {
      // If it is a milestone and its start date have been updated, update the duplicates and the successors
      if (result.milestoneDateModified) {
        const resultStartDate = result.newRecordValue.event.startDate ??
          new Date(result.newRecordValue.resource.object.startDate?.toString());
        const recordStartDate = this.recordStateMap[result.newRecordValue.id].event.startDate ??
          new Date(this.recordStateMap[result.newRecordValue.id].resource.object.startDate?.toString());
        const timeShift = resultStartDate.getTime() - recordStartDate.getTime();
        const dateShift = timeShift / (1000 * 3600 * 24);
        if (!result.overridingDateRemoved) {
          this.afterTaskUpdateHandler.updateSuccessors(result.newRecordValue.id, dateShift, false, []);
        }
        this.afterTaskUpdateHandler.updateDuplicates(result.newRecordValue, dateShift);
      }

      // Update the record and the element in the scheduler
      this.recordStateMap[result.newRecordValue.id] = result.newRecordValue;
      this.afterTaskUpdateHandler.updateDisplayElement(result.newRecordValue.id);

      // Update delay events
      this.updateDelays();
    }).catch(err => {
      if (err) console.error(err)
    });
  }

  updateIndependantDisplayTask(id: string): void {
    const modalRef = this.modalService.open(UpdateIndependantTaskComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.record = this.recordStateMap[id];
    modalRef.result.then((result: IBryntumRecordState) => {
      this.recordStateMap[id] = result;
      this.afterTaskUpdateHandler.updateDisplayElement(result.id);
    }).catch(err => {
      if (err) console.error(err);
    });
  }

  private updateDelays() {
    // Remove all events with delayEvent = true
    const eventsToRemove = this.scheduler.eventStore.allRecords.filter(eventRecord => eventRecord.get('delayEvent'));
    this.scheduler.eventStore.remove(eventsToRemove);
    this.initDelaysMap(this.recordStateMap, this.dependencyStateMap);
    const delaysData = ConvertData.recordsMapToDelayEvents(this.recordStateMap, this.delaysMap);
    this.addToScheduler({ ...delaysData, resources: [], events: [], assignments: [], dependencies: [], cloneEvents: [], cloneAssignments: [] });
  }

  // ----------------------------------Display configuration----------------------------------
  openDisplayConfigurationRules() {
    const modalRef = this.modalService.open(DisplayConfigurationRulesDialogcomponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.rules = this.displayConfigBryntum.displayConfigRules ?? [];
    modalRef.result.then((result: { rules: DisplayRule[] }) => {
      if (result.rules) {
        this.displayConfigBryntum.setDisplayConfigRules = result.rules;
        this.displayConfigBryntum.applyConfigRules();
      }
    });
  }

  openSaveDisplayConfiguration() {
    const modalRef = this.modalService.open(SaveDisplayConfigurationPreferencesDialogcomponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.rules = this.displayConfigBryntum.displayConfigRules ?? [];
  }

  openApplyDisplayConfiguration() {
    const modalRef = this.modalService.open(ApplyDisplayConfigurationPreferencesDialogcomponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.userId = this.userIdentity.id;
    modalRef.result.then((result: { rules: DisplayRule[] }) => {
      if (result.rules) {
        this.displayConfigBryntum.addDisplayConfRules(result.rules);
        this.displayConfigBryntum.applyConfigRules();
      }
    });
  }

  // ----------------------------------Hide and redisplay----------------------------------
  hiddenLength(): number {
    const hidden = Object
      .values(this.recordStateMap)
      .filter(record => record.resource.hidden && !record.resource.deleted);
    const disconnected = Object
      .values(this.recordStateMap)
      .filter(record => record.event.hidePredecessors && !record.resource.deleted);
    return hidden.length + disconnected.length;
  }

  redisplayRecord(id: string) {
    const resource = this.recordStateMap[id].resource
    resource.hidden = false;
    if (!resource.deleted) resource.displayed = true;
    // If the resource has a hidden parent, it must not be added to the store
    if (resource.parentId && this.recordStateMap[resource.parentId].resource.hidden) return;
    const record = this.scheduler.resourceStore.getById(id);
    record.set('displayed', true);

    // If there is a hidden clone, it must be redisplayed
    const clone = Object.values(this.clonesMap).find(c => c.eventId === id);
    if (clone) {
      this.scheduler.eventStore.add(clone.event);
      this.scheduler.assignmentStore.assignEventToResource(clone.id as any, clone.event.locationId as any);
    }

    const children = this.getChildren(id);
    children.forEach(child => this.redisplayRecord(child.id.toString()));
  }

  showHiddenElementsModal(): void {
    const modalRef = this.modalService.open(RoadMapHiddenElementsDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.hiddenElements = Object
      .values(this.recordStateMap)
      .filter(record => record.resource.hidden && !record.resource.deleted)
      .map(record => {
        return {
          ...record.resource,
          name: record.resource.name ?? record.resource.object.name,
        }
      });
    modalRef.componentInstance.disconnectedElements = Object
      .values(this.recordStateMap)
      .filter(record => record.event.hidePredecessors)
      .map(record => {
        return {
          ...record.resource,
          name: record.resource.name ?? record.resource.object.name,
        }
      });
    modalRef.result.then((result: { toRedisplay: IBryntumResource[], toReconnect: IBryntumEvent[] }) => {
      result.toRedisplay?.forEach(r => this.redisplayRecord(r.id));
      result.toReconnect?.forEach(r => this.reconnectWithPredecessors(r.id));
      this.schedulerStoresHandler.setStoresData();
      const dependencies = ConvertData.recordsToDependencies(this.recordStateMap, this.dependencyStateMap, this.milestones);
      this.scheduler.dependencyStore.add(dependencies);
    }).catch(err => {
      if (err) console.error(err);
    });
  }

  //----------------------------------Remove and reincarnate----------------------------------
  removedResourcesLength(): number {
    return Object.values(this.recordStateMap).filter(record => record.resource.deleted).length;
  }

  reincarnateResource(resource: IBryntumResource) {
    this.recordStateMap[resource.id].resource.deleted = false;
    if (!this.recordStateMap[resource.id].resource.hidden) {
      this.recordStateMap[resource.id].resource.displayed = true;
      const toReincarnate = this.scheduler.resourceStore.getById(resource.id);
      toReincarnate.set('displayed', true);
      toReincarnate.set('deleted', false);

      // If there is a hidden clone, it must be redisplayed
      const clone = Object.values(this.clonesMap).find(c => c.eventId === resource.id);
      if (clone) {
        this.scheduler.eventStore.add(clone.event);
        this.scheduler.assignmentStore.assignEventToResource(clone.id as any, clone.event.locationId as any);
      }
    }
    // All the resources children must be reincarnated
    const children = this.getChildren(resource.id);
    children.forEach(child => this.reincarnateResource(child.resource));
  }

  showDeletedElementsModal(): void {
    const modalRef = this.modalService.open(RoadMapDeletedElementsDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.deletedRecords = Object
      .values(this.recordStateMap)
      .filter(recordState => recordState.resource.deleted);
    modalRef.result.then((result: {
      toRedisplay: IBryntumResource[],
      toErase: IBryntumResource[],
    }) => {
      if (result) {
        result.toRedisplay.forEach(r => this.reincarnateResource(r));
        result.toErase.forEach(r => this.setToErase(r.id, !this.recordStateMap[r.id].toErase));
        this.schedulerStoresHandler.setStoresData();
        const dependencies = ConvertData.recordsToDependencies(this.recordStateMap, this.dependencyStateMap, this.milestones);
        this.scheduler.dependencyStore.add(dependencies);
      }
    }).catch(err => {
      if (err) console.error(err);
    });
  }

  private setToErase(id: string, value: boolean): void {
    this.recordStateMap[id].toErase = value;
    const children = this.getChildren(id);
    children.forEach(child => this.setToErase(child.id, value));
  }

  //--------------------------------Expand or collapse node resource----------------------------------
  expandAllLevel(expand: boolean = true) {
    this.scheduler.resourceStore.traverse(resource => {
      this.scheduler.resourceStore.toggleCollapse(resource?.data?.id, !expand);
      this.recordStateMap[resource?.data?.id].resource.expanded = resource.isExpanded();
    })
  }

  expandLevel(level: number) {
    if (level == 0) this.expandAllLevel(false);
    else {
      this.scheduler.resourceStore.traverse(resource => {
        if (resource.childLevel < level) {
          this.scheduler.resourceStore.toggleCollapse(resource?.data?.id, false);
          this.recordStateMap[resource?.data?.id].resource.expanded = true;
        } else {
          this.scheduler.resourceStore.toggleCollapse(resource?.data?.id, true);
        }
      })
    }
  }

  //----------------------------------Dependency management--------------------------------------
  updateDependency(predecessorId: string, successorId: string): void {
    const modalRef = this.modalService.open(TaskDependencyUpdateComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.predecessorId = predecessorId;
    modalRef.componentInstance.successorId = successorId;
    modalRef.result.then((res: { id: string, timeGapDuration: number, timeGapUnit: TimeUnit }) => {
      this.dependencyStateMap[res.id].timeGapDuration = res.timeGapDuration;
      this.dependencyStateMap[res.id].timeGapUnit = res.timeGapUnit;
    }).catch(err => {
      if (err) console.error(err);
    })
  }

  disconnectFromPredecessors(successorId: string): void {
    this.recordStateMap[successorId].event.hidePredecessors = true;
    const toRemoveFromStore = this.scheduler.dependencyStore.query((d) => {
      return d.originalData.to === successorId;
    });
    this.scheduler.dependencyStore.remove(toRemoveFromStore);
  }

  reconnectWithPredecessors(successorId: string): void {
    this.recordStateMap[successorId].event.hidePredecessors = false;
    const successorMilestoneId = this.recordStateMap[successorId]?.resource?.object?.id;
    if (successorMilestoneId) {
      const predecessors = ConvertData.getPredecessors(successorMilestoneId, this.recordStateMap, this.dependencyStateMap);
      const toAdd = predecessors.map(pred => {
        return {
          from: pred.id,
          to: successorId
        }
      });
      this.scheduler.dependencyStore.add(toAdd);
    }
  }

  //----------------------------------Text formatting--------------------------------------
  increaseResourceFontSize(): void {
    this.resourceFontSize += 1;
    this.displayConfigBryntum.applyTextFormatting(this.resourceFontSize, this.resourceFontColor);
  }

  reduceResourceFontSize(): void {
    if (this.resourceFontSize > 5) this.resourceFontSize -= 1;
    this.displayConfigBryntum.applyTextFormatting(this.resourceFontSize, this.resourceFontColor);
  }

  increaseEventFontSize(): void {
    this.eventFontSize += 1;
    this.displayConfigBryntum.applyEventTextFormatting(this.eventFontSize);
  }

  reduceEventFontSize(): void {
    if (this.eventFontSize > 5) this.eventFontSize -= 1;
    this.displayConfigBryntum.applyEventTextFormatting(this.eventFontSize);
  }

  changeResourceFontColor(): void {
    this.displayConfigBryntum.applyTextFormatting(this.resourceFontSize, this.resourceFontColor);
  }

  private formatTaskCell(id: string): void {
    const modalRef = this.modalService.open(FormatTaskCellComponent, { size: 'lg', backdrop: 'static' });
    const resourceData = this.scheduler.resourceStore.getById(id);
    modalRef.componentInstance.fontSize = resourceData?.getData('fontSize') ?? this.resourceFontSize;
    modalRef.componentInstance.color = resourceData?.getData('fontColor') ?? this.resourceFontColor;
    modalRef.componentInstance.rowHeight = resourceData?.getData('rowHeight') ?? this.scheduler.rowHeight;
    modalRef.componentInstance.name = resourceData.getData('name');
    modalRef.componentInstance.defaultRowHeight = this.scheduler.rowHeight;
    modalRef.componentInstance.defaultFontSize = this.resourceFontSize;
    modalRef.componentInstance.defaultColor = this.resourceFontColor;
    modalRef.result.then((result: { fontSize: number; rowHeight: number; color: string; }) => {
      this.displayConfigBryntum.applyIndividualTextFormatting(id, result.rowHeight, result.fontSize, result.color);
      const displayConfig = this.recordStateMap[id].event.displayConfig ?
        { ...this.recordStateMap[id].event.displayConfig } :
        { ...new DisplayConfiguration(), type: DisplayConfigurationType.TASK };
      const properties: DisplayConfTaskProperties = JSON.parse(displayConfig?.properties ?? '{}');
      properties.rowHeight = result.rowHeight;
      properties.resourceFontSize = result.fontSize;
      properties.resourceFontColor = result.color;
      displayConfig.properties = JSON.stringify(properties);
      this.recordStateMap[id].event.displayConfig = displayConfig;
    }).catch(err => {
      if (err) console.log(err);
    });
  }

  //----------------------------------Vertical time cursor--------------------------------------
  setTimeCursor(timeRangeMode: TimeRangeMode, event?: any): void {
    if (timeRangeMode !== TimeRangeMode.NONE) {
      let startDate = event ?? new Date();
      if (this.scheduler.project.timeRangeStore.allRecords?.length) {
        this.scheduler.project.timeRangeStore.allRecords[0].set('startDate', startDate);
        this.scheduler.project.timeRangeStore.allRecords[0].set('name', startDate.toLocaleDateString());
      } else {
        this.scheduler.project.timeRangeStore.add({
          startDate,
          name: startDate.toLocaleDateString(),
        })
      }
    } else {
      this.scheduler.project.timeRangeStore.removeAll();
    }
    this.timeRangeMode = timeRangeMode;
  }

  //----------------------------------Vertical time cursor--------------------------------------
  public setfilterDelayArrow(filterDelayArrow: FilterArrowMode, idFilter: string) {
    if (idFilter == 'hiddeArrow') this.filterDelayArrow = filterDelayArrow;
    else this.filterRpsDelayArrow = filterDelayArrow;
    this.displayConfigBryntum.applyFilterArrow(filterDelayArrow, idFilter);
  }

  //----------------------------------Bryntum functions--------------------------------------
  private addToScheduler(schedulerData: IBryntumData): void {
    const { resources, events, cloneEvents, delayEvents, assignments, cloneAssignments, delayAssignments, dependencies } = schedulerData;
    if (resources) this.scheduler.resourceStore?.add(resources);
    if (events) this.scheduler.eventStore?.add(events);
    if (cloneEvents) this.scheduler.eventStore?.add(cloneEvents);
    if (delayEvents) this.scheduler.eventStore?.add(delayEvents);
    if (assignments) this.scheduler.assignmentStore?.add(assignments);
    if (cloneAssignments) this.scheduler.assignmentStore?.add(cloneAssignments);
    if (delayAssignments) this.scheduler.assignmentStore?.add(delayAssignments);
    if (dependencies) this.scheduler.dependencyStore?.add(dependencies);
    this.schedulerStoresHandler.setStoresData();
    this.levelsResource = BryntumUtils.getLevelsOfNode(this.scheduler.resourceStore);
  }

  //----------------------------------Utils--------------------------------------
  private getChildren(id: string): IBryntumRecordState[] {
    return Object.values(this.recordStateMap).filter(record => record.resource.parentId === id);
  }

  private recordIsToSave(record: IBryntumRecordState): boolean {
    const initialRecord = this.initialRecordStateMap[record.id];
    return (
      !initialRecord ||
      record.isUpdated ||
      initialRecord.resource.hidden !== record.resource.hidden ||
      record.toErase ||
      initialRecord.resource.deleted !== record.resource.deleted ||
      record.event.locationId !== initialRecord.event.locationId ||
      record.resource.expanded !== initialRecord.resource.expanded ||
      record.event.hidePredecessors !== initialRecord.event.hidePredecessors ||
      record.resource.parentId !== initialRecord.resource.parentId ||
      record.resource.parentIndex !== initialRecord.resource.parentIndex ||
      record.event.displayConfig?.properties !== initialRecord.event.displayConfig?.properties
    );
  }
}
