import { AfterContentChecked, ChangeDetectorRef, Component, Input, OnInit } from '@angular/core';
import { FormBuilder, Validators } from '@angular/forms';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { IDropdownSettings } from 'ng-multiselect-dropdown';
import { IDomaine } from 'src/app/shared/model/entity/domaine.model';

@Component({
  selector: 'road-map-fields-dialog',
  templateUrl: './road-map-fields-dialog.component.html',
  styleUrls: ['../../bryntum/bryntum.scss'],
})
export class RoadMapFieldsDialogComponent implements OnInit, AfterContentChecked {
  @Input() fromRoadMap?: boolean = true;
  @Input() initialTitle: string;
  @Input() initialDomains: IDomaine[] = [];
  @Input() domains: IDomaine[] = []

  editForm = this.fb.group({
    title: [null, [Validators.required, Validators.maxLength(128)]],
    domains: [null, [Validators.required]],
  });

  selectedDomains: IDomaine[] = [];
  domainsDropdownSettings: IDropdownSettings;

  constructor(
    public activeModal: NgbActiveModal,
    private fb: FormBuilder,
    private cdRef: ChangeDetectorRef
  ) { }

  ngOnInit(): void {
    this.cdRef.detach();
    this.updateForm();
    this.initMultiDropDown();
  }

  ngAfterContentChecked(): void {
    this.cdRef.detectChanges();
  }

  updateForm(): void {
    this.editForm.patchValue({
      title: this.initialTitle,
      domains: this.initialDomains,
    });
    this.selectedDomains = this.initialDomains;
  }

  protected initMultiDropDown() {
    this.domainsDropdownSettings = {
      singleSelection: false,
      idField: 'id',
      textField: 'domaine',
      selectAllText: 'Tout sélectionner',
      unSelectAllText: 'Tout désélectionner',
      itemsShowLimit: 3,
      allowSearchFilter: true,
      searchPlaceholderText: 'Rechercher'
    };
  }

  cancel(): void {
    this.activeModal.dismiss();
  }

  save(): void {
    this.activeModal.close({
      title: this.editForm.get('title').value,
      domains: this.selectedDomains,
    });
  }

  focus(): void {
    const element = document.querySelectorAll('[aria-label="multiselect-search"]') as NodeListOf<HTMLInputElement>;
    if (element[0]) element[0].focus();
    this.editForm.get('domains').markAsTouched();
  }
}