import { SelectionModel } from '@angular/cdk/collections';
import { AfterContentChecked, ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { MatTableDataSource } from '@angular/material/table';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { DisplayRuleService } from 'src/app/bcs_ng/entity/display-rule/display-rule.service';
import { TaskableEntityService } from 'src/app/bcs_ng/entity/taskable-entity/taskable-entity.service';
import { DisplayRule } from 'src/app/bcs_ng/model/display-rule.model';
import { TaskableEntity } from 'src/app/bcs_ng/model/taskable-entity.model';
import { DisplayConfigBryntum } from '../../../bryntum/display-config';
import { SchedulerStoresHandler } from '../../../bryntum/scheduler-stores';

@Component({
  templateUrl: './apply-display-configuration-preferences-dialog.component.html',
  styleUrls: ['./apply-display-configuration-preferences-dialog.scss', '../../../bryntum/bryntum.scss', '../../display-configuration-rules/display-configuration-rules-dialog.scss'],
  providers: [DisplayConfigBryntum, SchedulerStoresHandler],
})
export class ApplyDisplayConfigurationPreferencesDialogcomponent implements OnInit, AfterContentChecked {

  myDisplayRules: DisplayRule[] = [];
  userId: string;
  dataSource = new MatTableDataSource<DisplayRule>();
  displayedColumns = ['select', 'name'];
  selection = new SelectionModel<DisplayRule>(true, []);
  taskableEntities: TaskableEntity[] = [];

  constructor(protected displayRuleService: DisplayRuleService,
    protected displayConfigBryntum: DisplayConfigBryntum,
    public taskableEntityService: TaskableEntityService,
    public activeModal: NgbActiveModal,
    private cdRef: ChangeDetectorRef) { }

  ngOnInit(): void {
    this.taskableEntityService.query().subscribe(res => this.taskableEntities = res.body || []);
    this.displayRuleService.findByOwner(this.userId).subscribe(res => {
      this.myDisplayRules = res.body || [];
      this.dataSource.data = this.myDisplayRules;
    });
  }

  ngAfterContentChecked(): void {
    this.cdRef.detectChanges();
  }

  save() {
    const rules = this.selection.selected;
    rules.forEach(r => {
      r.id = undefined;
      r.ownerId = null;
    })
    this.activeModal.close({ rules: rules });
  }

  cancel(): void {
    this.activeModal.close(null);
  }

  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.dataSource.data.length;
    return numSelected === numRows;
  }

  toggleAllRows() {
    if (this.isAllSelected()) {
      this.selection.clear();
      return;
    }
    this.selection.select(...this.dataSource.data);
  }

  getTaskableEntity(id: string) {
    return this.taskableEntities.find(t => t.id === id)?.name;
  }
}