import { SelectionModel } from '@angular/cdk/collections';
import { AfterContentChecked, ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { MatTableDataSource } from '@angular/material/table';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { DisplayRuleService } from 'src/app/bcs_ng/entity/display-rule/display-rule.service';
import { TaskableEntityService } from 'src/app/bcs_ng/entity/taskable-entity/taskable-entity.service';
import { DisplayRule } from 'src/app/bcs_ng/model/display-rule.model';
import { TaskableEntity } from 'src/app/bcs_ng/model/taskable-entity.model';
import { AccountService } from 'src/app/core/auth/account.service';
import { BaseComponent } from 'src/app/shared/base.component';

@Component({
  templateUrl: './save-display-configuration-preferences-dialog.component.html',
  styleUrls: ['../../../bryntum/bryntum.scss', '../../display-configuration-rules/display-configuration-rules-dialog.scss'],
})
export class SaveDisplayConfigurationPreferencesDialogcomponent extends BaseComponent implements OnInit, AfterContentChecked {

  rules: DisplayRule[] = [];
  taskableEntities: TaskableEntity[] = [];

  dataSource = new MatTableDataSource<DisplayRule>();
  displayedColumns: string[] = ['select', 'name'];
  selection = new SelectionModel<DisplayRule>(true, []);

  constructor(
    protected accountService: AccountService,
    public activeModal: NgbActiveModal,
    private cdRef: ChangeDetectorRef,
    public taskableEntityService: TaskableEntityService,
    public displayRuleService: DisplayRuleService
  ) { super(accountService) }

  ngOnInit(): void {
    this.dataSource.data = this.rules;
    this.taskableEntityService.query().subscribe(res => this.taskableEntities = res.body || []);
  }

  ngAfterContentChecked(): void {
    this.cdRef.detectChanges();
  }

  save() {
    let rulesToSave = this.selection.selected;
    rulesToSave.forEach(r => {
      r.ownerId = this.userIdentity.id;
      r.id = undefined;
    });
    this.displayRuleService.saveMany(rulesToSave).subscribe(res => {
      this.activeModal.close();
    })
  }

  cancel(): void {
    this.activeModal.close();
  }

  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.dataSource.data.length;
    return numSelected === numRows;
  }

  toggleAllRows() {
    if (this.isAllSelected()) {
      this.selection.clear();
      return;
    }
    this.selection.select(...this.dataSource.data);
  }

  getTaskableEntity(id: string) {
    return this.taskableEntities.find(t => t.id === id)?.name;
  }

}