import { Component, OnInit } from '@angular/core';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { Baseline } from 'src/app/bcs_ng/model/enumeration/baseline.enum';
import { IHistoric } from 'src/app/bcs_ng/model/historic.model';
import { IRoadMapBcs } from 'src/app/bcs_ng/model/road-map-bcs.model';
import { IDomaine } from 'src/app/shared/model/entity/domaine.model';
import { BcsNgHistoryService } from '../../../history/bcs-ng-history.service';

@Component({
  selector: 'road-map-bcs-domain-historic',
  templateUrl: './road-map-bcs-domain-historic.component.html',
})
export class RoadMapBcsDomainHistoricComponent implements OnInit {
  domaine: IDomaine;
  newBaselineValue: Baseline;
  roadMaps: IRoadMapBcs[] = [];
  roadMapIds: string[] = [];
  roadMapAndHistoricMap: {
    [key: string]: {
      roadMap: IRoadMapBcs,
      historic: IHistoric,
    }
  } = {};
  load: boolean = true;
  title: string = '';

  Baseline = Baseline;

  valuesAndLabels = {
    'NONE': 'Aucune baseline',
    'RPS': 'RPS passée',
    'RPS_A_VENIR': 'RPS à venir',
    'VISIBLE_CLIENT': 'Visible client',
  }

  constructor(public activeModal: NgbActiveModal, protected historicService: BcsNgHistoryService) { }

  ngOnInit(): void {
    this.title = `Validation "${this.valuesAndLabels[this.newBaselineValue]}" pour les feuilles de route du domaine ` + this.domaine?.label;
    const roadMapIds = this.roadMaps.map(rm => rm.id);
    const baselineToUpdate = this.newBaselineValue === Baseline.RPS ? Baseline.RPS_A_VENIR : undefined;
    this.historicService.findLatestByEntityForMany(roadMapIds, baselineToUpdate).subscribe(res => {
      res.body.forEach(h => {
        const roadMap = this.roadMaps.find(rm => rm.id === h.roadMap?.id);
        if (roadMap && h.baseline !== this.newBaselineValue) {
          this.roadMapIds.push(roadMap.id);
          this.roadMapAndHistoricMap[roadMap.id] = {
            roadMap,
            historic: h,
          }
        }
      });
      this.load = false;
    }, err => {
      if (err) console.error(err);
    });
  }

  save(): void {
    const historics = Object
      .values(this.roadMapAndHistoricMap)
      .map(obj => {
        return {
          ...obj.historic,
          baseline: this.newBaselineValue,
        }
      });
    this.historicService.updateMany(historics).subscribe(() => {
      this.activeModal.dismiss();
    }, err => {
      if (err) console.error(err);
    })
  }

  cancel(): void {
    this.activeModal.dismiss();
  }
}