import { AfterViewInit, Component, ElementRef, EventEmitter, OnInit, Output, ViewChild } from '@angular/core';

import { RoadMapBcsService } from '../road-map.service';
import { BryntumSchedulerComponent } from '@bryntum/scheduler-angular';
import { GeneralConfig } from '../bryntum/config';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { LocaleManager, Scheduler } from '@bryntum/scheduler';
import { IDomaine } from 'src/app/shared/model/entity/domaine.model';
import { DomaineService } from 'src/app/entities/domaine/domaine.service';
import { ActivatedRoute, Router } from '@angular/router';
import { ConvertData } from '../bryntum/convert-data';
import { DialogDeleteComponent } from 'src/app/shared/dialog/delete/dialog-delete.component';
import { DialogRebornComponent } from 'src/app/shared/dialog/reborn/dialog-reborn.component';
import { DialogEraseComponent } from 'src/app/shared/dialog/erase/dialog-erase.component';
import { Subscription } from 'rxjs';
import { JhiEventManager } from 'ng-jhipster';
import { SortData } from '../bryntum/sort-data';
import { IRoadMapBcs } from 'src/app/bcs_ng/model/road-map-bcs.model';
import { DisplayConfigBryntum } from '../bryntum/display-config';
import { BryntumUtils } from 'src/app/bcs_ng/utils/bryntumUtils';
import { localeFr } from 'src/app/bcs_ng/utils/custom.locale.Fr';
import { SchedulerStoresHandler } from '../bryntum/scheduler-stores';
import { ICruddomaine } from 'src/app/shared/model/entity/cruddomaine.model';
import { HttpResponse } from '@angular/common/http';
import { AccountService } from 'src/app/core/auth/account.service';
import { CruddomaineService } from 'src/app/entities/group-users/cruddomaine.service';
import { RoadMapBcsDTO } from 'src/app/bcs_ng/shared/dto/roadMapBcs.dto';
import { ITask } from 'src/app/bcs_ng/model/task/Itask.model';
import { Baseline } from 'src/app/bcs_ng/model/enumeration/baseline.enum';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { BcsNgHistoricUpdateComponent } from '../../history/bcs-ng-historic-update.component';
import { IDomainsConfig } from 'src/app/bcs_ng/model/bryntum/domains-config.model';
import { IDisplayConfiguration } from 'src/app/bcs_ng/model/display-configuration.model';
import { ViewConfig } from '../bryntum/view-config';
import { TaskableEntityTypeService } from 'src/app/bcs_ng/shared/services/taskable-entity-type.service';
import { DisplayConfRoadMapProperties, TimeRangeMode } from 'src/app/bcs_ng/model/bryntum/display-configuration/display-conf-road-map-properties';
import { DisplayRule, IDisplayRule } from 'src/app/bcs_ng/model/display-rule.model';
import { StorageService, StoreKey } from 'src/app/blocks/storage/storage-service';
import { routePath } from 'src/app/routes/path-routing.constant';
import { StatistiqueService } from 'src/app/admin/statistique/statistique.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { DisplayRuleService } from '../../display-rule/display-rule.service';
import { ReorderConfig } from '../bryntum/reorder-config';

@Component({
  selector: 'jhi-road-map-bcs-view',
  templateUrl: './road-map-bcs-view.component.html',
  styleUrls: ['../bryntum/bryntum.scss', '../../bcs-ng.scss'],
  providers: [GeneralConfig, ViewConfig, DisplayConfigBryntum, BryntumUtils, SchedulerStoresHandler]
})
export class RoadMapBcsViewComponent extends BaseComponent implements OnInit, AfterViewInit {
  @Output() switchMode: EventEmitter<any> = new EventEmitter<any>();
  id: string;
  roadMap?: IRoadMapBcs;
  alreadyLoadedBryntumData: {
    NONE?: IRoadMapBcs;
    RPS?: IRoadMapBcs;
    RPS_A_VENIR?: IRoadMapBcs;
    VISIBLE_CLIENT?: IRoadMapBcs;
  } = {};
  alreadyLodadedDomains: {
    NONE?: IDomainsConfig,
    RPS?: IDomainsConfig,
    RPS_A_VENIR?: IDomainsConfig,
    VISIBLE_CLIENT?: IDomainsConfig,
  } = {}

  taskConfigRulesList: DisplayRule[] = [];
  milestoneConfigRulesList: DisplayRule[] = [];
  arrowConfigRulesList: DisplayRule[] = [];

  milestones: ITask[] = [];
  domaines: IDomaine[] = [];
  domainesSelected: IDomaine[] = [];

  onLoad = false;
  isSaving = false;
  userAllowToUpdate = false;
  userAllowToDelete = false;
  userAllowToValidateRps = false;
  noRoadMap: boolean = false;
  showLegend: boolean = false;
  legendTooltip = 'Afficher la légende';

  @ViewChild('legend', { read: ElementRef }) legend: ElementRef<HTMLElement>;

  @ViewChild('scheduler') schedulerComponent!: BryntumSchedulerComponent;
  scheduler: Scheduler;
  schedulerConfig;
  schedulerViewConfig;
  resources = [];
  events = [];
  levelsResource: number[];

  resourceFontSize: number;
  eventFontSize: number;
  resourceFontColor: string = '';
  timeRangeStartDate: Date = null;
  timeRangeMode: TimeRangeMode = TimeRangeMode.NONE;
  TimeRangeMode = TimeRangeMode;

  eventSubscriber?: Subscription;

  baseline: Baseline;
  Baseline = Baseline;
  valuesAndLabels = {
    'NONE': 'Courante',
    'RPS': 'RPS passée',
    'RPS_A_VENIR': 'RPS à venir',
    'VISIBLE_CLIENT': 'Visible client'
  }

  milestoneTypeId = '';

  constructor(
    protected generalConfig: GeneralConfig,
    protected viewConfig: ViewConfig,
    protected roadMapService: RoadMapBcsService,
    protected displayConfigBryntum: DisplayConfigBryntum,
    public bryntumUtils: BryntumUtils,
    protected domaineService: DomaineService,
    protected modalService: NgbModal,
    protected activatedRoute: ActivatedRoute,
    protected eventManager: JhiEventManager,
    protected router: Router,
    protected schedulerStoresHandler: SchedulerStoresHandler,
    protected accountService: AccountService,
    protected crudDomaineService: CruddomaineService,
    protected roadMapBcsDTO: RoadMapBcsDTO,
    protected bmcsBaseService: BmcsBaseService,
    protected storageService: StorageService,
    protected statistiqueService: StatistiqueService,
    protected displayRuleService: DisplayRuleService
  ) {
    super(accountService);
  }

  ngOnInit(): void {
    this.onLoad = true;
    this.id = this.activatedRoute.snapshot.paramMap.get('id');
    this.schedulerConfig = this.generalConfig.schedulerConfig;
    this.schedulerViewConfig = this.viewConfig.schedulerConfig;
    const bl = this.storageService.get(StoreKey.BCS_BASELINE) as Baseline;
    if (bl) this.baseline = bl;
    else this.baseline = this.isUserMoss ? Baseline.NONE : Baseline.VISIBLE_CLIENT;
    this.registerChangeInRoadMap();
    LocaleManager.registerLocale('Fr', {
      desc: 'Français', locale: localeFr
    });
    LocaleManager.applyLocale('Fr');
    this.milestoneTypeId = TaskableEntityTypeService.getIdMilestone();
  }

  ngAfterViewInit(): void {
    this.scheduler = this.schedulerComponent.instance;
    this.setSchedulerForProviders();
    this.loadRoadMap();
  }

  loadRoadMap() {
    this.roadMapService.find(this.id).subscribe(async res => {
      let displayConfig;
      if (this.alreadyLoadedBryntumData[this.baseline] !== undefined) {
        this.roadMap = this.alreadyLoadedBryntumData[this.baseline] ? { ...this.alreadyLoadedBryntumData[this.baseline] } : null;
        if (this.roadMap?.displayConfig) {
          this.buildConfigRulesList(this.roadMap.displayConfig);
          this.setExportHeader();
          this.initRoadMapProperties();
        }
      } else {
        this.roadMap = await this.roadMapService.completeDisplayTasks(res.body, this.baseline);
        this.alreadyLoadedBryntumData[this.baseline] = this.roadMap ? {
          ...this.roadMap,
        } : null;
      }

      if (!this.roadMap) {
        this.noRoadMap = true;
        this.onLoad = false;
        this.taskConfigRulesList = [];
        this.milestoneConfigRulesList = [];
        this.arrowConfigRulesList = [];
        return;
      }
      this.noRoadMap = false;

      displayConfig = this.roadMap?.displayConfig;
      if (displayConfig) {
        this.buildConfigRulesList(displayConfig);
        this.setExportHeader();
        this.initRoadMapProperties();
      }

      this.loadDomaines(JSON.parse(this.roadMap.domainesId), this.baseline);
      const recordsMap = ConvertData.displayTasksToRecordsMap(SortData.sortData(this.roadMap?.tasks), true);
      const clonesMap = ConvertData.recordsToClonesMap(recordsMap);
      const dependenciesMap = ConvertData.displayTasksToDependenciesMap(this.roadMap?.tasks);
      const delaysMap = ConvertData.recordsMapToDelaysMap(recordsMap, dependenciesMap);
      const milestones = ConvertData.getMilestones(this.roadMap);
      const mainData = ConvertData.recordsToMainData(recordsMap, dependenciesMap, milestones, true);
      const cloneData = ConvertData.clonesMapToCloneEvents(clonesMap);
      const delayData = ConvertData.recordsMapToDelayEvents(recordsMap, delaysMap);

      const resources = mainData.resources;
      const events = mainData.events;
      const assignments = mainData.assignments;
      const dependencies = mainData.dependencies;
      const cloneEvents = cloneData.cloneEvents;
      const cloneAssignments = cloneData.cloneAssignments;
      const delayEvents = delayData.delayEvents;
      const delayAssignments = delayData.delayAssignments;

      this.scheduler.resourceStore?.add(resources);
      this.scheduler.resourceStore?.filter({ id: 'displayed', property: 'displayed', value: true });
      this.scheduler.resourceStore?.filter({ id: 'deleted', property: 'deleted', value: false });
      this.scheduler.eventStore?.add(events);
      this.scheduler.assignmentStore?.add(assignments);
      this.scheduler.dependencyStore?.add(dependencies);
      this.scheduler.eventStore?.add(cloneEvents);
      this.scheduler.assignmentStore?.add(cloneAssignments);
      this.scheduler.eventStore?.add(delayEvents);
      this.scheduler.assignmentStore?.add(delayAssignments);
      if (this.timeRangeMode !== TimeRangeMode.NONE) {
        const timeRange = this.bryntumUtils.buildTimeRange(this.timeRangeStartDate, this.timeRangeMode);
        this.scheduler.project?.timeRangeStore?.add(timeRange);
      }

      this.levelsResource = BryntumUtils.getLevelsOfNode(this.scheduler?.resourceStore);
      this.displayConfigBryntum.recordStateMap = { ...recordsMap };
      this.displayConfigBryntum.applyProperties(displayConfig);

      this.schedulerStoresHandler.setStoresData();
      this.onLoad = false;
    }, (err) => console.error(err))
  }

  private loadDomaines(roadMapDomainesId: string[], baseline: Baseline): void {
    if (this.alreadyLodadedDomains[baseline]) {
      const domainsConfig: IDomainsConfig = { ...this.alreadyLodadedDomains[baseline] };
      this.domaines = domainsConfig.domains;
      this.userAllowToUpdate = domainsConfig.userAllowToUpdate;
      this.userAllowToDelete = domainsConfig.userAllowToDelete;
      this.userAllowToValidateRps = domainsConfig.userAllowToValidateRps;
    } else {
      this.domaineService.queryAll().subscribe(res => {
        this.domaines = res.body.filter(d => roadMapDomainesId.includes(d.id));
        this.alreadyLodadedDomains[baseline] = {
          domains: this.domaines,
          userAllowToUpdate: false,
          userAllowToDelete: false,
          userAllowToValidateRps: false,
        }
        this.loadCrudDomaines(this.domaines, baseline);
      });
    }
  }

  private loadCrudDomaines(domaines: IDomaine[], baseline: Baseline) {
    if (this.isAdminBCS) {
      this.userAllowToUpdate = true;
      this.userAllowToDelete = true;
      this.userAllowToValidateRps = true;
      this.alreadyLodadedDomains[baseline].userAllowToUpdate = true;
      this.alreadyLodadedDomains[baseline].userAllowToDelete = true;
      this.alreadyLodadedDomains[baseline].userAllowToValidateRps = true;
    } else {
      this.crudDomaineService.queryMyne({ where: this.accountService.userGroup }).subscribe((res: HttpResponse<[ICruddomaine[]]>) => {
        const crud = this.roadMapBcsDTO.getCrudDomaine(res.body || [[]]);
        let crudDomainesRm = [];
        domaines?.forEach(domaine =>
          crudDomainesRm.push(crud[crud.findIndex(cr => cr.domaine?.id === domaine.id)])
        );
        if (crudDomainesRm?.some(crudDomaine => crudDomaine?.update)) this.userAllowToUpdate = true;
        if (crudDomainesRm?.some(crudDomaine => crudDomaine?.deleted)) this.userAllowToDelete = true;
        if (crudDomainesRm?.some(crudDomaine => crudDomaine?.is_referent)) this.userAllowToValidateRps = true;
        this.alreadyLodadedDomains[baseline].userAllowToUpdate = this.userAllowToUpdate;
        this.alreadyLodadedDomains[baseline].userAllowToDelete = this.userAllowToDelete
        this.alreadyLodadedDomains[baseline].userAllowToValidateRps = this.userAllowToValidateRps;
      });
    }
  }

  private setSchedulerForProviders(): void {
    this.bryntumUtils.scheduler = this.scheduler;
    this.displayConfigBryntum.scheduler = this.scheduler;
    this.schedulerStoresHandler.scheduler = this.scheduler;
  }

  private initRoadMapProperties(): void {
    const properties: DisplayConfRoadMapProperties = JSON.parse(this.roadMap.displayConfig.properties);
    this.resourceFontSize = properties.resourceFontSize ?? 16;
    this.resourceFontColor = properties.resourceFontColor ?? '#474444';
    this.eventFontSize = properties.eventFontSize ?? 16;
    this.timeRangeStartDate = properties.timeRangeStartDate;
    this.timeRangeMode = properties.timeRangeMode;
  }

  protected onSaveSuccess(res): void {
    this.isSaving = false;
  }

  protected onSaveError(err): void {
    this.isSaving = false;
    console.error(err);
  }

  registerChangeInRoadMap(): void {
    this.eventSubscriber = this.eventManager.subscribe('roadMapBcsListModification', () => {
      this.roadMapService.find(this.id).subscribe(res => { this.roadMap = res.body; });
    });
  }

  defineParamsDialog(modalRef, id) {
    modalRef.componentInstance.id = id;
    modalRef.componentInstance.entityMsg = "cette feuille de route";
    modalRef.componentInstance.serviceEntity = this.roadMapService;
    modalRef.componentInstance.eventManagerName = "roadMapBcsListModification";
  }

  delete(roadMap: IRoadMapBcs): void {
    const modalRef = this.modalService.open(DialogDeleteComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, roadMap.id);
  }

  reborn(roadMap: IRoadMapBcs): void {
    const modalRef = this.modalService.open(DialogRebornComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, roadMap.id);
  }

  erase(roadMap: IRoadMapBcs): void {
    const modalRef = this.modalService.open(DialogEraseComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, roadMap.id);
    modalRef.result.then(() => {
      this.router.navigate([routePath.BCS_ROADMAP]);
    }).catch(err => console.error(err));
  }

  navigateToEdit(): void {
    this.switchMode.emit();
  }

  exportPdf(): void {
    const date = new Date();
    const dateString = date.toLocaleDateString();
    this.scheduler.features.pdfExport.setConfig({
      footerTpl: () => `<div style="width: 100%; text-align: end; margin-top: 4px;">${dateString}</div>`
    });
    this.scheduler.features.pdfExport.showExportDialog();
  }

  private setExportHeader(): void {
    const legendElement = this.legend.nativeElement;
    this.scheduler.features.pdfExport.setConfig({
      headerTpl: () => `
        <div style="width: 100%; display: flex; flex-direction: column;">
          <div>
            <div style="width: 100%; display: flex; justify-content: center; height: 40px; margin-top: 4px;">
              <img src="../../../../../assets/images/diffusion_restreinte_long.png" style="max-height: 100%;" />
            </div>
            <h1 style="width: 100%; padding: 0px; margin: 15px 0 0 0;">${this.roadMap?.name}</h1>
          </div>
          <div style="width: 100%;">${legendElement.innerHTML}</div>
        </div>
      `
    });
  }

  exportExcel(): void {
    this.scheduler.features.excelExporter.export({});
  }

  changeBaseline(baseline: Baseline): void {
    this.baseline = baseline;
    this.scheduler.resourceStore.removeAll();
    this.scheduler.eventStore.removeAll();
    this.scheduler.dependencyStore.removeAll();
    this.scheduler.assignmentStore.removeAll();
    this.onLoad = true;
    this.loadRoadMap();
  }

  validateRoadMap(newBaselineValue: Baseline): void {
    const modalRef = this.modalService.open(BcsNgHistoricUpdateComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.fromHistoric = false;
    modalRef.componentInstance.id = this.id;
    modalRef.componentInstance.newBaselineValue = newBaselineValue;
  }

  showOrHideLegend(): void {
    this.legendTooltip = this.showLegend === true ? 'Afficher la légende' : 'Cacher la légende'
    this.showLegend = !this.showLegend;
  }

  buildConfigRulesList(displayConfig: IDisplayConfiguration): void {
    this.taskConfigRulesList = [];
    this.milestoneConfigRulesList = [];
    this.arrowConfigRulesList = [];
    this.displayRuleService.findCharter().subscribe((res: HttpResponse<IDisplayRule[]>) => {
      const rules = (res?.body || []).concat(displayConfig.displayRules);
      rules?.forEach(r => {
        if (r.multiColorsEventConfigsRules && Object.keys(r.multiColorsEventConfigsRules).length > 0) this.taskConfigRulesList.push(r);
        else if (TaskableEntityTypeService.isMilestone(r.taskableEntityId)) this.milestoneConfigRulesList.push(r);
        else this.taskConfigRulesList.push(r);
      });
    });

    // Not yet configurable
    this.arrowConfigRulesList.push({
      name: 'Décalage contractuel',
      bgColor: '#30B1EE',
    } as any);
    this.arrowConfigRulesList.push({
      name: 'Décalage RPS',
      bgColor: 'red',
    } as any);
  }

  setTimeCursor(timeRangeMode: TimeRangeMode, event?: any): void {
    if (timeRangeMode !== TimeRangeMode.NONE) {
      let startDate = event ?? new Date();
      if (this.scheduler.project.timeRangeStore.allRecords?.length) {
        this.scheduler.project.timeRangeStore.allRecords[0].set('startDate', startDate);
        this.scheduler.project.timeRangeStore.allRecords[0].set('name', startDate.toLocaleDateString());
      } else {
        this.scheduler.project.timeRangeStore.add({
          startDate,
          name: startDate.toLocaleDateString(),
        })
      }
    } else {
      this.scheduler.project.timeRangeStore.removeAll();
    }
    this.timeRangeMode = timeRangeMode;
  }

  increaseResourceFontSize(): void {
    this.resourceFontSize += 1;
    this.displayConfigBryntum.applyTextFormatting(this.resourceFontSize, this.resourceFontColor);
  }

  reduceResourceFontSize(): void {
    if (this.resourceFontSize > 5) this.resourceFontSize -= 1;
    this.displayConfigBryntum.applyTextFormatting(this.resourceFontSize, this.resourceFontColor);
  }

  increaseEventFontSize(): void {
    this.eventFontSize += 1;
    this.displayConfigBryntum.applyEventTextFormatting(this.eventFontSize);
  }

  reduceEventFontSize(): void {
    if (this.eventFontSize > 5) this.eventFontSize -= 1;
    this.displayConfigBryntum.applyEventTextFormatting(this.eventFontSize);
  }
}
