import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';

import { NgbActiveModal, NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';
import { AccountService } from 'src/app/core/auth/account.service';
import { DateAdapter } from '@angular/material/core';
import * as moment from 'moment';
import { IGenericTask } from 'src/app/bcs_ng/model/task/generic-task.model';
import { GenericTaskService } from '../generic-task.service';

@Component({
  selector: 'jhi-generic-task-update-dialog',
  templateUrl: './generic-task-update-dialog.component.html'
})
export class GenericTaskUpdateDialogComponent implements OnInit {
  genericTask?: IGenericTask;
  showForm: boolean = false;

  constructor(
    protected genericTaskService: GenericTaskService,
    protected activatedRoute: ActivatedRoute,
    public activeModal: NgbActiveModal,
    protected modalService: NgbModal,
    protected eventManager: JhiEventManager,
    protected accountService: AccountService,
    private dateAdapter: DateAdapter<Date>
  ) {
    this.dateAdapter.setLocale('fr');
  }

  ngOnInit(): void {
    this.loadPage();
  }

  loadPage(): void {
    if (this.genericTask?.id) {
      this.genericTaskService
        .find(this.genericTask?.id)
        .subscribe(
          (res: HttpResponse<IGenericTask>) => {
            this.genericTask = res.body;
            this.blockUpdate();
          },
        );
    } else {
      this.showForm = true;
    }
  }

  private blockUpdate() {
    if ((!this.genericTask?.modifyBlockingDate || moment().diff(this.genericTask?.modifyBlockingDate) / 1000 > 901) ||
      this.accountService.userIdentity?.login === this.genericTask?.modifyBlockingBy) {
      this.genericTaskService.blockUpdate(this.genericTask).subscribe((res) => {
        this.showForm = true;
        this.genericTask = res.body;
      });
    } else {
      this.showForm = false;
    }
  }

  cancel(): void {
    if (this.accountService.userIdentity?.login === this.genericTask?.modifyBlockingBy) this.genericTaskService.freeUpdate(this.genericTask).subscribe((res) => { });
    this.activeModal.dismiss();
  }
}
