import { KeyValue } from '@angular/common';
import { HttpResponse } from '@angular/common/http';
import { Component, EventEmitter, Input, OnDestroy, OnInit, Output, ViewChild } from '@angular/core';
import { MatPaginator, PageEvent } from '@angular/material/paginator';
import { MatSort, Sort } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { MatTableFilter } from 'mat-table-filter';
import * as moment from 'moment';
import { JhiEventManager } from 'ng-jhipster';
import { Subscription } from 'rxjs';
import { FilterKey, StorageService } from 'src/app/blocks/storage/storage-service';
import { AccountService } from 'src/app/core/auth/account.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { IGenericTask } from 'src/app/bcs_ng/model/task/generic-task.model';
import { ExcelService } from 'src/app/shared/util/excel.service';
import { FilterWithAccents } from 'src/app/shared/util/filter-with-accents';
import { GenericTaskUpdateDialogComponent } from './dialog/generic-task-update-dialog.component';
import { GenericTaskService } from './generic-task.service';
import { GenericTaskCrud, IGenericTaskCrud } from 'src/app/bcs_ng/shared/dto/model/generic-task-crud.dto';
import { GenericTaskDto } from 'src/app/bcs_ng/shared/dto/generic-task.dto';
import { SelectionModel } from '@angular/cdk/collections';
import { ITask } from 'src/app/bcs_ng/model/task/Itask.model';
import { DialogDeleteBcsComponent } from 'src/app/bcs_ng/shared/dialog/delete/dialog-delete-bcs.component';
import { DialogRebornBcsComponent } from 'src/app/bcs_ng/shared/dialog/reborn/dialog-reborn-bcs.component';
import { DialogEraseBcsComponent } from 'src/app/bcs_ng/shared/dialog/erase/dialog-erase-bcs.component';

@Component({
  selector: 'jhi-generic-task',
  templateUrl: './generic-task.component.html'
})
export class GenericTaskComponent extends BaseComponent implements OnInit, OnDestroy {
  @Input() showButtons: boolean = true;
  @Input() sizeMatPaginator?: number;
  @Input() readOnly?: boolean = false;
  @Input() onlyActive?: boolean = false;

  // Data to fetch
  genericTasks?: IGenericTask[];

  // Page
  load: boolean = true;
  eventSubscriber?: Subscription;

  // Mat-table
  dataSource = new MatTableDataSource<IGenericTaskCrud>([]);
  data: IGenericTaskCrud[] = [];
  @ViewChild(MatSort) matSorter!: MatSort;
  activeSort: Sort = { active: 'id', direction: 'asc' };
  @ViewChild(MatPaginator, { static: true }) paginator!: MatPaginator;
  pageSize = 50;
  allColumnsGenericTasks = new Map([
    ['name', 'Nom'],
    ['shortName', 'Abréviation'],
    ['startDate', 'Date de début'],
    ['endDate', 'Date de fin'],
    ['comment', 'Commentaire'],
    ['details', 'Détails'],
  ]);
  displayedColumnsGenericTasks: string[];
  public tempDisplayedCol: string[];
  public displayedSearchColumns: string[];
  filterEntity: IGenericTaskCrud = new GenericTaskCrud();
  filterType: MatTableFilter;
  propertyOptions: any;

  @Input() defaultDisplayColumns: string[]
    = ['name', 'shortName', 'startDate', 'endDate', 'comment'];
  defaultDisplayColumnsWithDetails: string[] = this.defaultDisplayColumns.concat(['details']);
  defaultSearchColumns: string[] = this.defaultDisplayColumns.map(c => `search-${c}`);
  defaultSearchColumnsWithDetails: string[] = this.defaultDisplayColumnsWithDetails.map(c => `search-${c}`);

  constructor(
    protected genericTaskService: GenericTaskService,
    protected accountService: AccountService,
    protected genericTaskDto: GenericTaskDto,
    protected storageService: StorageService,
    protected eventManager: JhiEventManager,
    protected modalService: NgbModal,
    protected filterWithAccents: FilterWithAccents,
    protected excelService: ExcelService,
  ) {
    super(accountService);
  }

  //----------------------------------------------------------------------------------------------------------------------
  //---                                            Angular component lifecycle hooks                                                  --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'lifecycle'

  ngOnInit(): void {
    if (this.sizeMatPaginator) this.pageSize = this.sizeMatPaginator;
    else this.pageSize = Number(this.storageService.get(FilterKey.NB_ITEMS)) || this.userPreferences?.nbItemTable;
    this.loadAll();
    this.initFilterByColumn();
    this.registerChangeInGenericTasks();
  }

  ngOnChanges(): void {
    this.updateGenericTasks();
  }

  ngOnDestroy(): void {
    if (this.eventSubscriber)
      this.eventManager.destroy(this.eventSubscriber);
  }

  //#endregion

  //----------------------------------------------------------------------------------------------------------------------
  //---                                            LOAD & UPDATE DATA                                                  --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'loadData'

  loadAll(): void {
    const getAll = this.onlyActive ? false : this.isAdminBCS;
    this.genericTaskService.query(getAll).subscribe((res: HttpResponse<IGenericTask[]>) => {
      this.genericTasks = res.body || [];
      this.updateGenericTasks();
    });
  }

  protected updateTable(): void {
    this.dataSource.data = this.data || [];
    this.dataSource.sort = this.matSorter;
    if (!this.isScully) {
      this.paginator.pageSize = this.pageSize;
      this.dataSource.paginator = this.paginator;
      this.dataSource.paginator._intl.itemsPerPageLabel = "Items par page :";
    }
    this.load = false;
  }

  protected async updateGenericTasks(): Promise<void> {
    this.data = this.genericTaskDto.convertToCrud(this.genericTasks);
    this.updateTable();
  }

  //#endregion
  //----------------------------------------------------------------------------------------------------------------------
  //---                                               USER MODIFICATIONS/ACTIONS                                                --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'modification'

  exportGenericTasks(): void {
    const filteredGenericTasks = this.genericTaskDto.convertFromCrud(this.dataSource.filteredData);
    this.excelService.exportAsExcelFile(this.genericTaskDto.convertGenericTasksDataToExcel(filteredGenericTasks), 'GenericTasks-SCCOA');
  }

  defineParamsDialog(modalRef, id) {
    modalRef.componentInstance.id = id;
    modalRef.componentInstance.entityMsg = "cette tâche";
    modalRef.componentInstance.serviceEntity = this.genericTaskService;
    modalRef.componentInstance.eventManagerName = "genericTaskListModification";
  }

  trackId(index: number, item: IGenericTask): string {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
    return item.id!;
  }

  registerChangeInGenericTasks(): void {
    this.eventSubscriber = this.eventManager.subscribe('genericTaskListModification', () => this.loadAll());
  }

  update(genericTask: IGenericTask | undefined): void {
    const modalRef = this.modalService.open(GenericTaskUpdateDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.genericTask = genericTask;
  }

  delete(genericTask: IGenericTask): void {
    const modalRef = this.modalService.open(DialogDeleteBcsComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, genericTask.id);
  }

  reborn(genericTask: IGenericTask): void {
    const modalRef = this.modalService.open(DialogRebornBcsComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, genericTask.id);
  }

  erase(genericTask: IGenericTask): void {
    const modalRef = this.modalService.open(DialogEraseBcsComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, genericTask.id);
  }

  //#endregion
  //----------------------------------------------------------------------------------------------------------------------
  //---                                             FILTER ON TABLE                                                     --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'Filter'

  initFilterByColumn() {
    this.initAllFilters();
    //Récupère le type de filtre, ici anywhere donc on va garder les données contenant le filtre demandé
    this.filterType = MatTableFilter.ANYWHERE;
    this.setFilterOnInit();
    this.propertyOptions = {
      globalFilter: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.globalFilter);
      },
      name: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.name);
      },
      shortName: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.shortName);
      },
      comment: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.comment);
      },
      deleted: (route: boolean) => {
        return (this.filterEntity.deleted && route) || !this.filterEntity.deleted;
      }
    }
  }

  setFilterOnInit() {
    if (this.showButtons) {
      //get local storage filters
      const filters = this.storageService.get(FilterKey.GTASK_FILTER);
      if (filters) {
        //S'il y a des filtres enregistrés, on les assignes à la liste des filtres par colonnes
        this.filterEntity = filters as IGenericTaskCrud;
      }
      const sort = this.storageService.get(FilterKey.GTASK_SORT);
      if (sort) this.activeSort = sort as Sort;
    }

    const columns = this.storageService.get(FilterKey.GTASK_COLUMNS);
    if (columns && !this.readOnly) {
      //On assigne la liste des colonnes affichées sauvegardées au tableau des colonnes qui doivent être affichées
      this.displayedColumnsGenericTasks = columns as string[];
      this.tempDisplayedCol = columns as string[];
      //Donne la liste des actions correspondantes et la sauvegarde
      this.changeColumns();
    } else if (this.isScully) {
      this.displayedColumnsGenericTasks = this.defaultDisplayColumns;
      this.tempDisplayedCol = this.defaultDisplayColumns;
      this.displayedSearchColumns = this.defaultSearchColumns;
    } else {
      //S'il n'y a pas de colonnes sauvegardées, on affiche les colonnes préférentielles de l'utilisateur
      const colsPreferences = this.userPreferences?.genericTaskColumns;
      if (colsPreferences && !this.readOnly) {
        this.displayedColumnsGenericTasks = colsPreferences;
        this.tempDisplayedCol = colsPreferences;
        this.changeColumns();
      }
      else {
        if (this.readOnly) {
          this.displayedColumnsGenericTasks = this.defaultDisplayColumns;
          this.displayedSearchColumns = this.defaultDisplayColumns.map(c => `search-${c}`);
        }
        else {
          //S'il n'y a pas de colonnes sauvegardées et que l'utilisateur n'a pas de préférence, on affiche ces colonnes et leur jalons correspondantes
          this.displayedColumnsGenericTasks = this.defaultDisplayColumnsWithDetails;
          this.tempDisplayedCol = this.defaultDisplayColumnsWithDetails;
          this.displayedSearchColumns = this.defaultSearchColumnsWithDetails;
        }
      }
    }
  }

  changeColumns() {
    //On assigne les filtres des colonnes qui doivent être affichées dans un tableau
    this.displayedSearchColumns = this.displayedColumnsGenericTasks.map(col => 'search-' + col);
    //On sauvegarde ce tableau
    this.storageService.set(FilterKey.GTASK_COLUMNS, this.displayedColumnsGenericTasks);
  }

  asOrderCols(a: KeyValue<string, any>, b: KeyValue<string, any>): number {
    const aStepId = parseInt(a.key, 10);
    const bStepId = parseInt(b.key, 10);
    return aStepId > bStepId ? 1 : (bStepId > aStepId ? -1 : 0);
  }

  openedChange(opened: boolean) {
    if (!opened) {
      this.displayedColumnsGenericTasks = this.tempDisplayedCol;
      this.changeColumns();
    }
  }

  deleteAllFilter() {
    this.initAllFilters();
    this.storeFilters();
  }

  initAllFilters() {
    this.filterEntity = new GenericTaskCrud();
    this.filterEntity.deleted = false;
  }

  changefilter() {
    this.storeFilters();
  }

  //Retourne si le filtre envoyé en paramètre doit être activé ou non
  activeFilter(filterEntityData): boolean {
    //S'il n'est pas déjà activé, on retourne true, sino on retourne false
    return filterEntityData === undefined || filterEntityData === null || filterEntityData === '' || filterEntityData?.length === 0
      ? false : true;
  }

  isAllFilterNull(): boolean {
    return (
      !this.activeFilter(this.filterEntity.name) &&
      !this.activeFilter(this.filterEntity.shortName) &&
      !this.activeFilter(this.filterEntity.startDate) &&
      !this.activeFilter(this.filterEntity.endDate) &&
      !this.activeFilter(this.filterEntity.comment)
    ) ? true : false;
  }

  updateFilterChangeCols(data: any) {
    let lastSelectedValue = data.value;
    const isChecked = data.selected;
    if (!isChecked) {
      switch (lastSelectedValue) {
        case 'startDate':
          this.filterEntity.startDate = undefined;
          break;
        case 'endDate':
          this.filterEntity.endDate = undefined;
          break;
      }
    }
  }

  //Fonction appelée lorsque l'on va utiliser un filtre sur les dates
  updatePropertyOptions(options, colName) {
    this.propertyOptions[colName] = options;
  }
  updateFilterEntity(newFilter, colName) {
    this.filterEntity[colName] = newFilter ? moment(newFilter) : undefined;
  }

  //----------------------------------------------------------------------------------------------------------------------
  //---                                            SAVE LOCAL STORAGE                                                   --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'LocalStorage'

  storeFilters(): void {
    if (this.showButtons) {
      //Si tous les filtres ne sont pas désactivés, alors on remplace les filtres sauvegardés précédemment par la liste des filtres actuellements utilisés
      if (!this.isAllFilterNull()) this.storageService.set(FilterKey.GTASK_FILTER, this.filterEntity);
      //Sinon, on supprime la liste des filtres sauvegardées
      else this.storageService.remove(FilterKey.GTASK_FILTER);
    }
  }

  storeSort(sort: Sort): void {
    this.activeSort = sort;
    if (this.showButtons) {
      if (sort.active !== '' && sort.direction !== '') this.storageService.set(FilterKey.GTASK_SORT, sort);
      else this.storageService.remove(FilterKey.GTASK_SORT);
    }
  }

  getPaginatorChange(event: PageEvent): PageEvent {
    this.storageService.set(FilterKey.NB_ITEMS, event.pageSize);
    return event;
  }
  //#endregion
  //----------------------------------------------------------------------------------------------------------------------
  //---                                            SELECT ITEM                                                   --        
  //----------------------------------------------------------------------------------------------------------------------

  selection = new SelectionModel<IGenericTaskCrud>(true, []);
  @Output() onChangeSelectItem = new EventEmitter<ITask[]>();

  onChangeSelection(row) {
    this.selection.toggle(row);
    this.onChangeSelectItem.emit(this.selection.selected.map(t => t.genericTask));
  }
  /** Whether the number of selected elements matches the total number of rows. */
  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.dataSource.filteredData.length;
    return numSelected === numRows;
  }

  /** Selects all rows if they are not all selected; otherwise clear selection. */
  toggleAllRows() {
    if (this.isAllSelected()) {
      this.selection.clear();
      this.onChangeSelectItem.emit(this.selection.selected.map(t => t.genericTask));
      return;
    }
    this.selection.select(...this.dataSource.filteredData);
    this.onChangeSelectItem.emit(this.selection.selected.map(t => t.genericTask));
  }
}