import { HttpResponse } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { ActivatedRouteSnapshot, Resolve, Router, Routes } from '@angular/router';
import { EMPTY, Observable, of } from 'rxjs';
import { flatMap } from 'rxjs/operators';
import { UserRouteAccessService } from 'src/app/core/auth/user-route-access-service';
import { routePath } from 'src/app/routes/path-routing.constant';
import { AuthorityName } from 'src/app/shared/constants/authority.constants';
import { GenericTask, IGenericTask } from 'src/app/bcs_ng/model/task/generic-task.model';
import { GenericTaskDetailComponent } from './generic-task-detail.component';
import { GenericTaskComponent } from './generic-task.component';
import { GenericTaskService } from './generic-task.service';

@Injectable({ providedIn: 'root' })
export class GenericTaskResolve implements Resolve<IGenericTask> {
  constructor(private service: GenericTaskService, private router: Router) { }

  resolve(route: ActivatedRouteSnapshot): Observable<IGenericTask> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        flatMap((genericTask: HttpResponse<GenericTask>) => {
          if (genericTask.body) {
            return of(genericTask.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new GenericTask());
  }
}

export const genericTaskRoute: Routes = [
  {
    path: routePath.GENERIC_TASK,
    component: GenericTaskComponent,
    data: {
      authorities: [AuthorityName.USER_MOSS_SCCOA, AuthorityName.ADMIN, AuthorityName.USER_DGA_MQP],
      pageTitle: 'Tâches BCS'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: routePath.GENERIC_TASK + '/:id/view',
    component: GenericTaskDetailComponent,
    resolve: {
      genericTask: GenericTaskResolve
    },
    data: {
      authorities: [AuthorityName.USER_MOSS_SCCOA, AuthorityName.ADMIN, AuthorityName.USER_DGA_MQP],
      pageTitle: 'Tâche BCS'
    },
    canActivate: [UserRouteAccessService]
  },
];
