import { Component, Input } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';
import { AccountService } from 'src/app/core/auth/account.service';
import { StatutHistory } from 'src/app/shared/entity/history/detail/statutHistory';
import { IHistoric } from 'src/app/bcs_ng/model/historic.model';
import { IMilestone } from 'src/app/bcs_ng/model/milestone.model';
import { EncoderService } from 'src/app/shared/util/encoder-util';

@Component({
  selector: 'milestone-diff-card',
  templateUrl: './milestone-diff-card.component.html',
})
export class MilestoneDiffCardComponent {
  @Input() history: IHistoric;
  @Input() historyToCompare: IHistoric;
  milestone: IMilestone;
  milestoneToCompare: IMilestone;
  CompareNowOrOlder: boolean = true;
  name: string;
  shortName: string;
  comments: string;
  typeMilestone: string;
  source: string;
  predecessorsToCompare: Map<StatutHistory, string[]> = new Map<StatutHistory, string[]>();

  constructor(
    protected eventManager: JhiEventManager,
    protected activatedRoute: ActivatedRoute,
    protected modalService: NgbModal,
    protected accountService: AccountService,
    protected encoderService: EncoderService
  ) { }

  ngOnChanges(): void {
    this.loadMilestones();
  }

  ngOnInit(): void {
    this.loadMilestones();
  }

  loadMilestones(): void {
    if (this.history?.object) {
      this.milestone = JSON.parse(this.history?.object) as IMilestone
    } else {
      this.milestone = undefined;
    }
    if (this.historyToCompare?.object) {
      this.milestoneToCompare = JSON.parse(this.historyToCompare?.object) as IMilestone;
    } else {
      this.milestoneToCompare = undefined;
    }
    this.includePredecessors();
    if (this.milestone && this.milestoneToCompare) {
      this.name = this.encoderService.compareStringDifference(this.milestone.name, this.milestoneToCompare.name);
      this.shortName = this.encoderService.compareStringDifference(this.milestone.shortName, this.milestoneToCompare.shortName);
      this.comments = this.encoderService.compareStringDifference(this.milestone.comment, this.milestoneToCompare.comment);
      this.typeMilestone = this.encoderService.compareStringDifference(this.milestone?.typeMilestone?.name, this.milestoneToCompare?.typeMilestone?.name);
      this.source = this.encoderService.compareStringDifference(this.milestone?.source?.name, this.milestoneToCompare?.source?.name);
    }
  }

  includePredecessors(): void {
    const predecessorsAdd: string[] = [];
    const predecessorsLess: string[] = [];
    const predecesorsEqual: string[] = [];
    // this.milestoneToCompare?.predecessors?.forEach(predToCompare => {
    //   const index = this.milestone?.predecessors.findIndex(pred => predToCompare.id === pred.id)
    //   if (index === -1) predecessorsLess.push(predToCompare.name)
    //   else if (this.milestone?.predecessors[index]) predecesorsEqual.push(this.milestone.predecessors[index].name)
    // })
    // this.milestone?.predecessors?.forEach(pred => {
    //   const index = this.milestoneToCompare?.predecessors.findIndex(predToCompare => predToCompare.id === pred.id)
    //   if (index === -1) predecessorsAdd.push(pred.name)
    // });
    this.predecessorsToCompare.set(StatutHistory.ADD, predecessorsAdd)
    this.predecessorsToCompare.set(StatutHistory.LESS, predecessorsLess)
    this.predecessorsToCompare.set(StatutHistory.EQUALS, predecesorsEqual)
  }

  compareChange(): void {
    this.CompareNowOrOlder = !this.CompareNowOrOlder
  }
}