import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { ActivatedRoute } from '@angular/router';

import { MilestoneService } from '../milestone.service';
import { IMilestone } from 'src/app/bcs_ng/model/milestone.model';
import { NgbActiveModal, NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';
import { AccountService } from 'src/app/core/auth/account.service';
import { DateAdapter } from '@angular/material/core';
import * as moment from 'moment';
import { IOperation } from 'src/app/bcs_ng/model/operation.model';

@Component({
  selector: 'jhi-milestone-update-dialog',
  templateUrl: './milestone-update-dialog.component.html'
})
export class MilestoneUpdateDialogComponent implements OnInit {
  milestone?: IMilestone;
  showForm: boolean = false;
  operationFromDetail: IOperation;

  constructor(
    protected milestoneService: MilestoneService,
    protected activatedRoute: ActivatedRoute,
    public activeModal: NgbActiveModal,
    protected modalService: NgbModal,
    protected eventManager: JhiEventManager,
    protected accountService: AccountService,
    private dateAdapter: DateAdapter<Date>
  ) {
    this.dateAdapter.setLocale('fr');
  }

  ngOnInit(): void {
    this.loadPage();
  }

  loadPage(): void {
    if (this.milestone?.id) {
      this.milestoneService
        .find(this.milestone?.id)
        .subscribe(
          (res: HttpResponse<IMilestone>) => {
            this.milestone = res.body;
            this.blockUpdate();
          },
        );
    } else {
      this.showForm = true;
    }
  }

  private blockUpdate() {
    if ((!this.milestone?.modifyBlockingDate || moment().diff(this.milestone?.modifyBlockingDate) / 1000 > 901) ||
      this.accountService.userIdentity?.login === this.milestone?.modifyBlockingBy) {
      this.milestoneService.blockUpdate(this.milestone).subscribe((res) => {
        this.showForm = true;
        this.milestone = res.body;
      });
    } else {
      this.showForm = false;
    }
  }

  cancel(): void {
    if (this.accountService.userIdentity?.login === this.milestone?.modifyBlockingBy) this.milestoneService.freeUpdate(this.milestone).subscribe((res) => { });
    this.activeModal.dismiss();
  }
}
