import { Component, OnInit, Input, ViewChild } from '@angular/core';
import { KeyValue } from '@angular/common';
import { MatSort, Sort } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';
import { MatPaginator } from '@angular/material/paginator';

import { JhiEventManager } from 'ng-jhipster';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import * as moment from 'moment';
import { MatTableFilter } from 'mat-table-filter';

import { MilestoneService } from './milestone.service';
import { IMilestone, Milestone } from 'src/app/bcs_ng/model/milestone.model';
import { StorageService } from 'src/app/blocks/storage/storage-service';
import { AccountService } from 'src/app/core/auth/account.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { FilterWithAccents } from 'src/app/shared/util/filter-with-accents';
import { ExcelService } from 'src/app/shared/util/excel.service';
import { MilestoneDTO } from 'src/app/bcs_ng/shared/dto/milestone.dto';
import { IBcsNgListOption } from 'src/app/bcs_ng/model/bcs-ng-list-option.model';
import { BcsNgListOptionService } from '../../bcs-ng-list-option/bcs-ng-list-option.service';


@Component({
  selector: 'jhi-milestone-shared-table',
  templateUrl: './milestone-shared-table.component.html',
  styleUrls: ['../table-bcs-ng.scss'],
})
export class MilestoneSharedTableComponent extends BaseComponent implements OnInit {
  // Data to fetch
  @Input() milestones: IMilestone[] = [];
  typeValues: IBcsNgListOption[] = [];
  sourceValues: IBcsNgListOption[] = [];

  // Page
  load: boolean = true;

  // Mat-table
  dataSource = new MatTableDataSource<IMilestone>([]);
  @ViewChild(MatSort) matSorter!: MatSort;
  activeSort: Sort = { active: 'id', direction: 'asc' };
  @ViewChild(MatPaginator, { static: true }) paginator!: MatPaginator;

  displayedColumnsMilestone: string[] = ['name', 'shortName', 'type', 'source', 'startDate', 'predecessorDependencies'];
  displayedSearchColumns: string[];
  filterEntity: IMilestone = new Milestone();
  filterType: MatTableFilter;
  propertyOptions: any;
  filterInputContractDate: string = '';

  constructor(
    protected milestoneService: MilestoneService,
    protected bcsNgListOptionService: BcsNgListOptionService,
    protected accountService: AccountService,
    protected milestoneDto: MilestoneDTO,
    protected storageService: StorageService,
    protected eventManager: JhiEventManager,
    protected modalService: NgbModal,
    protected filterWithAccents: FilterWithAccents,
    protected excelService: ExcelService,
  ) {
    super(accountService);
  }

  ngOnInit(): void {
    this.initFilterByColumn();
    this.bcsNgListOptionService.query().subscribe(res => {
      this.typeValues = res.body.filter(optionList => optionList.type === 'MILESTONE_TYPE');
      this.sourceValues = res.body.filter(optionList => optionList.type === 'MILESTONE_SOURCE');
    });
    this.displayedSearchColumns = this.displayedColumnsMilestone.map(col => 'search-' + col);
    this.dataSource.data = this.milestones;
    this.dataSource.paginator = this.paginator;
    this.load = false;
  }


  //----------------------------------------------------------------------------------------------------------------------
  //---                                             FILTER ON TABLE                                                     --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'Filter'
  initFilterByColumn() {
    this.initAllFilters();
    //Récupère le type de filtre, ici anywhere donc on va garder les données contenant le filtre demandé
    this.filterType = MatTableFilter.ANYWHERE;
    this.propertyOptions = {
      name: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.name);
      },
      shortName: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.shortName);
      },
      typeMilestone: (route: IBcsNgListOption) => {
        return this.filterWithAccents.compareStrings(route?.name, this.filterEntity.typeMilestone?.toString());
      },
      source: (route: IBcsNgListOption) => {
        return this.filterWithAccents.compareStrings(route?.name, this.filterEntity.source?.toString());
      },
      deleted: (route: boolean) => {
        return (this.filterEntity.deleted && route) || !this.filterEntity.deleted;
      }
    }
  }

  asOrderCols(a: KeyValue<string, any>, b: KeyValue<string, any>): number {
    const aStepId = parseInt(a.key, 10);
    const bStepId = parseInt(b.key, 10);
    return aStepId > bStepId ? 1 : (bStepId > aStepId ? -1 : 0);
  }

  deleteAllFilter() {
    this.initAllFilters();
  }

  initAllFilters() {
    this.filterEntity = new Milestone();
    this.filterEntity.deleted = false;
  }

  //Retourne si le filtre envoyé en paramètre doit être activé ou non
  activeFilter(filterEntityData): boolean {
    //S'il n'est pas déjà activé, on retourne true, sino on retourne false
    return filterEntityData === undefined || filterEntityData === null || filterEntityData === '' || filterEntityData?.length === 0
      ? false : true;
  }

  isAllFilterNull(): boolean {
    return (
      !this.activeFilter(this.filterEntity.name) &&
      !this.activeFilter(this.filterEntity.shortName) &&
      !this.activeFilter(this.filterEntity.comment) &&
      !this.activeFilter(this.filterEntity.contractDate) &&
      !this.activeFilter(this.filterEntity.typeMilestone) &&
      !this.activeFilter(this.filterEntity.source)
    ) ? true : false;
  }

  updateFilterChangeCols(data: any) {
    let lastSelectedValue = data.value;
    const isChecked = data.selected;
    if (!isChecked) {
      switch (lastSelectedValue) {
        case 'contractDate':
          this.filterEntity.contractDate = undefined;
          break;
      }
    }
  }

  //Fonction appelée lorsque l'on va utiliser un filtre sur les dates
  updatePropertyOptions(options, colName) {
    this.propertyOptions[colName] = options;
  }

  updateFilterEntity(newFilter, colName) {
    this.filterEntity[colName] = newFilter ? moment(newFilter) : undefined;
  }

  //#endregion
}
