import { Component, Inject, Input, OnInit, ViewChild } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable, Subscription } from 'rxjs';

import { MilestoneService } from './milestone.service';
import { IMilestone, Milestone } from 'src/app/bcs_ng/model/milestone.model';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import * as moment from 'moment';
import { JhiEventManager } from 'ng-jhipster';
import { IOperation } from 'src/app/bcs_ng/model/operation.model';
import { OperationService } from '../operation/operation.service';
import { AccountService } from 'src/app/core/auth/account.service';
import { IDependency } from 'src/app/bcs_ng/model/dependency.model';
import { DependencyType } from 'src/app/bcs_ng/model/enumeration/dependency-type.enum';
import { Utils } from 'src/app/bcs_ng/utils/util';
import { MatStepper } from '@angular/material/stepper';
import { DateAdapter, MAT_DATE_FORMATS, MAT_DATE_LOCALE } from '@angular/material/core';
import { MAT_MOMENT_DATE_FORMATS, MomentDateAdapter, MAT_MOMENT_DATE_ADAPTER_OPTIONS } from '@angular/material-moment-adapter';
import { TimeUnit, TimeUnitLabels } from 'src/app/bcs_ng/model/enumeration/time-unit.enum';
import { GroupUsersService } from 'src/app/entities/group-users/group-users.service';
import { ISystem } from 'src/app/bmcs/model/system.model';
import { MilestoneDTO } from 'src/app/bcs_ng/shared/dto/milestone.dto';
import { IBcsNgListOption } from 'src/app/bcs_ng/model/bcs-ng-list-option.model';
import { BcsNgListOptionService } from '../../bcs-ng-list-option/bcs-ng-list-option.service';
import { CrudType } from 'src/app/bcs_ng/model/enumeration/crud-type';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { BaseComponent } from 'src/app/shared/base.component';
import { IVersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { DuplicateFieldsValidator } from 'src/app/bcs_ng/shared/validators/duplicate-fields-validators';
import { IDeployedCenter } from 'src/app/bmcs/model/deployed-center.model';

@Component({
  selector: 'jhi-milestone-update',
  templateUrl: './milestone-update.component.html',
  styleUrls: ['../table-bcs-ng.scss', '../../bcs-ng.scss'],
  providers: [
    {
      provide: DateAdapter,
      useClass: MomentDateAdapter,
      deps: [MAT_DATE_LOCALE, MAT_MOMENT_DATE_ADAPTER_OPTIONS],
    },
    { provide: MAT_DATE_FORMATS, useValue: MAT_MOMENT_DATE_FORMATS },
  ]
})
export class MilestoneUpdateComponent extends BaseComponent implements OnInit {
  @Input() milestone: IMilestone;
  initialMilestone: IMilestone;
  @Input() operationFromDetail: IOperation;
  selectedOperation: IOperation[] = [];
  selectedPredecessors: IMilestone[] = [];
  dependencies: { [key: string]: IDependency } = {};
  invalidDependencies: { [key: string]: boolean } = {};
  milestones: IMilestone[] = [];
  potentialPredecessors: IMilestone[];
  operations: IOperation[];
  typeValues: IBcsNgListOption[] = [];
  sourceValues: IBcsNgListOption[] = [];
  filteredSourceValues: IBcsNgListOption[] = [];
  versionedSystems: IVersionedSystem[] = [];
  deployedCenters: IDeployedCenter[] = [];
  allNumberIds: number[] = [];

  loadOperations: boolean = false;
  loadMilestones: boolean = false;
  displayOperationTable: boolean = false;
  displayMilestoneTable: boolean = false;
  isSaving = false;
  eventSubscriber: Subscription;
  @ViewChild('stepper') stepper: MatStepper;

  editForm = this.fb.group({
    id: [],
    name: [null, [Validators.required, Validators.maxLength(128)]],
    shortName: [null, [Validators.maxLength(25)]],
    comment: [null, Validators.maxLength(255)],
    date: [null, [Validators.required]],
    contractDate: [],
    typeMilestoneId: [null, [Validators.required]],
    sourceId: [null, [Validators.required]],
    predecessors: [],
    operationId: [null, [Validators.required]],
    number: [null, [Validators.required, Validators.maxLength(10)]]
  });

  displayColumnsOperation: string[] = ['select', 'manual_id', 'domaine', 'versionedSystem', 'deployedCenter', 'name', 'type'];
  displayColumnsMilestone: string[] = ['select', 'versionedSystem', 'deployedCenter', 'operation', 'name', 'type', 'date'];

  canUpdate: boolean = false;
  tryUpdate: boolean = false;

  predecessorsText = "À chaque prédécesseur doit être associé un délai (qui peut être positif ou négatif).\n" +
    "Si la date d'un jalon est modifiée, celle de tous les jalons qui en dépendent seront aussi modifiées.";

  timeUnitLabels = TimeUnitLabels;
  timeUnitValues = Object.keys(this.timeUnitLabels);

  constructor(
    protected milestoneService: MilestoneService,
    protected milestoneDto: MilestoneDTO,
    protected operationService: OperationService,
    protected bcsNgListOptionService: BcsNgListOptionService,
    protected accountService: AccountService,
    protected activatedRoute: ActivatedRoute,
    protected groupUsersService: GroupUsersService,
    protected bmcsBaseService: BmcsBaseService,
    private fb: FormBuilder,
    protected activeModal: NgbActiveModal,
    protected eventManager: JhiEventManager,
    private _adapter: DateAdapter<any>,
    @Inject(MAT_DATE_LOCALE) private _locale: string
  ) { super(accountService) }

  ngOnInit(): void {
    this.initialMilestone = { ...this.milestone };
    this.bcsNgListOptionService.query().subscribe(res => {
      this.typeValues = res.body?.filter(optionList => optionList.type === 'MILESTONE_TYPE');
      this.sourceValues = res.body?.filter(optionList => optionList.type === 'MILESTONE_SOURCE');
      this.filteredSourceValues = this.sourceValues?.filter(tv => tv.label !== 'UNKNOWN');
    });
    this.setNumberIdsControl();
    if (this.milestone != null) {
      this.updateForm(this.milestone);
    } else if (this.operationFromDetail) {
      this.editForm.patchValue({ operationId: this.operationFromDetail.id });
      this.selectedOperation.push(this.operationFromDetail);
    } else {
      this.displayOperationTable = true;
      this.queryOperations();
    }
    this._locale = 'fr';
    this._adapter.setLocale(this._locale);
    this.versionedSystems = this.bmcsBaseService.getVersionedSystems;
    this.deployedCenters = this.bmcsBaseService.getDeployedCenters;
    this.eventSubscriber = this.eventManager.subscribe('milestoneListModification', (obj) => this.queryMilestones(obj));
  }

  updateForm(milestone: IMilestone): void {
    this.editForm.patchValue({
      id: milestone?.id,
      name: milestone?.name,
      shortName: milestone?.shortName,
      comment: milestone.comment,
      date: new Date(milestone.startDate?.toString()),
      contractDate: milestone.contractDate ? new Date(milestone.contractDate?.toString()) : null,
      typeMilestoneId: milestone?.typeMilestone?.id,
      sourceId: milestone?.source?.id,
      predecessors: milestone?.predecessorDependencies,
      operationId: milestone?.operation?.id,
      number: milestone?.number
    });
    this.selectedPredecessors = milestone.predecessorDependencies.map(dep => dep.predecessor);
    milestone.predecessorDependencies.forEach(dep => {
      this.dependencies[dep.predecessor.id] = { ...dep };
    });
    this.selectedOperation.push(milestone?.operation);
    this.selectedOperation[0].versionedSystem = this.bmcsBaseService.findVersionedSystem(milestone?.operation?.versionedSystemId);
    if (this.selectedOperation[0].deployedCenterId) this.selectedOperation[0].deployedCenter = this.bmcsBaseService.findDeployedCenter(milestone?.operation?.deployedCenterId);
    if (this.selectedPredecessors?.length) this.editForm.controls['date'].disable();
  }

  cancel(): void {
    if (this.accountService.userIdentity?.login === this.milestone?.modifyBlockingBy)
      this.milestoneService.freeUpdate(this.milestone).subscribe(() => { });
    this.activeModal.dismiss();
  }

  save(): void {
    this.isSaving = true;
    const milestone = this.createFromForm();
    const fullMilestone = this.completeMilestone(milestone);
    const system = this.selectedOperation[0]?.versionedSystem?.system;
    if (milestone.id !== undefined && milestone.id !== null) {
      this.isAllowedToSave(this.milestoneService.update(fullMilestone), system, CrudType.UPDATE);
    } else {
      milestone.id = undefined
      this.isAllowedToSave(this.milestoneService.create(fullMilestone), system, CrudType.CREATE);
    }
  }

  private createFromForm(): IMilestone {
    return {
      ...new Milestone(),
      id: this.editForm.get(['id'])!.value,
      name: this.editForm.get(['name'])!.value,
      shortName: this.editForm.get(['shortName'])!.value,
      startDate: moment(this.editForm.get(['date'])!.value),
      endDate: moment(this.editForm.get(['date'])!.value),
      comment: this.editForm.get(['comment'])!.value,
      contractDate: this.editForm.get(['contractDate'])!.value ? moment(this.editForm.get(['contractDate'])!.value) : undefined,
      typeMilestone: this.typeValues.find(t => t.id == this.editForm.get(['typeMilestoneId'])!.value),
      source: this.sourceValues.find(s => s.id === this.editForm.get(['sourceId'])!.value),
      operation: this.selectedOperation[0],
      number: this.editForm.get(['number'])!.value,
    };
  }

  private completeMilestone(incompleteMilestone: IMilestone): IMilestone {
    const copy = { ...incompleteMilestone };
    if (!copy?.id) copy.id = Utils.createUUID();
    // Dependencies
    const dependenciesToSave = Object.values(this.dependencies).map(dep => {
      const dependencyMilestone = this.selectedPredecessors?.find(pred => pred.id === dep.predecessor.id);
      const dependencyType = copy.operation.id === dependencyMilestone.operation.id ?
        DependencyType.INTRA :
        DependencyType.INTER;
      return {
        ...dep,
        predecessor: { id: dep.predecessor.id, typeMilestone: dependencyMilestone?.typeMilestone } as any,
        successor: { id: copy.id } as any,
        type: dependencyType,
      }
    });
    copy.predecessorDependencies = dependenciesToSave;
    return copy;
  }

  protected isAllowedToSave(result: Observable<HttpResponse<IOperation>>, system: ISystem, crudType: CrudType) {
    if (this.isAdminBCS) this.subscribeToSaveResponse(result);
    else {
      this.groupUsersService.isAllowToCreateInBcsBySystem(system.id, crudType).subscribe(res => {
        this.subscribeToSaveResponse(result);
      },
        err => this.onSaveError(err));
    }
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IMilestone>>): void {
    result.subscribe(
      (res) => this.onSaveSuccess(res.body),
      (err) => this.onSaveError(err)
    );
  }

  protected onSaveSuccess(res: IMilestone): void {
    this.isSaving = false;
    this.eventManager.broadcast({ name: 'milestoneListModification', content: res.id });
    this.activeModal.close();
  }

  protected onSaveError(err): void {
    this.isSaving = false;
    console.error(err);
  }

  protected setNumberIdsControl() {
    this.milestoneService.getAllNumberIds().subscribe(res => {
      this.allNumberIds = res.filter(number => number !== this.milestone?.number);
      this.editForm.setValidators([
        DuplicateFieldsValidator.duplicateNumberField('number', this.allNumberIds, { duplicateNumberId: true })
      ])
    });
  }

  protected queryMilestones(obj?) {
    this.loadMilestones = true;
    this.milestoneService.query().subscribe((res: HttpResponse<IMilestone[]>) => {
      this.milestones = res.body || [];
      this.potentialPredecessors = this.allowedPredecessors(this.milestones);
      if (obj?.content) {
        this.selectedPredecessors.push(this.potentialPredecessors.find(p => p.id === obj.content));
        this.editForm.patchValue({ predecessors: this.selectedPredecessors });
      }
      this.loadMilestones = false;
    });
  }

  protected queryOperations() {
    this.loadOperations = true;
    this.operationService.query().subscribe((res: HttpResponse<IOperation[]>) => {
      this.operations = res.body || [];
      this.operations.forEach(o => {
        o.versionedSystem = this.bmcsBaseService.findVersionedSystem(o.versionedSystemId);
        o.deployedCenter = this.bmcsBaseService.findDeployedCenter(o.deployedCenterId);
      });
      this.loadOperations = false;
    });
  }

  loadAndDisplayOperationTable() {
    this.queryOperations();
    this.displayOperationTable = true;
  }

  loadAndDisplayMilestoneTable() {
    this.queryMilestones();
    this.displayMilestoneTable = true;
  }

  protected freeUpdate(doesUpdate: boolean) {
    if (this.accountService.userIdentity?.login === this.milestone?.modifyBlockingBy)
      this.milestoneService.freeUpdate(this.milestone).subscribe((res) => {
        this.milestone = res.body;
        this.importActionForUpdate(doesUpdate);
      });
  }

  protected importActionForUpdate(doesUpdate: boolean): void {
    if (doesUpdate) this.updateForm(this.milestone);
    this.canUpdate = true;
    this.editForm.enable()
    this.tryUpdate = false;
  }

  allowedPredecessors(milestones: IMilestone[]): IMilestone[] {
    return milestones?.filter(m => this.milestone?.id !== m.id);
  }

  allDependenciesValid(): boolean {
    return !Object.values(this.dependencies).some(dep => {
      const depIntegerTimeGapValue = Number(dep.timeGapDuration);
      return !Number.isInteger(depIntegerTimeGapValue);
    })
  }

  onChangeDate(event: any) {
    // this.potentialPredecessors = this.allowedPredecessors(this.milestones, event.value);
  }

  onSelectPredecessor(milestone: IMilestone[]) {
    this.onDeselectAllPredecessors();
    if (milestone?.length) {
      milestone.forEach(m => {
        const predecessor = this.potentialPredecessors.find(mil => mil.id === m.id);
        this.dependencies[m.id] = {
          predecessor,
          timeGapDuration: 0,
          timeGapUnit: TimeUnit.MONTHS,
        }
      });
      this.selectedPredecessors = milestone;
      this.editForm.controls['date'].disable();
      this.onChangeTimeGap();
    } else this.editForm.controls['date'].enable();
  }

  onDeselectAllPredecessors() {
    this.selectedPredecessors = [];
    this.dependencies = {};
    this.editForm.controls['date'].enable();
  }

  dependencyList(): IDependency[] {
    return Object.values(this.dependencies);
  }

  isStepValid(step: number): boolean {
    return step == 0 ? this.isStep0Valid() : this.isStep1Valid();
  }

  isStep0Valid(): boolean {
    return this.editForm.get(['operationId']).valid &&
      this.editForm.get(['name']).valid &&
      this.editForm.get(['number']).valid &&
      this.editForm.get(['typeMilestoneId']).valid &&
      this.editForm.get(['sourceId']).valid &&
      !this.editForm?.hasError('duplicateNumberId');
  }

  isStep1Valid(): boolean {
    return this.editForm.get(['date'])?.value;
  }

  changeName(value) {
    const typeValue = this.typeValues.find(tv => tv.id === value);
    this.editForm?.controls?.name.setValue(typeValue?.name);
    this.editForm?.controls?.shortName.setValue(typeValue?.label);
  }

  async onChangeOperation(selected) {
    this.editForm.controls['operationId'].setValue(selected[0]?.id);
    this.selectedOperation = selected;
  }

  // On change occurs after the model change, so dependencies are up to date
  onChangeTimeGap(): void {
    // If all dependencies including this one ara not valid, do not update the date
    if (!this.allDependenciesValid()) return;
    // Otherwise update the date
    const firstDep = Object.values(this.dependencies)[0]
    const firstPredMilestone = this.potentialPredecessors.find(pred => pred.id === firstDep.predecessor.id);
    let startDate = Utils.addTimeGapToMoment(firstPredMilestone.startDate, firstDep.timeGapDuration, firstDep.timeGapUnit);
    Object.values(this.dependencies).slice(1).forEach(dep => {
      const predecessorMilestone = this.potentialPredecessors.find(pred => pred.id === dep.predecessor.id);
      const possibleNewDate = Utils.addTimeGapToMoment(predecessorMilestone.startDate, dep.timeGapDuration, dep.timeGapUnit);
      if (possibleNewDate > startDate) startDate = possibleNewDate;
    });
    this.editForm.patchValue({ date: startDate });
  }
}
