import { Component, OnInit, OnDestroy, Input, Output, EventEmitter, ViewChild } from '@angular/core';
import { KeyValue } from '@angular/common';
import { HttpResponse } from '@angular/common/http';
import { MatSort, Sort } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';
import { MatPaginator, PageEvent } from '@angular/material/paginator';

import { fromEvent, Observable, Subscription } from 'rxjs';
import { JhiEventManager } from 'ng-jhipster';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import * as moment from 'moment';
import { MatTableFilter } from 'mat-table-filter';

import { MilestoneService } from './milestone.service';
import { IMilestone } from 'src/app/bcs_ng/model/milestone.model';
import { MilestoneUpdateDialogComponent } from './dialog/milestone-update-dialog.component';
import { FilterKey, StorageService } from 'src/app/blocks/storage/storage-service';
import { AccountService } from 'src/app/core/auth/account.service';
import { Scope } from 'src/app/shared/util/scope';
import { FilterWithAccents } from 'src/app/shared/util/filter-with-accents';
import { ExcelService } from 'src/app/shared/util/excel.service';
import { MilestoneHistoryListDialogComponent } from './dialog/milestone-history-list-dialog.component';
import { IMilestoneCrud, MilestoneCrud } from 'src/app/bcs_ng/shared/dto/model/milestone-crud.dto';
import { MilestoneDTO } from 'src/app/bcs_ng/shared/dto/milestone.dto';
import { SelectionModel } from '@angular/cdk/collections';
import { ITask } from 'src/app/bcs_ng/model/task/Itask.model';
import { IDependency } from 'src/app/bcs_ng/model/dependency.model';
import { DeployedVersionedSystemService } from 'src/app/bmcs/entity/deployed-versioned-system/deployed-versioned-system.service';
import { IVersionedSystem, VersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { System } from 'src/app/bmcs/model/system.model';
import { DialogDeleteBcsComponent } from 'src/app/bcs_ng/shared/dialog/delete/dialog-delete-bcs.component';
import { DialogRebornBcsComponent } from 'src/app/bcs_ng/shared/dialog/reborn/dialog-reborn-bcs.component';
import { DialogEraseBcsComponent } from 'src/app/bcs_ng/shared/dialog/erase/dialog-erase-bcs.component';
import { ISystem } from 'src/app/shared/model/entity/system.model';
import { DeployedCenter, IDeployedCenter } from 'src/app/bmcs/model/deployed-center.model';
import { ISite, Site } from 'src/app/bmcs/model/site.model';
import { Domaine, IDomaine } from 'src/app/shared/model/entity/domaine.model';
import { Cruddomaine } from 'src/app/shared/model/entity/cruddomaine.model';
import { IBcsNgListOption } from 'src/app/bcs_ng/model/bcs-ng-list-option.model';
import { BcsNgListOptionService } from '../../bcs-ng-list-option/bcs-ng-list-option.service';
import { MatMenuTrigger } from '@angular/material/menu';
import { BcsNgHistoricUpdateComponent } from 'src/app/bcs_ng/entity/history/bcs-ng-historic-update.component';
import { Baseline } from 'src/app/bcs_ng/model/enumeration/baseline.enum';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { BaseComponent } from 'src/app/shared/base.component';


@Component({
  selector: 'jhi-milestone',
  templateUrl: './milestone.component.html',
  styleUrls: ['../table-bcs-ng.scss', '../../bcs-ng.scss'],
})
export class MilestoneComponent extends BaseComponent implements OnInit, OnDestroy {
  // Inputs
  @Input() showButtons: boolean = true;
  @Input() sizeMatPaginator?: number;
  @Output() scopeChange = new EventEmitter<Scope>();
  @Input() selectTable?: boolean = false;
  @Input() selectedItem?: IMilestone[] = [];

  // Data to fetch
  @Input() milestones?: IMilestone[] = [];
  @Input() versionedSystems?: IVersionedSystem[] = [];
  @Input() deployedCenters?: IDeployedCenter[] = [];
  @Input() crudDomaines?: [Cruddomaine[]] = [[]];
  @Input() onlyActive?: boolean = false;
  @Input() baseline: Baseline;
  typeValues: IBcsNgListOption[] = [];
  sourceValues: IBcsNgListOption[] = [];

  // Page
  load: boolean = true;
  eventSubscriber?: Subscription;
  leftClickObservable: Observable<Event>;
  leftClickSubscription: Subscription;
  scope: Scope = this.isAdminBCS ? Scope.ALL : Scope.ALLACTIVE;
  isMyScopeActive: boolean = false;
  scopeMilestone: boolean = false;

  // Mat-table
  dataSource = new MatTableDataSource<IMilestoneCrud>([]);
  data: IMilestoneCrud[] = [];
  @ViewChild(MatSort) matSorter!: MatSort;
  activeSort: Sort = { active: 'id', direction: 'asc' };
  @ViewChild(MatPaginator, { static: true }) paginator!: MatPaginator;
  pageSize = 50;
  @ViewChild(MatMenuTrigger, { static: true }) matMenuTrigger: MatMenuTrigger;
  contextMenuPosition = { x: '0px', y: '0px' };
  allColumnsMilestone = new Map([
    ['number', 'ID'],
    ['domaine', 'Domaine'],
    ['system', 'Système'],
    ['versionedSystem', 'Système versionné'],
    ['deployedCenter', 'Centre déployé'],
    ['site', 'Site'],
    ['operation', 'Opération'],
    ['name', 'Nom'],
    ['shortName', 'Abréviation'],
    ['comment', 'Commentaire'],
    ['date', 'Date'],
    ['contractDate', 'Date contractuelle'],
    ['type', 'Type'],
    ['source', 'Source'],
    ['predecessorDependencies', 'Prédécesseurs'],
    ['details', 'Détails']
  ]);
  displayedColumnsMilestone: string[];
  public tempDisplayedCol: string[];
  public displayedSearchColumns: string[];
  filterEntity: IMilestoneCrud = new MilestoneCrud();
  filterType: MatTableFilter;
  propertyOptions: any;
  filterInputContractDate: string = '';

  @Input() defaultDisplayColumns: string[]
    = ['number', 'domaine', 'versionedSystem', 'deployedCenter', 'operation', 'name', 'date', 'type', 'predecessorDependencies'];
  defaultDisplayColumnsWithDetails: string[] = this.defaultDisplayColumns.concat(['details']);
  defaultSearchColumns: string[] = this.defaultDisplayColumns.map(c => `search-${c}`);
  defaultSearchColumnsWithDetails: string[] = this.defaultDisplayColumnsWithDetails.map(c => `search-${c}`);

  Baseline = Baseline;

  constructor(
    protected milestoneService: MilestoneService,
    protected deployedVersionedSystemService: DeployedVersionedSystemService,
    protected bcsNgListOptionService: BcsNgListOptionService,
    protected accountService: AccountService,
    protected milestoneDto: MilestoneDTO,
    protected storageService: StorageService,
    protected eventManager: JhiEventManager,
    protected modalService: NgbModal,
    protected filterWithAccents: FilterWithAccents,
    protected excelService: ExcelService,
    protected bmcsbaseService: BmcsBaseService,
  ) {
    super(accountService);
  }

  //----------------------------------------------------------------------------------------------------------------------
  //---                                            Angular component lifecycle hooks                                                  --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'lifecycle'
  ngOnInit(): void {
    if (this.sizeMatPaginator) this.pageSize = this.sizeMatPaginator;
    else this.pageSize = Number(this.storageService.get(FilterKey.NB_ITEMS)) || this.userPreferences?.nbItemTable;
    this.initFilterByColumn();
    if (this.selectTable) {
      this.pageSize = 5;
      if (this.milestones?.length === 0) this.loadAll();
      else this.updateMilestones();
    }
    this.registerChangeInMilestones();
    this.bcsNgListOptionService.query().subscribe(res => {
      this.typeValues = res.body.filter(optionList => optionList.type === 'MILESTONE_TYPE');
      this.sourceValues = res.body.filter(optionList => optionList.type === 'MILESTONE_SOURCE');
    });
    this.leftClickObservable = fromEvent(window, 'click');
    this.leftClickSubscription = this.leftClickObservable.subscribe(event => {
      this.matMenuTrigger?.closeMenu();
    })
  }

  ngOnChanges(): void {
    this.updateMilestones(this.baseline != Baseline.NONE);
  }

  ngAfterViewChecked() {
    if (this.selectTable) {
      const sortState: Sort = { active: 'select', direction: 'asc' };
      this.matSorter.active = sortState.active;
      this.matSorter.direction = sortState.direction;
      this.matSorter.sortChange.emit(sortState);
    }
  }

  ngOnDestroy(): void {
    if (this.eventSubscriber)
      this.eventManager.destroy(this.eventSubscriber);
    this.leftClickSubscription.unsubscribe();
  }
  //#endregion

  //----------------------------------------------------------------------------------------------------------------------
  //---                                            LOAD & UPDATE DATA                                                  --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'loadData'
  protected onError(): void {
    this.load = false;
  }

  protected loadAll(): void {
    this.load = true;
    const getAll = this.onlyActive ? false : this.isAdminBCS;
    this.milestoneService.query(getAll).subscribe((res: HttpResponse<IMilestone[]>) => {
      this.milestones = res.body || [];
      this.updateMilestones();
      this.load = false;
    });
  }

  protected updateMilestones(isBaseline: boolean = false): void {
    this.data = this.milestoneDto.convertToCrud(this.milestones, this.versionedSystems, this.deployedCenters, this.crudDomaines, isBaseline);
    this.updateTable();
    if (this.selectTable) {
      this.selection.clear();
      this.selectedItem.forEach(milestone => {
        this.selection?.toggle(this.data?.find(d => d.id === milestone.id));
      });
    }
  }

  protected updateTable(): void {
    this.load = true;
    this.dataSource.data = this.data || [];
    this.dataSource.sort = this.matSorter;
    this.dataSource.sortingDataAccessor = (data, columnDef) => {
      switch (columnDef) {
        case 'select': return this.selectedItem.find(i => i.id === data.id) ? 0 : 1
        default: return data[columnDef];
      }
    };
    if (!this.isScully) {
      this.paginator.pageSize = this.pageSize;
      this.dataSource.paginator = this.paginator;
      this.dataSource.paginator._intl.itemsPerPageLabel = "Items par page :";
    }
    this.load = false;
  }

  getFilteredMilestones(): IMilestone[] {
    return this.dataSource.filteredData.map(crudM => crudM.milestone);
  }

  //#endregion
  //----------------------------------------------------------------------------------------------------------------------
  //---                                               USER MODIFICATIONS/ACTIONS                                        --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'modification'

  exportMilestones(): void {
    const filteredMilestones = this.milestoneDto.convertFromCrud(this.dataSource.filteredData);
    this.excelService.exportAsExcelFile(this.milestoneDto.convertMilestonesDataToExcel(filteredMilestones), 'Jalons-SCCOA');
  }

  defineParamsDialog(modalRef, id) {
    modalRef.componentInstance.id = id;
    modalRef.componentInstance.entityMsg = "ce jalon";
    modalRef.componentInstance.serviceEntity = this.milestoneService;
    modalRef.componentInstance.eventManagerName = "milestoneListModification";
  }

  trackId(index: number, item: IMilestone): string {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
    return item.id!;
  }

  registerChangeInMilestones(): void {
    this.eventSubscriber = this.eventManager.subscribe('milestoneListModification', () => this.loadAll());
  }

  update(milestone: IMilestone | undefined): void {
    const modalRef = this.modalService.open(MilestoneUpdateDialogComponent, { windowClass: 'xxlModal', backdrop: 'static' });
    modalRef.componentInstance.milestone = milestone;
  }

  updateHistory(data: IMilestoneCrud): void {
    const modalRef = this.modalService.open(MilestoneHistoryListDialogComponent, { windowClass: 'xxlModal', backdrop: 'static' });
    modalRef.componentInstance.id = data.milestone.id;
    modalRef.componentInstance.milestoneEntity = true;
    modalRef.componentInstance.isAllowedToUpdate = !data.milestone.deleted && (this.isAdminBCS || data?.crudDomaine?.update);
  }

  delete(milestone: IMilestone): void {
    const modalRef = this.modalService.open(DialogDeleteBcsComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, milestone.id);
  }

  reborn(action: IMilestone): void {
    const modalRef = this.modalService.open(DialogRebornBcsComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, action.id);
  }

  erase(action: IMilestone): void {
    const modalRef = this.modalService.open(DialogEraseBcsComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, action.id);
  }

  //#endregion
  //----------------------------------------------------------------------------------------------------------------------
  //---                                             CONTEXTUAL MENU                                                 --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'Contextual menu'
  onRightClick(event: any, row: IMilestoneCrud): void {
    this.matMenuTrigger?.closeMenu();
    event.preventDefault();
    this.contextMenuPosition.x = event.clientX + 'px';
    this.contextMenuPosition.y = event.clientY + 'px';
    this.matMenuTrigger.menuData = { 'item': row };
    this.matMenuTrigger.menu.focusFirstItem('mouse');
    if (this.baseline == 'NONE' && (this.isAdminBCS || row?.crudDomaine?.update))
      this.matMenuTrigger.openMenu();
  }

  setHistoricToBaseline(item: IMilestoneCrud, newBaselineValue?: Baseline): void {
    const modalRef = this.modalService.open(BcsNgHistoricUpdateComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.fromHistoric = false;
    modalRef.componentInstance.id = item.milestone.id;
    modalRef.componentInstance.newBaselineValue = newBaselineValue;
  }

  //#endregion
  //----------------------------------------------------------------------------------------------------------------------
  //---                                             FILTER ON TABLE                                                     --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'Filter'
  initFilterByColumn() {
    this.initAllFilters();
    //Récupère le type de filtre, ici anywhere donc on va garder les données contenant le filtre demandé
    this.filterType = MatTableFilter.ANYWHERE;
    this.setFilterOnInit();
    this.propertyOptions = {
      globalFilter: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.globalFilter);
      },
      versionedSystem: (route: IVersionedSystem) => {
        return this.filterWithAccents.compareStrings(route?.name, this.filterEntity.versionedSystem.name);
      },
      deployedCenter: (route: IDeployedCenter) => {
        return this.filterWithAccents.compareStrings(route?.name, this.filterEntity.deployedCenter.name);
      },
      site: (route: ISite) => {
        let routeSite = route?.fullName;
        if (route?.name) routeSite += route?.name;
        return this.filterWithAccents.compareStrings(routeSite, this.filterEntity.site.fullName);
      },
      system: (route: ISystem) => {
        return this.filterWithAccents.compareStrings(route?.name, this.filterEntity.system.name);
      },
      domaine: (route: IDomaine) => {
        return this.filterWithAccents.compareStrings(route?.label, this.filterEntity.domaine.label);
      },
      name: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.name);
      },
      shortName: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.shortName);
      },
      comment: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.comment);
      },
      typeMilestone: (route: IBcsNgListOption) => {
        return this.filterWithAccents.compareStrings(route?.name, this.filterEntity.typeMilestone?.toString());
      },
      source: (route: IBcsNgListOption) => {
        return this.filterWithAccents.compareStrings(route?.name, this.filterEntity.source?.toString());
      },
      predecessorDependencies: (route: IDependency[]) => {
        const filter = this.filterEntity.predecessorDependencies.toString();
        const milestones = route.map(d => d.predecessor.name).toString();
        return (filter === 'vide' && route.length === 0) || this.filterWithAccents.compareStrings(milestones, filter);
      },
      operation: (route: string) => {
        return (this.filterEntity.operation === 'vide' && !route) || this.filterWithAccents.compareStrings(route, this.filterEntity.operation);
      },
      deleted: (route: boolean) => {
        return (this.filterEntity.deleted && route) || !this.filterEntity.deleted;
      }
    }
  }

  setFilterOnInit() {
    if (this.filterInputContractDate) {
      this.selectScope();
    } else if (this.showButtons && !this.selectTable) {
      //get local storage filters
      const filters = this.storageService.get(FilterKey.MIL_FILTER);
      if (filters) {
        //S'il y a des filtres enregistrés, on les assignes à la liste des filtres par colonnes
        this.filterEntity = filters as IMilestoneCrud;
      }
      const fav = this.storageService.get(FilterKey.MIL_FAV);
      if (fav) {
        this.scope = Scope.MYSCOPE;
        this.isMyScopeActive = true;
      }
      const sort = this.storageService.get(FilterKey.MIL_SORT);
      if (sort) this.activeSort = sort as Sort;
    }

    const columns = this.storageService.get(FilterKey.MIL_COLUMNS);
    if (columns && !this.selectTable) {
      //On assigne la liste des colonnes affichées sauvegardées au tableau des colonnes qui doivent être affichées
      this.displayedColumnsMilestone = columns as string[];
      this.tempDisplayedCol = columns as string[];
      //Donne la liste des actions correspondantes et la sauvegarde
      this.changeColumns();
    } else if (this.isScully) {
      this.displayedColumnsMilestone = this.defaultDisplayColumns;
      this.tempDisplayedCol = this.defaultDisplayColumns;
      this.displayedSearchColumns = this.defaultSearchColumns;
    } else {
      //S'il n'y a pas de colonnes sauvegardées, on affiche les colonnes préférentielles de l'utilisateur
      const colsPreferences = this.userPreferences?.milestoneColumns;
      if (colsPreferences && !this.selectTable) {
        this.displayedColumnsMilestone = colsPreferences;
        this.tempDisplayedCol = colsPreferences;
        this.changeColumns();
      }
      else {
        if (this.selectTable) {
          this.displayedColumnsMilestone = this.defaultDisplayColumns;
          this.displayedSearchColumns = this.defaultDisplayColumns.map(c => `search-${c}`);
        }
        else {//S'il n'y a pas de colonnes sauvegardées et que l'utilisateur n'a pas de préférence, on affiche ces colonnes et leur jalons correspondantes
          this.displayedColumnsMilestone = this.defaultDisplayColumnsWithDetails;
          this.tempDisplayedCol = this.defaultDisplayColumnsWithDetails;
          this.displayedSearchColumns = this.defaultSearchColumnsWithDetails;
        }
      }
    }
  }

  changeColumns() {
    //On assigne les filtres des colonnes qui doivent être affichées dans un tableau
    this.displayedSearchColumns = this.displayedColumnsMilestone.map(col => 'search-' + col);
    //On sauvegarde ce tableau
    this.storageService.set(FilterKey.MIL_COLUMNS, this.displayedColumnsMilestone);
  }

  asOrderCols(a: KeyValue<string, any>, b: KeyValue<string, any>): number {
    const aStepId = parseInt(a.key, 10);
    const bStepId = parseInt(b.key, 10);
    return aStepId > bStepId ? 1 : (bStepId > aStepId ? -1 : 0);
  }

  selectScope(): void {
    if (this.scope === Scope.MYSCOPE) {
      this.scope = this.isAdminBCS ? Scope.ALL : Scope.ALLACTIVE;
      this.scopeChange.emit(this.scope)
    } else {
      this.scope = Scope.MYSCOPE;
      this.scopeChange.emit(this.scope)
    }
    this.isMyScopeActive = !this.isMyScopeActive;
    if (this.isMyScopeActive) this.storageService.set(FilterKey.MIL_FAV, this.isMyScopeActive);
    else this.storageService.remove(FilterKey.MIL_FAV);
  }

  openedChange(opened: boolean) {
    if (!opened) {
      this.displayedColumnsMilestone = this.tempDisplayedCol;
      this.changeColumns();
    }
  }

  deleteAllFilter() {
    this.scopeMilestone = false;
    this.initAllFilters();
    this.storeFilters();
  }

  initAllFilters() {
    this.filterEntity = new MilestoneCrud();
    this.filterEntity.versionedSystem = new VersionedSystem();
    this.filterEntity.deployedCenter = new DeployedCenter();
    this.filterEntity.site = new Site();
    this.filterEntity.system = new System();
    this.filterEntity.domaine = new Domaine();
    this.filterEntity.deleted = false;
  }

  changefilter() {
    this.storeFilters();
  }

  //Retourne si le filtre envoyé en paramètre doit être activé ou non
  activeFilter(filterEntityData): boolean {
    //S'il n'est pas déjà activé, on retourne true, sino on retourne false
    return filterEntityData === undefined || filterEntityData === null || filterEntityData === '' || filterEntityData?.length === 0
      ? false : true;
  }

  isAllFilterNull(): boolean {
    return (
      !this.activeFilter(this.filterEntity.number) &&
      !this.activeFilter(this.filterEntity.name) &&
      !this.activeFilter(this.filterEntity.versionedSystem?.name) &&
      !this.activeFilter(this.filterEntity.system?.name) &&
      !this.activeFilter(this.filterEntity.domaine?.label) &&
      !this.activeFilter(this.filterEntity.deployedCenter?.name) &&
      !this.activeFilter(this.filterEntity.site?.name) &&
      !this.activeFilter(this.filterEntity.operation) &&
      !this.activeFilter(this.filterEntity.shortName) &&
      !this.activeFilter(this.filterEntity.comment) &&
      !this.activeFilter(this.filterEntity.date) &&
      !this.activeFilter(this.filterEntity.contractDate) &&
      !this.activeFilter(this.filterEntity.typeMilestone) &&
      !this.activeFilter(this.filterEntity.source) &&
      !this.activeFilter(this.filterEntity.predecessorDependencies)
    ) ? true : false;
  }

  updateFilterChangeCols(data: any) {
    let lastSelectedValue = data.value;
    const isChecked = data.selected;
    if (!isChecked) {
      switch (lastSelectedValue) {
        case 'date':
          this.filterEntity.date = undefined;
          break;
        case 'contractDate':
          this.filterEntity.contractDate = undefined;
          break;
      }
    }
  }

  //Fonction appelée lorsque l'on va utiliser un filtre sur les dates
  updatePropertyOptions(options, colName) {
    this.propertyOptions[colName] = options;
  }
  updateFilterEntity(newFilter, colName) {
    this.filterEntity[colName] = newFilter ? moment(newFilter) : undefined;
  }


  //#endregion
  //----------------------------------------------------------------------------------------------------------------------
  //---                                            SAVE LOCAL STORAGE                                                   --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'LocalStorage'

  storeFilters(): void {
    if (this.showButtons && !this.selectTable) {
      //Si tous les filtres ne sont pas désactivés, alors on remplace les filtres sauvegardés précédemment par la liste des filtres actuellements utilisés
      if (!this.isAllFilterNull()) this.storageService.set(FilterKey.MIL_FILTER, this.filterEntity);
      //Sinon, on supprime la liste des filtres sauvegardées
      else this.storageService.remove(FilterKey.MIL_FILTER);
    }
  }

  storeSort(sort: Sort): void {
    this.activeSort = sort;
    if (this.showButtons && !this.selectTable) {
      if (sort.active !== '' && sort.direction !== '') this.storageService.set(FilterKey.MIL_SORT, sort);
      else this.storageService.remove(FilterKey.MIL_SORT);
    }
  }

  getPaginatorChange(event: PageEvent): PageEvent {
    if (this.showButtons && !this.selectTable) this.storageService.set(FilterKey.NB_ITEMS, event.pageSize);
    return event;
  }
  //#endregion
  //----------------------------------------------------------------------------------------------------------------------
  //---                                               SELECT ITEM                                                       --        
  //----------------------------------------------------------------------------------------------------------------------

  selection = new SelectionModel<IMilestoneCrud>(true, []);
  @Output() onChangeSelectItem = new EventEmitter<ITask[]>();

  onChangeSelection(row) {
    this.selection.toggle(row);
    this.onChangeSelectItem.emit(this.selection.selected.map(t => t.milestone));
  }
  /** Whether the number of selected elements matches the total number of rows. */
  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.dataSource.filteredData.length;
    return numSelected === numRows;
  }

  /** Selects all rows if they are not all selected; otherwise clear selection. */
  toggleAllRows() {
    if (this.isAllSelected()) {
      this.selection.clear();
      this.onChangeSelectItem.emit(this.selection.selected.map(t => t.milestone));
      return;
    }
    this.selection.select(...this.dataSource.filteredData);
    this.onChangeSelectItem.emit(this.selection.selected.map(t => t.milestone));
  }
}
