import { Component, Input, OnChanges, OnInit } from "@angular/core";
import { ActivatedRoute } from '@angular/router';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';
import { AccountService } from 'src/app/core/auth/account.service';
import { IHistoric } from 'src/app/bcs_ng/model/historic.model';
import { IOperation } from 'src/app/bcs_ng/model/operation.model';
import { EncoderService } from 'src/app/shared/util/encoder-util';

@Component({
  selector: 'operation-diff-card',
  templateUrl: './operation-diff-card.component.html',
})
export class OperationDiffCardComponent implements OnInit, OnChanges {
  @Input() history: IHistoric;
  @Input() historyToCompare: IHistoric;
  operation: IOperation;
  operationToCompare: IOperation;
  CompareNowOrOlder: boolean = true;
  name: string;
  manual_id: string;
  description: string;
  shortName: string;
  number: string;
  contract: string;
  step: string;
  bmsComplements: string;
  domaine: string;
  system: string;
  versionedSystem: string;
  deployedCenter: string;
  site: string;
  typeOperation: string;

  constructor(
    protected eventManager: JhiEventManager,
    protected activatedRoute: ActivatedRoute,
    protected modalService: NgbModal,
    protected accountService: AccountService,
    protected encoderService: EncoderService
  ) { }

  ngOnChanges(): void {
    this.loadOperations();
  }

  ngOnInit(): void {
    this.loadOperations();
  }

  loadOperations(): void {
    if (this.history?.object) {
      this.operation = JSON.parse(this.history?.object) as IOperation;
    } else {
      this.operation = undefined;
    }
    if (this.historyToCompare?.object) {
      this.operationToCompare = JSON.parse(this.historyToCompare?.object) as IOperation;
    } else {
      this.operationToCompare = undefined;
    }
    if (this.operation && this.operationToCompare) {
      this.manual_id = this.encoderService.compareStringDifference(this.operation?.manual_id, this.operationToCompare?.manual_id);
      this.name = this.encoderService.compareStringDifference(this.operation?.name, this.operationToCompare?.name);
      this.description = this.encoderService.compareStringDifference(this.operation?.description, this.operationToCompare?.description);
      this.shortName = this.encoderService.compareStringDifference(this.operation?.shortName, this.operationToCompare?.shortName);
      this.number = this.encoderService.compareStringDifference(this.operation?.number?.toString(), this.operationToCompare?.number?.toString());
      this.contract = this.encoderService.compareStringDifference(this.operation?.contractDGA, this.operationToCompare?.contractDGA);
      this.step = this.encoderService.compareStringDifference(this.operation?.program, this.operationToCompare?.program);
      this.bmsComplements = this.encoderService.compareStringDifference(this.operation?.bmsComplement, this.operationToCompare?.bmsComplement);
      this.versionedSystem = this.encoderService.compareStringDifference(this.operation?.versionedSystem?.name, this.operationToCompare?.versionedSystem?.name);
      this.system = this.encoderService.compareStringDifference(this.operation?.versionedSystem?.system?.name, this.operationToCompare?.versionedSystem?.system?.name);
      this.domaine = this.encoderService.compareStringDifference(this.operation?.versionedSystem?.system?.domaine?.domaine, this.operationToCompare?.versionedSystem?.system?.domaine?.domaine);
      this.deployedCenter = this.encoderService.compareStringDifference(this.operation?.deployedCenter?.name, this.operationToCompare?.deployedCenter?.name);
      this.site = this.encoderService.compareStringDifference(this.operation?.deployedCenter?.site?.name, this.operationToCompare?.deployedCenter?.site?.name);
      this.typeOperation = this.encoderService.compareStringDifference(this.operation?.typeOperation?.name, this.operationToCompare?.typeOperation?.name);
    }
  }

  compareChange(): void {
    this.CompareNowOrOlder = !this.CompareNowOrOlder
  }
}