import { HttpResponse } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { DateAdapter } from '@angular/material/core';
import { ActivatedRoute } from '@angular/router';
import { NgbActiveModal, NgbModal } from '@ng-bootstrap/ng-bootstrap';
import * as moment from 'moment';
import { JhiEventManager } from 'ng-jhipster';
import { AccountService } from 'src/app/core/auth/account.service';
import { IOperation } from 'src/app/bcs_ng/model/operation.model';
import { OperationService } from '../operation.service';

@Component({
  selector: 'jhi-operation-update-dialog',
  templateUrl: './operation-update-dialog.component.html'
})

export class OperationUpdateDialogComponent implements OnInit {
  operation?: IOperation;
  showForm: boolean = false;

  constructor(
    protected operationService: OperationService,
    protected activatedRoute: ActivatedRoute,
    public activeModal: NgbActiveModal,
    protected modalService: NgbModal,
    protected eventManager: JhiEventManager,
    protected accountService: AccountService,
    private dateAdapter: DateAdapter<Date>
  ) {
    this.dateAdapter.setLocale('fr');
  }

  ngOnInit(): void {
    this.loadPage();
  }

  loadPage(): void {
    if (this.operation?.id) {
      this.operationService
        .find(this.operation?.id)
        .subscribe(
          (res: HttpResponse<IOperation>) => {
            this.operation = res.body
            this.blockUpdate();
          },
        );
    } else {
      this.showForm = true;
    }
  }

  private blockUpdate() {
    if ((!this.operation?.modifyBlockingDate || moment().diff(this.operation?.modifyBlockingDate) / 1000 > 901) ||
      this.accountService.userIdentity?.login === this.operation?.modifyBlockingBy) {
      this.operationService.blockUpdate(this.operation).subscribe((res) => {
        this.showForm = true;
        this.operation = res.body;
      });
    } else {
      this.showForm = false;
    }
  }

  cancel(): void {
    if (this.accountService.userIdentity?.login === this.operation?.modifyBlockingBy) this.operationService.freeUpdate(this.operation).subscribe((res) => { });
    this.activeModal.dismiss();
  }
}