import { HttpResponse } from '@angular/common/http';
import { Component, Input, OnInit } from '@angular/core';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { AngularEditorConfig } from '@kolkov/angular-editor';
import * as moment from 'moment';
import { JhiEventManager } from 'ng-jhipster';
import { Observable, Subscription } from 'rxjs';
import { AccountService } from 'src/app/core/auth/account.service';
import { IOperation, Operation } from 'src/app/bcs_ng/model/operation.model';
import { OperationService } from './operation.service';
import { DuplicateFieldsValidator } from 'src/app/bcs_ng/shared/validators/duplicate-fields-validators';
import { DeployedVersionedSystemService } from 'src/app/bmcs/entity/deployed-versioned-system/deployed-versioned-system.service';
import { IVersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { IDeployedCenter } from 'src/app/bmcs/model/deployed-center.model';
import { GroupUsersService } from 'src/app/entities/group-users/group-users.service';
import { ISystem } from 'src/app/bmcs/model/system.model';
import { BaseComponent } from 'src/app/shared/base.component';
import { Domaine } from 'src/app/shared/model/entity/domaine.model';
import { DomaineService } from 'src/app/entities/domaine/domaine.service';
import { IBcsNgListOption } from 'src/app/bcs_ng/model/bcs-ng-list-option.model';
import { BcsNgListOptionService } from '../../bcs-ng-list-option/bcs-ng-list-option.service';
import { CrudType } from 'src/app/bcs_ng/model/enumeration/crud-type';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';

@Component({
  templateUrl: './dialog/operation-update-dialog.component.html',
})
export class OperationBaseUpdateComponent extends BaseComponent implements OnInit {
  @Input() operation: IOperation;
  showForm: boolean = false;
  isSaving = false;

  editForm = this.fb.group({
    id: [],
    manual_id: [null, [Validators.required, Validators.pattern('^O-.*$'), Validators.maxLength(10)]],
    name: [null, [Validators.required, Validators.maxLength(128)]],
    shortName: [null, [Validators.maxLength(25)]],
    versionedSystem: [null, [Validators.required]],
    deployedCenter: [],
    description: [],
    number: [null, Validators.maxLength(64)],
    isBMS: [],
    contractDGA: [null, Validators.maxLength(64)],
    program: [null, Validators.maxLength(64)],
    typeOperationId: [null, [Validators.required]],
    bmsComplement: [null, Validators.maxLength(25)],
  });

  canUpdate: boolean = false;
  editorTrue: boolean = false;
  tryUpdate: boolean = false;
  changeOnSvOrCd: boolean = false;

  description: string = '';
  editorConfig: AngularEditorConfig = {
    editable: true,
    showToolbar: true,
    spellcheck: false,
    height: 'auto',
    minHeight: '50',
    maxHeight: 'auto',
    width: 'auto',
    minWidth: '0',
    translate: 'yes',
    enableToolbar: true,
    placeholder: 'Enter text here...',
    defaultParagraphSeparator: '',
    defaultFontName: '',
    defaultFontSize: '',
    uploadUrl: 'v1/image',
    uploadWithCredentials: false,
    sanitize: true,
    toolbarPosition: 'top',
    toolbarHiddenButtons: [
      [
        'subscript',
        'superscript',
        'indent',
        'outdent',
        'insertUnorderedList',
        'insertOrderedList',
        'heading',
        'fontName'
      ],
      [
        'customClasses',
        'backgroundColor',
        'link',
        'unlink',
        'insertImage',
        'insertVideo',
        'insertHorizontalRule',
        'toggleEditorMode'
      ]
    ]
  };
  versionedSystems?: IVersionedSystem[] = [];
  deployedCenters?: IDeployedCenter[] = [];
  versionedSystemSelected?: IVersionedSystem[] = [];
  deployedCenterSelected?: IDeployedCenter[] = [];

  typeValues: IBcsNgListOption[] = [];
  filteredTypeValues: IBcsNgListOption[] = [];

  dropdownSettings = {
    singleSelection: true,
    idField: 'id',
    textField: 'name',
    itemsShowLimit: 5,
    allowSearchFilter: true,
    searchPlaceholderText: 'Rechercher'
  };
  eventSubscriber: Subscription;

  constructor(
    protected operationService: OperationService,
    protected deployedVersionedSystemService: DeployedVersionedSystemService,
    protected bcsNgListOptionService: BcsNgListOptionService,
    protected accountService: AccountService,
    protected activatedRoute: ActivatedRoute,
    protected fb: FormBuilder,
    protected eventManager: JhiEventManager,
    protected groupUsersService: GroupUsersService,
    protected doamaineService: DomaineService,
    protected bmcsBaseService: BmcsBaseService,
  ) { super(accountService) }

  ngOnInit(): void {
    this.versionedSystems = this.bmcsBaseService.getVersionedSystems;
    this.deployedCenters = this.bmcsBaseService.getDeployedCenters;

    if (this.operation != null) this.updateForm(this.operation);
    this.operationService.getAvailableManualId().subscribe(res => {
      this.editForm.controls['manual_id'].setValue(res);
      console.log(res)
    });
    this.bcsNgListOptionService.query().subscribe(res => {
      this.typeValues = res.body?.filter(optionList => optionList.type === 'OPERATION_TYPE');
      this.filteredTypeValues = this.typeValues?.filter(tv => tv.label !== 'UNKNOWN');
    });
  }

  updateForm(operation: IOperation): void {
    this.editForm.patchValue({
      id: operation.id,
      manual_id: operation.manual_id,
      name: operation.name,
      shortName: operation.shortName,
      description: operation.description,
      number: operation.number,
      isBMS: operation.isBMS,
      contractDGA: operation.contractDGA,
      program: operation.program,
      typeOperationId: operation.typeOperation?.id,
      bmsComplement: operation.bmsComplement
    });
    this.versionedSystemSelected = this.versionedSystems.filter(sv => sv.id === this.operation?.versionedSystemId?.toUpperCase());
    this.editForm.controls['versionedSystem'].setValue(this.versionedSystemSelected);
    this.deployedCenterSelected = this.deployedCenters.filter(cd => cd.id === this.operation?.deployedCenterId?.toUpperCase());
    this.editForm.controls['deployedCenter'].setValue(this.deployedCenterSelected);
    this.description = this.operation?.description?.replace(/\r?\n|\r|&#10;/g, "<br>") ?? '';
  }

  private createFromForm(): IOperation {
    const operationFromForm = new Operation();
    operationFromForm.id = this.editForm.get(['id'])!.value;
    operationFromForm.manual_id = this.editForm.get(['manual_id'])!.value;
    operationFromForm.name = this.editForm.get(['name'])!.value;
    operationFromForm.shortName = this.editForm.get(['shortName'])!.value;
    operationFromForm.description = this.description;
    operationFromForm.number = this.editForm.get(['number'])!.value;
    operationFromForm.isBMS = this.editForm.get(['isBMS'])!.value;
    operationFromForm.contractDGA = this.editForm.get(['contractDGA'])!.value;
    operationFromForm.program = this.editForm.get(['program'])!.value;
    operationFromForm.typeOperation = this.typeValues.find(tv => tv.id === this.editForm.get(['typeOperationId'])!.value);
    operationFromForm.versionedSystemId = this.versionedSystemSelected[0].id;
    operationFromForm.versionedSystem = this.versionedSystems.find(sv => sv.id === this.versionedSystemSelected[0].id);
    operationFromForm.bmsComplement = this.editForm.get(['bmsComplement'])!.value;
    if (this.deployedCenterSelected.length > 0) {
      operationFromForm.deployedCenterId = this.deployedCenterSelected[0]?.id;
      operationFromForm.deployedCenter = this.deployedCenterSelected[0];
    } else {
      operationFromForm.deployedCenterId = null;
      operationFromForm.deployedCenter = null;
    }
    return operationFromForm;
  }

  loadPage(doesUpdate: boolean): void {
    if (this.operation?.id) {
      this.operationService.find(this.operation?.id)
        .subscribe(
          (res: HttpResponse<IOperation>) => {
            this.operation = res.body;
            // this.blockUpdate(doesUpdate);
          }
        );
    }
  }

  protected importOperationForUpdate(doesUpdate: boolean): void {
    if (doesUpdate) this.updateForm(this.operation);
    this.canUpdate = true;
    this.editorConfig.editable = true;
    this.editForm.enable()
    this.tryUpdate = false;
  }

  private blockUpdate(doesUpdate: boolean) {
    if (doesUpdate) this.updateForm(this.operation);
    if ((!this.operation?.modifyBlockingDate || moment().diff(this.operation?.modifyBlockingDate) / 1000 > 901) ||
      this.accountService.userIdentity?.login === this.operation?.modifyBlockingBy) {
      this.operationService.blockUpdate(this.operation).subscribe((res) => {
        if (this.accountService.userIdentity?.login !== this.operation?.modifyBlockingBy) this.operation = res.body;
        this.importOperationForUpdate(doesUpdate);
      });
    } else {
      this.tryUpdate = true;
      this.canUpdate = false;
      this.editorConfig.editable = false;
      this.editForm.disable();
    }
  }

  protected freeUpdate(doesUpdate: boolean) {
    if (this.accountService.userIdentity?.login === this.operation?.modifyBlockingBy) {
      this.operationService.freeUpdate(this.operation).subscribe((res) => {
        this.operation = res.body;
        this.importOperationForUpdate(doesUpdate);
      });
    }
  }

  async save(): Promise<void> {
    this.isSaving = true;
    const operation = this.createFromForm();
    let system = this.versionedSystems.find(sv => sv.id === operation.versionedSystem.id).system;
    if (operation.id !== undefined && operation.id !== null) {
      this.isAllowToSave(this.operationService.update(operation), system, CrudType.UPDATE);
    } else {
      operation.id = undefined;
      operation.versionedSystem.system.domaine = await this.loadDomaineAssociated(operation.versionedSystem.system);
      this.isAllowToSave(this.operationService.create(operation), system, CrudType.CREATE);
    }
  }

  cancel(): void {
    if (this.accountService.userIdentity?.login === this.operation?.modifyBlockingBy)
      this.operationService.freeUpdate(this.operation).subscribe(() => { });
  }

  protected onSaveSuccess(res: IOperation): void {
    this.isSaving = false;
    this.eventManager.broadcast({ name: 'operationListModification', content: res.id });
    this.eventManager.broadcast('historyListModification');

    if (this.changeOnSvOrCd) this.deployedVersionedSystemService.updateOperation({
      idOperation: res.id,
      sv: res.versionedSystem,
      cd: res.deployedCenter,
      bmsComplement: res.bmsComplement
    }).subscribe(res => { });
  }

  protected onSaveError(err): void {
    this.isSaving = false;
    console.error(err)
  }

  protected isAllowToSave(result: Observable<HttpResponse<IOperation>>, system: ISystem, crudType: CrudType) {
    if (this.isAdminBCS) this.subscribeToSaveResponse(result);
    else {
      this.groupUsersService.isAllowToCreateInBcsBySystem(system.id, crudType).subscribe(res => {
        this.subscribeToSaveResponse(result);
      },
        err => this.onSaveError(err));
    }
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IOperation>>): void {
    result.subscribe(
      (res) => this.onSaveSuccess(res.body)
      ,
      (err) => this.onSaveError(err)
    );
  }

  onChangeOnSvOrCd() {
    this.changeOnSvOrCd = true;
  }

  loadDomaineAssociated(system): Promise<Domaine> {
    return new Promise(resolve => {
      this.doamaineService.queryBySystemId(system.id).subscribe(res => {
        resolve(res.body);
      });
    });
  }
}