import { Component, EventEmitter, OnDestroy, OnInit, Output } from '@angular/core';
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';
import { AccountService } from 'src/app/core/auth/account.service';
import { DialogCancelComponent } from 'src/app/shared/dialog/cancel/dialog-cancel.component';
import { IOperation } from 'src/app/bcs_ng/model/operation.model';
import { OperationBaseUpdateComponent } from './operation-base-update.component';
import { OperationService } from './operation.service';
import { DeployedVersionedSystemService } from 'src/app/bmcs/entity/deployed-versioned-system/deployed-versioned-system.service';
import { GroupUsersService } from 'src/app/entities/group-users/group-users.service';
import { DomaineService } from 'src/app/entities/domaine/domaine.service';
import { BcsNgListOptionService } from '../../bcs-ng-list-option/bcs-ng-list-option.service';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';

@Component({
  selector: 'jhi-operation-detail-update',
  templateUrl: './operation-detail-update.component.html',
})
export class OperationDetailUpdateComponent extends OperationBaseUpdateComponent implements OnInit, OnDestroy {
  @Output() editor: EventEmitter<boolean> = new EventEmitter<boolean>()

  constructor(
    protected operationService: OperationService,
    protected accountService: AccountService,
    protected deployedVersionedSystemService: DeployedVersionedSystemService,
    protected bcsNgListOptionService: BcsNgListOptionService,
    protected activatedRoute: ActivatedRoute,
    protected fb: FormBuilder,
    protected eventManager: JhiEventManager,
    protected modalService: NgbModal,
    protected groupUsersService: GroupUsersService,
    protected doamaineService: DomaineService,
    protected bmcsBaseService: BmcsBaseService
  ) {
    super(operationService,
      deployedVersionedSystemService,
      bcsNgListOptionService,
      accountService,
      activatedRoute,
      fb,
      eventManager,
      groupUsersService,
      doamaineService,
      bmcsBaseService);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.registerChangeInOperations();
    this.editorConfig.height = '300px'
    this.editorConfig.minHeight = '300px'
    this.editorConfig.maxHeight = '500px'
  }

  ngOnDestroy(): void {
    if (this.eventSubscriber) {
      this.eventManager.destroy(this.eventSubscriber);
    }
  }

  onEdit(): void {
    this.freeUpdate(false)
    this.editor.emit(true);
  }

  saveForm(): void {
    this.save();
    this.editor.emit(true);
  }

  registerChangeInOperations(): void {
    this.eventSubscriber = this.eventManager.subscribe('operationListModification', () => this.loadPage(true));
    this.eventSubscriber = this.eventManager.subscribe('cancelListModification', () => this.onEdit());
  }

  focus(length: number): void {
    const element = document.querySelectorAll('[aria-label="multiselect-search"]') as NodeListOf<HTMLInputElement>;
    element[length].focus()
  }

  defineParamsDialog(modalRef, id): void {
    modalRef.componentInstance.id = id;
    modalRef.componentInstance.eventManagerName = "cancelListModification";
  }

  cancelChanges(operation: IOperation | undefined): void {
    const modalRef = this.modalService.open(DialogCancelComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, operation.id);
    modalRef.result.then(() => {
      if (this.accountService.userIdentity?.login === this.operation?.modifyBlockingBy)
        this.operationService.freeUpdate(this.operation).subscribe(() => { });
    });
  }
}