import { HttpResponse } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import * as moment from 'moment';
import { JhiEventManager } from 'ng-jhipster';
import { Subscription } from 'rxjs';
import { AccountService } from 'src/app/core/auth/account.service';
import { IHistoric } from 'src/app/bcs_ng/model/historic.model';

import { IOperation, Operation } from 'src/app/bcs_ng/model/operation.model';
import { OperationHistoricDialogComponent } from './dialog/operation-historic-dialog.component';
import { OperationService } from './operation.service';
import { Moment } from 'moment';
import { VersionedSystemService } from 'src/app/bmcs/entity/versioned-system/versioned-system.service';
import { IDeployedCenter } from 'src/app/bmcs/model/deployed-center.model';
import { DeployedCenterService } from 'src/app/bmcs/entity/deployed-center/deployed-center.service';
import { DomaineService } from 'src/app/entities/domaine/domaine.service';
import { IVersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { IDomaine } from 'src/app/shared/model/entity/domaine.model';
import { CruddomaineService } from 'src/app/entities/group-users/cruddomaine.service';
import { ICruddomaine } from 'src/app/shared/model/entity/cruddomaine.model';
import { MilestoneUpdateDialogComponent } from '../milestone/dialog/milestone-update-dialog.component';
import { BmcsBaseComponent } from 'src/app/bmcs/shared/bmcs-base-component';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { StatistiqueService } from 'src/app/admin/statistique/statistique.service';
import { BcsNgHistoryService } from '../../history/bcs-ng-history.service';
import { Baseline, valuesAndLabelsBaseline } from 'src/app/bcs_ng/model/enumeration/baseline.enum';

@Component({
  selector: 'jhi-operation-detail',
  templateUrl: './operation-detail.component.html',
  styleUrls: ['../../bcs-ng.scss']
})
export class OperationDetailComponent extends BmcsBaseComponent implements OnInit {
  id: string;
  operation: IOperation | null = null;
  baseline: Baseline = Baseline.NONE;
  Baseline = Baseline;
  valuesAndLabels = valuesAndLabelsBaseline;
  eventSubscriber: Subscription;
  editorTrue: boolean;
  tryUpdate: boolean = false;
  allowUpdate: boolean = false;
  numberCols: number = 5;
  heightHistoric: number;
  history: IHistoric;
  startDate: Moment;
  endDate: Moment;
  duration: string = ``;

  constructor(
    protected operationService: OperationService,
    protected versionedSystemService: VersionedSystemService,
    protected deployedCenterService: DeployedCenterService,
    protected domaineService: DomaineService,
    protected crudDomaineService: CruddomaineService,
    protected accountService: AccountService,
    protected bmcsBaseService: BmcsBaseService,
    protected statistiqueService: StatistiqueService,
    protected bcsNgHistoryService: BcsNgHistoryService,
    protected eventManager: JhiEventManager,
    protected activatedRoute: ActivatedRoute,
    protected modalService: NgbModal
  ) { super(accountService, statistiqueService, bmcsBaseService) }

  async ngOnInit(): Promise<void> {
    await super.ngOnInit();
    this.activatedRoute.paramMap.subscribe(params => {
      this.id = params.get("id");
      this.activatedRoute.queryParams
        .subscribe(params => {
          console.log(params);
          params?.baseline ? this.loadPageWithBaseline(params?.baseline) : this.loadPage(true);
        }
        );

    });
    this.registerChangeInOperations();
    this.activatedRoute.data.subscribe(({ operation }) => {
      this.operation = operation;
      const emptyOperation = new Operation();
      emptyOperation.milestones = operation.milestones;
      const operationDates = emptyOperation.getDates;
      this.startDate = operationDates[0];
      this.endDate = operationDates[1];
      if (this.startDate && this.endDate) {
        const duration = emptyOperation.getDuration;
        if (duration.years) this.duration += `${duration.years}` + (duration.years > 1 ? ` années ` : ` année `);
        if (duration.months) this.duration += `${duration.months} mois `;
        if (duration.days) this.duration += `${duration.days}` + (duration.days > 1 ? ` jours` : ` jour`);;
      }
    });
  }

  registerChangeInOperations(): void {
    this.eventSubscriber = this.eventManager.subscribe('operationListModification', () => this.loadPage(true));
  }

  previousState(): void {
    window.history.back();
  }

  onEdit(isFromUpdateComponent?: boolean): void {
    this.editorTrue = !this.editorTrue;
    this.numberCols = 5;
    this.loadPage(isFromUpdateComponent);
  }

  emitHistoryDetail(event: any) {
    this.history = event.history;
    const modalRef = this.modalService.open(OperationHistoricDialogComponent, { windowClass: 'xxlModal', backdrop: 'static' });
    modalRef.componentInstance.history = event.history;
    modalRef.componentInstance.historyNow = event.historyNow;
    modalRef.componentInstance.historyPrevious = event.historyPrevious;
  }

  showHistoric() {
    this.setHeightHistoric();
    this.numberCols = this.numberCols === 5 ? 6 : 5;
  }

  private setHeightHistoric() {
    this.heightHistoric = document.getElementById('detail')?.offsetHeight;
  }

  loadPageWithBaseline(baseline: Baseline) {
    this.baseline = baseline;
    this.bcsNgHistoryService.queryOperationAndMilestoneByBaseline(baseline, this.id).subscribe((res: HttpResponse<IOperation[]>) => {
      this.operation = res.body[0] || null;
      if (this.operation) this.operation.milestones = this.operation?.milestones;
      console.log(res.body, this.operation, this.operation.milestones)
    });
  }

  loadPage(dontBlock: boolean): void {
    if (this.id !== undefined) {
      this.operationService
        .find(this.id)
        .subscribe(
          (res: HttpResponse<IOperation>) => {
            this.operation = res.body;
            if (this.operation.versionedSystemId) {
              this.versionedSystemService.find(this.operation.versionedSystemId).subscribe((res: HttpResponse<IVersionedSystem>) => {
                this.operation.versionedSystem = res.body;
                this.domaineService.queryBySystemId(res.body.system.id).subscribe((res: HttpResponse<IDomaine>) => {
                  if (this.operation.versionedSystem?.system) this.operation.versionedSystem.system.domaine = res.body;
                  this.isAllowToUpdate();
                });
              });
            }
            if (this.operation.deployedCenterId) this.deployedCenterService.find(this.operation.deployedCenterId).subscribe((res: HttpResponse<IDeployedCenter>) => {
              this.operation.deployedCenter = res.body;
            });
            this.operation.milestones = this.operation.milestones?.sort((a, b) => (a.name ?? '').localeCompare(b.name ?? ''));
            if (!this.operation.deleted)
              this.operation.milestones = this.operation.milestones?.filter(m => !m.deleted);
            this.setHeightHistoric();
            if (!dontBlock) { // try to block if necessary
              if ((!this.operation?.modifyBlockingDate || moment().diff(this.operation?.modifyBlockingDate) / 1000 > 901) ||
                this.accountService.userIdentity?.login === this.operation?.modifyBlockingBy) {
                this.operationService.blockUpdate(this.operation).subscribe((res) => {
                  this.operation = res.body;
                  this.tryUpdate = false; //show the timer
                });
              } else { // the operation is blocked => show blocked message
                this.tryUpdate = true;
                this.editorTrue = false;
              }
            }
          });
    }
  }

  changeBaseline(baseline: Baseline) {
    this.baseline = baseline;
    this.baseline === Baseline.NONE ? this.loadPage(true) : this.loadPageWithBaseline(this.baseline);
  }

  isAllowToUpdate(): boolean {
    this.crudDomaineService.queryMyne({ where: this.accountService.userGroup })
      .subscribe(
        (res: HttpResponse<[ICruddomaine[]]>) => {
          const crudDomaine = res.body || [[]];
          if (this.operation) {
            for (let element of crudDomaine) {
              if (element.find(crud => crud.domaine.id === this.operation.versionedSystem?.system?.domaine?.id && crud.update === true)) this.allowUpdate = true;
            }
          }
        }
      );
    return true;
  }

  createMilestone(): void {
    const modalRef = this.modalService.open(MilestoneUpdateDialogComponent, { windowClass: 'xxlModal', backdrop: 'static' });
    modalRef.componentInstance.operationFromDetail = this.operation;
    modalRef.result
      .then(() => window.location.reload())
      .catch((err) => {
        if (err) console.error(err);
      })
  }
}
