import { Component, ViewChild } from '@angular/core';
import { FormBuilder } from '@angular/forms';
import { MatStepper } from '@angular/material/stepper';
import { ActivatedRoute } from '@angular/router';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';
import { AccountService } from 'src/app/core/auth/account.service';
import { IOperation } from 'src/app/bcs_ng/model/operation.model';
import { OperationBaseUpdateComponent } from './operation-base-update.component';
import { OperationService } from './operation.service';
import { DeployedVersionedSystemService } from 'src/app/bmcs/entity/deployed-versioned-system/deployed-versioned-system.service';
import { GroupUsersService } from 'src/app/entities/group-users/group-users.service';
import { DomaineService } from 'src/app/entities/domaine/domaine.service';
import { BcsNgListOptionService } from '../../bcs-ng-list-option/bcs-ng-list-option.service';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';


@Component({
  selector: 'jhi-operation-update',
  templateUrl: './operation-update.component.html'
})
export class OperationUpdateComponent extends OperationBaseUpdateComponent {
  @ViewChild('stepper') stepper: MatStepper;

  constructor(
    protected operationService: OperationService,
    protected deployedVersionedSystemService: DeployedVersionedSystemService,
    protected bcsNgListOptionService: BcsNgListOptionService,
    protected accountService: AccountService,
    protected activatedRoute: ActivatedRoute,
    protected fb: FormBuilder,
    protected eventManager: JhiEventManager,
    protected activeModal: NgbActiveModal,
    protected groupUsersService: GroupUsersService,
    protected doamaineService: DomaineService,
    protected bmcsBaseService: BmcsBaseService
  ) {
    super(operationService,
      deployedVersionedSystemService,
      bcsNgListOptionService,
      accountService,
      activatedRoute,
      fb,
      eventManager,
      groupUsersService,
      doamaineService,
      bmcsBaseService);
  }

  protected onSaveSuccess(res: IOperation): void {
    super.onSaveSuccess(res);
    this.activeModal.close();
  }

  cancel(): void {
    if (this.accountService.userIdentity?.login === this.operation?.modifyBlockingBy)
      this.operationService.freeUpdate(this.operation).subscribe(() => { });
    this.activeModal.dismiss();
  }

  previousState(): void {
    window.history.back();
  }

  isStep0Valid(): boolean {
    return this.editForm.get(['manual_id']).valid && !this.editForm?.hasError('duplicateManualId') &&
      this.editForm.get(['name']).valid &&
      this.editForm.get(['typeOperationId']).valid &&
      this.editForm.get(['versionedSystem']).valid;
  }
}
