import { Component, OnInit, OnDestroy, Input, ViewChild, EventEmitter, Output, AfterViewChecked } from '@angular/core';
import { fromEvent, Observable, Subscription } from 'rxjs';
import { JhiEventManager } from 'ng-jhipster';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { IOperation } from 'src/app/bcs_ng/model/operation.model';
import { OperationService } from './operation.service';
import { OperationUpdateDialogComponent } from './dialog/operation-update-dialog.component';
import { MatTableDataSource } from '@angular/material/table';
import { MatSort, Sort } from '@angular/material/sort';
import { MatPaginator, PageEvent } from '@angular/material/paginator';
import { MatTableFilter } from 'mat-table-filter';
import { AccountService } from 'src/app/core/auth/account.service';
import { FilterKey, StorageService } from 'src/app/blocks/storage/storage-service';
import { FilterWithAccents } from 'src/app/shared/util/filter-with-accents';
import { ExcelService } from 'src/app/shared/util/excel.service';
import { KeyValue } from '@angular/common';
import { IMilestone } from 'src/app/bcs_ng/model/milestone.model';
import * as moment from 'moment';
import { OperationHistoryListDialogComponent } from './dialog/operation-history-list-dialog.component';
import { IOperationCrud, OperationCrud } from 'src/app/bcs_ng/shared/dto/model/operation-crud.dto';
import { OperationDto } from 'src/app/bcs_ng/shared/dto/operation.dto';
import { SelectionModel } from '@angular/cdk/collections';
import { ITask } from 'src/app/bcs_ng/model/task/Itask.model';
import { IVersionedSystem, VersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { ISystem, System } from 'src/app/bmcs/model/system.model';
import { DialogDeleteBcsComponent } from 'src/app/bcs_ng/shared/dialog/delete/dialog-delete-bcs.component';
import { DialogRebornBcsComponent } from 'src/app/bcs_ng/shared/dialog/reborn/dialog-reborn-bcs.component';
import { DialogEraseBcsComponent } from 'src/app/bcs_ng/shared/dialog/erase/dialog-erase-bcs.component';
import { VersionedSystemService } from 'src/app/bmcs/entity/versioned-system/versioned-system.service';
import { DeployedCenter, IDeployedCenter } from 'src/app/bmcs/model/deployed-center.model';
import { DeployedCenterService } from 'src/app/bmcs/entity/deployed-center/deployed-center.service';
import { ISite, Site } from 'src/app/bmcs/model/site.model';
import { SaveScrollComponent } from 'src/app/shared/save-scroll.component';
import { routePath } from 'src/app/routes/path-routing.constant';
import { Domaine, IDomaine } from 'src/app/shared/model/entity/domaine.model';
import { ICruddomaine } from 'src/app/shared/model/entity/cruddomaine.model';
import { IBcsNgListOption } from 'src/app/bcs_ng/model/bcs-ng-list-option.model';
import { BcsNgListOptionService } from '../../bcs-ng-list-option/bcs-ng-list-option.service';
import { Baseline, RpsValidationScope } from 'src/app/bcs_ng/model/enumeration/baseline.enum';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { MatMenuTrigger } from '@angular/material/menu';
import { BcsNgHistoricScopeUpdateComponent } from '../../history/bcs-ng-historic-scope-update.component';
import { HttpResponse } from '@angular/common/http';

@Component({
  selector: 'jhi-operation',
  templateUrl: './operation.component.html',
  styleUrls: ['../../bcs-ng.scss']
})
export class OperationComponent extends SaveScrollComponent implements OnInit, AfterViewChecked, OnDestroy {
  // Inputs
  @Input() showButtons: boolean = true;
  @Input() sizeMatPaginator?: number;

  @Input() selectTable?: boolean = false;
  @Input() singleSelection?: boolean = false;
  @Input() selectedItem?: IOperation[] = [];
  @Input() onlyActive?: boolean = false;

  // Data to fetch
  @Input() operations?: IOperation[] = [];
  @Input() baseline: Baseline;
  @Input() versionedSystems?: IVersionedSystem[] = [];
  @Input() deployedCenters?: IDeployedCenter[] = [];
  @Input() crudDomaines?: [ICruddomaine[]];
  typeValues: IBcsNgListOption[] = [];
  Baseline = Baseline;

  // Page
  load: boolean = false;
  eventSubscriber?: Subscription;
  leftClickObservable: Observable<Event>;
  leftClickSubscription: Subscription;

  // Mat-table
  dataSource = new MatTableDataSource<IOperationCrud>([]);
  data: IOperationCrud[] = [];
  @ViewChild(MatSort) matSorter!: MatSort;
  activeSort: Sort = { active: 'id', direction: 'asc' };
  @ViewChild(MatPaginator, { static: true }) paginator!: MatPaginator;
  pageSize = 50;

  @ViewChild(MatMenuTrigger, { static: true }) matMenuTrigger: MatMenuTrigger;
  contextMenuPosition = { x: '0px', y: '0px' };

  allColumnsOperation = new Map([
    ['manual_id', 'ID'],
    ['domaine', 'Domaine'],
    ['system', 'Système'],
    ['versionedSystem', 'Système versionné'],
    ['deployedCenter', 'Centre déployé'],
    ['site', 'Site'],
    ['bmsComplement', 'Complément BMS'],
    ['name', 'Nom'],
    ['shortName', 'Abréviation'],
    ['startDate', 'Date de début'],
    ['endDate', 'Date de fin'],
    ['number', 'Numéro'],
    ['isBMS', 'BMS'],
    ['contractDGA', 'Contrat DGA'],
    ['program', 'Programme'],
    ['type', 'Type'],
    ['milestones', 'Jalons'],
    ['details', 'Détails']
  ]);
  displayedColumnsOperation: string[];
  public tempDisplayedCol: string[];
  public displayedSearchColumns: string[];
  filterEntity: IOperationCrud = new OperationCrud();
  filterType: MatTableFilter;
  propertyOptions: any;

  @Input() defaultDisplayColumns: string[]
    = ['manual_id', 'domaine', 'versionedSystem', 'deployedCenter', 'name', 'type', 'startDate', 'endDate', 'milestones'];
  defaultDisplayColumnsWithDetails: string[] = this.defaultDisplayColumns.concat(['details']);
  defaultSearchColumns: string[] = this.defaultDisplayColumns.map(c => `search-${c}`);
  defaultSearchColumnsWithDetails: string[] = this.defaultDisplayColumnsWithDetails.map(c => `search-${c}`);

  selection: SelectionModel<IOperationCrud>;
  @Output() onChangeSelectItem = new EventEmitter<ITask[]>();

  constructor(
    protected operationService: OperationService,
    protected versionedSystemService: VersionedSystemService,
    protected deployedCenterService: DeployedCenterService,
    protected bcsNgListOptionService: BcsNgListOptionService,
    protected accountService: AccountService,
    protected operationDto: OperationDto,
    protected storageService: StorageService,
    protected eventManager: JhiEventManager,
    protected modalService: NgbModal,
    protected filterWithAccents: FilterWithAccents,
    protected excelService: ExcelService,
    protected bmcsBaseService: BmcsBaseService
  ) {
    super(accountService, storageService, routePath.OPERATION);
  }

  //----------------------------------------------------------------------------------------------------------------------
  //---                                        Angular component lifecycle hooks                                        --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'lifecycle'

  ngOnInit(): void {
    if (this.sizeMatPaginator) this.pageSize = this.sizeMatPaginator;
    else this.pageSize = Number(this.storageService.get(FilterKey.NB_ITEMS)) || this.userPreferences?.nbItemTable;
    this.initFilterByColumn();
    if (this.selectTable) {
      this.selection = new SelectionModel<IOperationCrud>(!this.singleSelection, []);
      this.versionedSystems = this.bmcsBaseService.getVersionedSystems;
      this.deployedCenters = this.bmcsBaseService.getDeployedCenters;
      this.pageSize = 5;
      if (this.operations?.length === 0) this.loadAll();
      else this.updateOperations();
    }
    this.bcsNgListOptionService.query().subscribe(res => {
      this.typeValues = res.body.filter(optionList => optionList.type === 'OPERATION_TYPE');
    });
    this.leftClickObservable = fromEvent(window, 'click');
    this.leftClickSubscription = this.leftClickObservable.subscribe(event => {
      this.matMenuTrigger.closeMenu();
    })
  }

  ngOnChanges(): void {
    this.updateOperations(this.baseline != Baseline.NONE);
  }

  ngAfterViewChecked() {
    super.ngAfterViewChecked();
    if (this.selectTable) {
      const sortState: Sort = { active: 'select', direction: 'asc' };
      this.matSorter.active = sortState.active;
      this.matSorter.direction = sortState.direction;
      this.matSorter.sortChange.emit(sortState);
    }
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
    if (this.eventSubscriber)
      this.eventManager.destroy(this.eventSubscriber);
  }

  //#endregion

  //----------------------------------------------------------------------------------------------------------------------
  //---                                             LOAD & UPDATE DATA                                                  --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'loadData'

  protected updateTable(): void {
    this.load = true;
    this.dataSource.data = this.data || [];
    this.dataSource.sort = this.matSorter;
    this.dataSource.sortingDataAccessor = (data, columnDef) => {
      switch (columnDef) {
        case 'isBMS': return data.isBMS ? 0 : 1;
        case 'number': return data.number;
        case 'select': return this.selectedItem.find(i => i.id === data.id) ? 0 : 1
        default: return data[columnDef];
      }
    };
    if (!this.isScully) {
      this.paginator.pageSize = this.pageSize;
      this.dataSource.paginator = this.paginator;
      this.dataSource.paginator._intl.itemsPerPageLabel = "Items par page :";
    }
    this.load = false;
  }

  protected loadAll(): void {
    this.load = true;
    const getAll = this.onlyActive ? false : this.isAdminBCS;
    this.operationService.query(getAll).subscribe((res: HttpResponse<IOperation[]>) => {
      this.operations = res.body || [];
      this.updateOperations();
      this.load = false;
    });
  }

  protected updateOperations(isBaseline: boolean = false): void {
    this.data = this.operationDto.convertToCrud(this.operations, this.versionedSystems, this.deployedCenters, this.crudDomaines, isBaseline);
    this.updateTable();
    if (this.selectTable) {
      this.selectedItem.forEach(ope => {
        this.selection?.toggle(this.data.find(d => d.id === ope.id));
      });
    }
  }

  //#endregion
  //----------------------------------------------------------------------------------------------------------------------
  //---                                            USER MODIFICATIONS/ACTIONS                                           --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'modification'

  exportOperations(): void {
    const filteredOperations = this.operationDto.convertFromCrud(this.dataSource.filteredData);
    this.excelService.exportAsExcelFile(this.operationDto.convertOperationsDataToExcel(filteredOperations), 'Opérations-SCCOA');
  }

  defineParamsDialog(modalRef, id) {
    modalRef.componentInstance.id = id;
    modalRef.componentInstance.entityMsg = "cette opération";
    modalRef.componentInstance.serviceEntity = this.operationService;
    modalRef.componentInstance.eventManagerName = "operationListModification";
  }

  trackId(index: number, item: IOperation): string {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
    return item.id!;
  }

  update(operation: IOperation | undefined): void {
    const modalRef = this.modalService.open(OperationUpdateDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.operation = operation;
  }

  updateHistory(operation: IOperation): void {
    const modalRef = this.modalService.open(OperationHistoryListDialogComponent, { windowClass: 'xxlModal', backdrop: 'static' });
    modalRef.componentInstance.id = operation.id;
  }

  delete(operation: IOperation): void {
    const modalRef = this.modalService.open(DialogDeleteBcsComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, operation.id);
  }

  reborn(operation: IOperation): void {
    const modalRef = this.modalService.open(DialogRebornBcsComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, operation.id);
  }

  erase(operation: IOperation): void {
    const modalRef = this.modalService.open(DialogEraseBcsComponent, { size: 'lg', backdrop: 'static' });
    this.defineParamsDialog(modalRef, operation.id);
  }

  //#endregion
  //----------------------------------------------------------------------------------------------------------------------
  //---                                             CONTEXTUAL MENU                                                 --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'Contextual menu'
  onRightClick(event: any, row: IOperationCrud): void {
    this.matMenuTrigger?.closeMenu();
    event.preventDefault();
    this.contextMenuPosition.x = event.clientX + 'px';
    this.contextMenuPosition.y = event.clientY + 'px';
    this.matMenuTrigger.menuData = { 'item': row };
    this.matMenuTrigger.menu.focusFirstItem('mouse');
    if (this.baseline == 'NONE' && (this.isAdminBCS || row?.crudDomaine?.update))
      this.matMenuTrigger.openMenu();
  }

  setHistoricToBaseline(item: IOperationCrud, newBaselineValue?: Baseline): void {
    const modalRef = this.modalService.open(BcsNgHistoricScopeUpdateComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.id = item.id;
    modalRef.componentInstance.validationScope = RpsValidationScope.OPERATION;
    modalRef.componentInstance.newBaselineValue = newBaselineValue;
  }

  //#endregion
  //----------------------------------------------------------------------------------------------------------------------
  //---                                             FILTER ON TABLE                                                     --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'Filter'

  initFilterByColumn() {
    this.initAllFilters();
    //Récupère le type de filtre, ici anywhere donc on va garder les données contenant le filtre demandé
    this.filterType = MatTableFilter.ANYWHERE;
    this.setFilterOnInit();
    this.propertyOptions = {
      globalFilter: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.globalFilter);
      },
      manual_id: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.manual_id);
      },
      name: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.name);
      },
      versionedSystem: (route: IVersionedSystem) => {
        return this.filterWithAccents.compareStrings(route?.name, this.filterEntity.versionedSystem.name);
      },
      deployedCenter: (route: IDeployedCenter) => {
        return this.filterWithAccents.compareStrings(route?.name, this.filterEntity.deployedCenter.name);
      },
      domaine: (route: IDomaine) => {
        return this.filterWithAccents.compareStrings(route?.label, this.filterEntity.domaine.label);
      },
      site: (route: ISite) => {
        let routeSite = route?.fullName;
        if (route?.name) routeSite += route?.name;
        return this.filterWithAccents.compareStrings(routeSite, this.filterEntity.site.fullName);
      },
      system: (route: ISystem) => {
        return this.filterWithAccents.compareStrings(route?.name, this.filterEntity.system.name);
      },
      shortName: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.shortName);
      },
      bmsComplement: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.bmsComplement);
      },
      number: (route: number) => {
        return this.filterWithAccents.compareStrings(route.toString(), this.filterEntity.number.toString());
      },
      isBMS: (route: boolean) => {
        const bms_codeFilter: string[] = this.filterEntity.isBMS.toString().split(',');
        return bms_codeFilter.includes(route.toString());
      },
      contractDGA: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.contractDGA);
      },
      program: (route: string) => {
        return this.filterWithAccents.compareStrings(route, this.filterEntity.program);
      },
      typeOperation: (route: IBcsNgListOption) => {
        return this.filterWithAccents.compareStrings(route?.name, this.filterEntity.typeOperation?.toString());
      },
      milestones: (route: IMilestone[]) => {
        const filter = this.filterEntity.milestones.toString();
        const milestones = route.map(m => m.shortName).toString();
        return (filter === 'vide' && route.length === 0) || this.filterWithAccents.compareStrings(milestones, filter);
      },
      deleted: (route: boolean) => {
        return (this.filterEntity.deleted && route) || !this.filterEntity.deleted;
      }
    }
  }

  setFilterOnInit() {
    if (this.showButtons && !this.selectTable) {
      //get local storage filters
      const filters = this.storageService.get(FilterKey.OPE_FILTER);
      if (filters) {
        //S'il y a des filtres enregistrés, on les assignes à la liste des filtres par colonnes
        this.filterEntity = filters as IOperationCrud;
      }
      const sort = this.storageService.get(FilterKey.OPE_SORT);
      if (sort) this.activeSort = sort as Sort;
    }

    const columns = this.storageService.get(FilterKey.OPE_COLUMNS);
    if (columns && !this.selectTable) {
      //On assigne la liste des colonnes affichées sauvegardées au tableau des colonnes qui doivent être affichées
      this.displayedColumnsOperation = columns as string[];
      this.tempDisplayedCol = columns as string[];
      //Donne la liste des actions correspondantes et la sauvegarde
      this.changeColumns();
    } else if (this.isScully) {
      this.displayedColumnsOperation = this.defaultDisplayColumns;
      this.tempDisplayedCol = this.defaultDisplayColumns;
      this.displayedSearchColumns = this.defaultSearchColumns;
    } else {
      //S'il n'y a pas de colonnes sauvegardées, on affiche les colonnes préférentielles de l'utilisateur
      const colsPreferences = this.userPreferences?.operationColumns;
      if (colsPreferences && !this.selectTable) {
        this.displayedColumnsOperation = colsPreferences;
        this.tempDisplayedCol = colsPreferences;
        this.changeColumns();
      }
      else {
        if (this.selectTable) {
          this.displayedColumnsOperation = this.defaultDisplayColumns;
          this.displayedSearchColumns = this.defaultDisplayColumns.map(c => `search-${c}`);;
        } else {
          //S'il n'y a pas de colonnes sauvegardées et que l'utilisateur n'a pas de préférence, on affiche ces colonnes et leur jalons correspondantes
          this.displayedColumnsOperation = this.defaultDisplayColumnsWithDetails;
          this.tempDisplayedCol = this.defaultDisplayColumnsWithDetails;
          this.displayedSearchColumns = this.defaultSearchColumnsWithDetails;
        }

      }
    }
  }

  changeColumns() {
    //On assigne les filtres des colonnes qui doivent être affichées dans un tableau
    this.displayedSearchColumns = this.displayedColumnsOperation.map(col => 'search-' + col);
    //On sauvegarde ce tableau
    this.storageService.set(FilterKey.OPE_COLUMNS, this.displayedColumnsOperation);
  }

  asOrderCols(a: KeyValue<string, any>, b: KeyValue<string, any>): number {
    const aStepId = parseInt(a.key, 10);
    const bStepId = parseInt(b.key, 10);
    return aStepId > bStepId ? 1 : (bStepId > aStepId ? -1 : 0);
  }

  openedChange(opened: boolean) {
    if (!opened) {
      this.displayedColumnsOperation = this.tempDisplayedCol;
      this.changeColumns();
    }
  }

  deleteAllFilter() {
    this.initAllFilters();
    this.storeFilters();
  }

  initAllFilters() {
    this.filterEntity = new OperationCrud();
    this.filterEntity.deployedCenter = new DeployedCenter();
    this.filterEntity.site = new Site();
    this.filterEntity.versionedSystem = new VersionedSystem();
    this.filterEntity.system = new System();
    this.filterEntity.domaine = new Domaine();
    this.filterEntity.deleted = false;
  }

  changefilter() {
    this.storeFilters();
  }

  //Retourne si le filtre envoyé en paramètre doit être activé ou non
  activeFilter(filterEntityData): boolean {
    //S'il n'est pas déjà activé, on retourne true, sino on retourne false
    return filterEntityData === undefined || filterEntityData === null || filterEntityData === '' || filterEntityData?.length === 0
      ? false : true;
  }

  isAllFilterNull(): boolean {
    return (
      !this.activeFilter(this.filterEntity.manual_id) &&
      !this.activeFilter(this.filterEntity.name) &&
      !this.activeFilter(this.filterEntity.versionedSystem?.name) &&
      !this.activeFilter(this.filterEntity.deployedCenter?.name) &&
      !this.activeFilter(this.filterEntity.site?.name) &&
      !this.activeFilter(this.filterEntity.system?.name) &&
      !this.activeFilter(this.filterEntity.domaine?.label) &&
      !this.activeFilter(this.filterEntity.shortName) &&
      !this.activeFilter(this.filterEntity.startDate) &&
      !this.activeFilter(this.filterEntity.endDate) &&
      !this.activeFilter(this.filterEntity.bmsComplement) &&
      !this.activeFilter(this.filterEntity.number) &&
      !this.activeFilter(this.filterEntity.isBMS) &&
      !this.activeFilter(this.filterEntity.contractDGA) &&
      !this.activeFilter(this.filterEntity.program) &&
      !this.activeFilter(this.filterEntity.typeOperation)
    ) ? true : false;
  }

  updateFilterChangeCols(data: any) {
    let lastSelectedValue = data.value;
    const isChecked = data.selected;
    if (!isChecked) {
      switch (lastSelectedValue) {
        case 'startDate':
          this.filterEntity.startDate = undefined;
          break;
        case 'endDate':
          this.filterEntity.endDate = undefined;
          break;
      }
    }
  }

  //Fonction appelée lorsque l'on va utiliser un filtre sur les dates
  updatePropertyOptions(options, colName) {
    this.propertyOptions[colName] = options;
  }
  updateFilterEntity(newFilter, colName) {
    this.filterEntity[colName] = newFilter ? moment(newFilter) : undefined;
  }

  //----------------------------------------------------------------------------------------------------------------------
  //---                                            SAVE LOCAL STORAGE                                                   --        
  //----------------------------------------------------------------------------------------------------------------------
  //#region 'LocalStorage'

  storeFilters(): void {
    if (this.showButtons && !this.selectTable) {
      //Si tous les filtres ne sont pas désactivés, alors on remplace les filtres sauvegardés précédemment par la liste des filtres actuellements utilisés
      if (!this.isAllFilterNull()) this.storageService.set(FilterKey.OPE_FILTER, this.filterEntity);
      //Sinon, on supprime la liste des filtres sauvegardées
      else this.storageService.remove(FilterKey.OPE_FILTER);
    }
  }

  storeSort(sort: Sort): void {
    this.activeSort = sort;
    if (this.showButtons && !this.selectTable) {
      if (sort.active !== '' && sort.direction !== '') this.storageService.set(FilterKey.OPE_SORT, sort);
      else this.storageService.remove(FilterKey.OPE_SORT);
    }
  }

  getPaginatorChange(event: PageEvent): PageEvent {
    if (this.showButtons && !this.selectTable) this.storageService.set(FilterKey.NB_ITEMS, event.pageSize);
    return event;
  }

  //----------------------------------------------------------------------------------------------------------------------
  //---                                            SELECT ITEM                                                   --        
  //----------------------------------------------------------------------------------------------------------------------

  onChangeSelection(row) {
    this.selection.toggle(row);
    this.onChangeSelectItem.emit(this.selection.selected.map(t => t.operation));
  }
  /** Whether the number of selected elements matches the total number of rows. */
  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.dataSource.filteredData.length;
    return numSelected === numRows;
  }

  /** Selects all rows if they are not all selected; otherwise clear selection. */
  toggleAllRows() {
    if (this.isAllSelected()) {
      this.selection.clear();
      this.onChangeSelectItem.emit(this.selection.selected.map(t => t.operation));
      return;
    }
    this.selection.select(...this.dataSource.filteredData);
    this.onChangeSelectItem.emit(this.selection.selected.map(t => t.operation));
  }
}
