import { Injectable } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Resolve, ActivatedRouteSnapshot, Routes, Router } from '@angular/router';
import { Observable, of, EMPTY } from 'rxjs';
import { flatMap } from 'rxjs/operators';

import { AuthorityName } from 'src/app/shared/constants/authority.constants';
import { UserRouteAccessService } from 'src/app/core/auth/user-route-access-service';
import { IOperation, Operation } from 'src/app/bcs_ng/model/operation.model';
import { OperationService } from './operation.service';
import { OperationComponent } from './operation.component';
import { OperationDetailComponent } from './operation-detail.component';
import { OperationUpdateComponent } from './operation-update.component';
import { routePath } from 'src/app/routes/path-routing.constant';

@Injectable({ providedIn: 'root' })
export class OperationResolve implements Resolve<IOperation> {
  constructor(private service: OperationService, private router: Router) { }

  resolve(route: ActivatedRouteSnapshot): Observable<IOperation> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        flatMap((operation: HttpResponse<Operation>) => {
          if (operation.body) {
            return of(operation.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new Operation());
  }
}

export const operationRoute: Routes = [
  {
    path: routePath.OPERATION,
    component: OperationComponent,
    data: {
      authorities: [AuthorityName.USER_MOSS_SCCOA, AuthorityName.ADMIN, AuthorityName.USER_DGA_MQP],
      pageTitle: 'Opérations BCS'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: routePath.OPERATION + '/:id/view',
    component: OperationDetailComponent,
    resolve: {
      operation: OperationResolve
    },
    data: {
      authorities: [AuthorityName.USER_MOSS_SCCOA, AuthorityName.ADMIN, AuthorityName.USER_DGA_MQP],
      pageTitle: 'Opération BCS'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: routePath.OPERATION + '/new',
    component: OperationUpdateComponent,
    resolve: {
      operation: OperationResolve
    },
    data: {
      authorities: [AuthorityName.USER_MOSS_SCCOA],
      pageTitle: 'Opération BCS'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: routePath.OPERATION + '/:id/edit',
    component: OperationUpdateComponent,
    resolve: {
      operation: OperationResolve
    },
    data: {
      authorities: [AuthorityName.USER_MOSS_SCCOA],
      pageTitle: 'Opération BCS'
    },
    canActivate: [UserRouteAccessService]
  }
];
