import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';

import { TaskableEntityService } from './taskable-entity.service';
import { ITaskableEntity, TaskableEntity } from '../../model/taskable-entity.model';

@Component({
  selector: 'jhi-taskable-entity-update',
  templateUrl: './taskable-entity-update.component.html'
})
export class TaskableEntityUpdateComponent implements OnInit {
  isSaving = false;

  editForm = this.fb.group({
    id: [],
    name: []
  });

  constructor(protected taskableEntityService: TaskableEntityService, protected activatedRoute: ActivatedRoute, private fb: FormBuilder) { }

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ taskableEntity }) => {
      this.updateForm(taskableEntity);
    });
  }

  updateForm(taskableEntity: ITaskableEntity): void {
    this.editForm.patchValue({
      id: taskableEntity.id,
      name: taskableEntity.name
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const taskableEntity = this.createFromForm();
    if (taskableEntity.id !== undefined) {
      this.subscribeToSaveResponse(this.taskableEntityService.update(taskableEntity));
    } else {
      this.subscribeToSaveResponse(this.taskableEntityService.create(taskableEntity));
    }
  }

  private createFromForm(): ITaskableEntity {
    return {
      ...new TaskableEntity(),
      id: this.editForm.get(['id'])!.value,
      name: this.editForm.get(['name'])!.value
    };
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<ITaskableEntity>>): void {
    result.subscribe(
      () => this.onSaveSuccess(),
      () => this.onSaveError()
    );
  }

  protected onSaveSuccess(): void {
    this.isSaving = false;
    this.previousState();
  }

  protected onSaveError(): void {
    this.isSaving = false;
  }
}
