import { Component, OnInit, OnDestroy } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Subscription } from 'rxjs';
import { JhiEventManager } from 'ng-jhipster';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { TaskableEntityService } from './taskable-entity.service';
import { TaskableEntityDeleteDialogComponent } from './taskable-entity-delete-dialog.component';
import { ITaskableEntity } from '../../model/taskable-entity.model';

@Component({
  selector: 'jhi-taskable-entity',
  templateUrl: './taskable-entity.component.html'
})
export class TaskableEntityComponent implements OnInit, OnDestroy {
  taskableEntities?: ITaskableEntity[];
  eventSubscriber?: Subscription;

  constructor(
    protected taskableEntityService: TaskableEntityService,
    protected eventManager: JhiEventManager,
    protected modalService: NgbModal
  ) { }

  loadAll(): void {
    this.taskableEntityService.query().subscribe((res: HttpResponse<ITaskableEntity[]>) => (this.taskableEntities = res.body || []));
  }

  ngOnInit(): void {
    this.loadAll();
    this.registerChangeInTaskableEntities();
  }

  ngOnDestroy(): void {
    if (this.eventSubscriber) {
      this.eventManager.destroy(this.eventSubscriber);
    }
  }

  trackId(index: number, item: ITaskableEntity): string {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
    return item.id!;
  }

  registerChangeInTaskableEntities(): void {
    this.eventSubscriber = this.eventManager.subscribe('taskableEntityListModification', () => this.loadAll());
  }

  delete(taskableEntity: ITaskableEntity): void {
    const modalRef = this.modalService.open(TaskableEntityDeleteDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.taskableEntity = taskableEntity;
  }
}
