import { Injectable } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Resolve, ActivatedRouteSnapshot, Routes, Router } from '@angular/router';
import { Observable, of, EMPTY } from 'rxjs';
import { flatMap } from 'rxjs/operators';

import { TaskableEntityService } from './taskable-entity.service';
import { TaskableEntityComponent } from './taskable-entity.component';
import { TaskableEntityDetailComponent } from './taskable-entity-detail.component';
import { TaskableEntityUpdateComponent } from './taskable-entity-update.component';
import { ITaskableEntity, TaskableEntity } from '../../model/taskable-entity.model';
import { UserRouteAccessService } from 'src/app/core/auth/user-route-access-service';
import { AuthorityName } from 'src/app/shared/constants/authority.constants';

@Injectable({ providedIn: 'root' })
export class TaskableEntityResolve implements Resolve<ITaskableEntity> {
  constructor(private service: TaskableEntityService, private router: Router) { }

  resolve(route: ActivatedRouteSnapshot): Observable<ITaskableEntity> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        flatMap((taskableEntity: HttpResponse<TaskableEntity>) => {
          if (taskableEntity.body) {
            return of(taskableEntity.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new TaskableEntity());
  }
}

export const taskableEntityRoute: Routes = [
  {
    path: '',
    component: TaskableEntityComponent,
    data: {
      authorities: [AuthorityName.ADMIN_BCS],
      pageTitle: 'TaskableEntities'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: ':id/view',
    component: TaskableEntityDetailComponent,
    resolve: {
      taskableEntity: TaskableEntityResolve
    },
    data: {
      authorities: [AuthorityName.ADMIN_BCS],
      pageTitle: 'TaskableEntities'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: 'new',
    component: TaskableEntityUpdateComponent,
    resolve: {
      taskableEntity: TaskableEntityResolve
    },
    data: {
      authorities: [AuthorityName.ADMIN_BCS],
      pageTitle: 'TaskableEntities'
    },
    canActivate: [UserRouteAccessService]
  },
  {
    path: ':id/edit',
    component: TaskableEntityUpdateComponent,
    resolve: {
      taskableEntity: TaskableEntityResolve
    },
    data: {
      authorities: [AuthorityName.ADMIN_BCS],
      pageTitle: 'TaskableEntities'
    },
    canActivate: [UserRouteAccessService]
  }
];
