import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';
import { TaskableEntity } from 'src/app/bcs_ng/model/taskable-entity.model';
import { SERVER_BCS_URL } from 'src/app/app.constants';
import { createRequestOption } from 'src/app/shared/util/request-util';


type EntityResponseType = HttpResponse<TaskableEntity>;
type EntityArrayResponseType = HttpResponse<TaskableEntity[]>;

@Injectable({ providedIn: 'root' })
export class TaskableEntityService {
  public resourceUrl = SERVER_BCS_URL + 'api/taskable-entities';

  constructor(protected http: HttpClient) { }

  create(taskableEntity: TaskableEntity): Observable<EntityResponseType> {
    return this.http.post<TaskableEntity>(this.resourceUrl, taskableEntity, { observe: 'response' });
  }

  update(taskableEntity: TaskableEntity): Observable<EntityResponseType> {
    return this.http.put<TaskableEntity>(this.resourceUrl, taskableEntity, { observe: 'response' });
  }

  find(id: string): Observable<EntityResponseType> {
    return this.http.get<TaskableEntity>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http.get<TaskableEntity[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  delete(id: string): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }
}
