import { TaskableEntityTypeService } from '../../shared/services/taskable-entity-type.service';
import { IBryntumEvent } from './bryntum-event.model';
import { IBryntumOverridableData } from './bryntum-overridable-data.model';
import { IBryntumResource } from './bryntum-resource.model';

export interface IBryntumRecordState {
    id: string;
    toErase: boolean;
    isUpdated: boolean; // through form
    resource: IBryntumResource;
    event: IBryntumEvent;
}

export class BryntumRecordState implements IBryntumRecordState {
    constructor(
        public id: string,
        public toErase: boolean,
        public isUpdated: boolean,
        public resource: IBryntumResource,
        public event: IBryntumEvent,
    ) { }

    public get getOverridableData(): IBryntumOverridableData {
        if (this.resource.object) {
            const startDate = this.event?.startDate ?? new Date(this.resource.object?.startDate?.toString());
            let endDate = startDate;
            if (!TaskableEntityTypeService.isMilestone(this.resource.typeId))
                endDate = this.event.endDate ?? new Date(this.resource.object?.endDate?.toString());
            let name;
            if (TaskableEntityTypeService.isDomaine(this.resource.typeId)) {
                name = (this.resource.name && this.resource.name.length) ? this.resource.name : this.resource.object.domaine;
            } else {
                name = (this.resource.name && this.resource.name.length) ? this.resource.name : this.resource.object.name;
            }
            return {
                name: name,
                shortName: (this.event.name && this.event.name.length) ? this.event.name : this.resource.object.shortName ?? this.resource.object.name,
                startDate,
                endDate,
            }
        }
        else return {
            name: this.resource.name ?? null,
            shortName: this.event.name ?? null,
            startDate: this.event.startDate ?? null,
            endDate: this.event.endDate ?? null,
        }
    }
}