import { TaskableEntityTypeService } from '../shared/services/taskable-entity-type.service';
import { IAdditionalProperties } from './bryntum/additional-properties.model';
import { IBryntumOverridableData } from './bryntum/bryntum-overridable-data.model';
import { DisplayConfiguration } from './display-configuration.model';
import { IRoadMapBcs } from './road-map-bcs.model';
import { GenericTask } from './task/generic-task.model';
import { ITask } from './task/Itask.model';
import { ITaskableEntity } from './taskable-entity.model';


export interface IDisplayTask extends GenericTask {
    id?: string;
    type?: ITaskableEntity;
    parentId?: string;
    resourceId?: string[];
    defaultDisplayConfig?: DisplayConfiguration;
    displayConfig?: DisplayConfiguration;
    roadMap?: IRoadMapBcs;
    object?: ITask;
    rpsObjects?: ITask[]; // None to two previous RPS objects
    additionalProperties?: string;
}

export class DisplayTask extends GenericTask {
    id?: string;
    type?: ITaskableEntity;
    parentId?: string;
    resourceId?: string[];
    defaultDisplayConfig?: DisplayConfiguration;
    displayConfig?: DisplayConfiguration;
    roadMap?: IRoadMapBcs;
    object?: ITask;
    rpsObjects?: ITask[];
    additionalProperties?: string;

    public get getOverridableData(): IBryntumOverridableData {
        const startDate = this.startDate ? new Date(this.startDate.toString()) : new Date(this.object.startDate?.toString());
        let endDate = startDate;
        if (!TaskableEntityTypeService.isMilestone(this.type.id))
            endDate = this.endDate ? new Date(this.endDate.toString()) : new Date(this.object.endDate?.toString());
        return {
            name: (this.name && this.name.length) ? this.name : this.object.name,
            shortName: (this.name && this.name.length) ? this.name : this.object.shortName,
            startDate,
            endDate,
        }
    }

    public get getAdditionalProperties(): IAdditionalProperties {
        const parsed = JSON.parse(this.additionalProperties);
        return {
            hidden: parsed.hidden ?? false,
            hidePredecessors: parsed.hidePredecessors ?? false,
            expanded: parsed.expanded ?? false,
            parentIndex: parsed.parentIndex ?? undefined,
            independantMilestone: parsed.independantMilestone ?? false,
            resourcesId: parsed.resourcesId ?? null
        }
    }
}
