import { Moment } from 'moment';
import { IDeployedCenter } from 'src/app/bmcs/model/deployed-center.model';
import { IVersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { Utils } from '../utils/util';
import { IBcsNgListOption } from './bcs-ng-list-option.model';
import { IMilestone } from './milestone.model';
import { GenericTask } from './task/generic-task.model';

export interface IOperation extends GenericTask {
  manual_id?: string;
  description?: string;
  number?: number;
  isBMS?: boolean;
  contractDGA?: string;
  program?: string;
  typeOperation?: IBcsNgListOption;
  milestones?: IMilestone[];
  versionedSystemId?: string;
  versionedSystem?: IVersionedSystem;
  deployedCenterId?: string;
  deployedCenter?: IDeployedCenter;
  bmsComplement?: string;
}

export class Operation extends GenericTask implements IOperation {
  manual_id?: string;
  description?: string;
  number?: number;
  isBMS?: boolean;
  contractDGA?: string;
  program?: string;
  typeOperation?: IBcsNgListOption;
  milestones?: IMilestone[];
  versionedSystemId?: string;
  versionedSystem?: IVersionedSystem;
  deployedCenterId?: string;
  deployedCenter?: IDeployedCenter;
  bmsComplement?: string;

  super() { }

  public get getDates(): Moment[] {
    const milestonesList = this.milestones ?? [];
    if (milestonesList.length === 0) return [null, null];
    const sorted = milestonesList.sort((milestoneA, milestoneB) => {
      return milestoneA.startDate < milestoneB.startDate ? -1 : 1;
    }).map(milestone => milestone.startDate);
    return [sorted[0], sorted[sorted.length - 1]];
  }

  public get getDuration(): { years: number, months: number, days: number } {
    const dates = this.getDates;
    if (!dates[0] || !dates[1]) return { years: 0, months: 0, days: 0 };
    return Utils.calculateMomentPreciseDiff(dates[0], dates[1]);
    //return Utils.calculateMomentDiff(dates[0], dates[1]);
  }
}
