import { Injectable } from '@angular/core';
import { IOperation, Operation } from '../../model/operation.model';
import { IMilestone, Milestone } from '../../model/milestone.model';
import { BmcsBaseService } from 'src/app/bmcs/shared/bmcs-base.service';
import { IDeployedCenter } from 'src/app/bmcs/model/deployed-center.model';
import * as moment from 'moment';
import { IBcsNgListOption } from '../../model/bcs-ng-list-option.model';
import { IVersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { ExcelService } from 'src/app/shared/util/excel.service';
import { Utils } from '../../utils/util';
import { FilterWithAccents } from 'src/app/shared/util/filter-with-accents';
import { Dependency } from '../../model/dependency.model';
import { TimeUnit } from '../../model/enumeration/time-unit.enum';
import { DependencyType } from '../../model/enumeration/dependency-type.enum';


@Injectable()
export class BcsDataDto {

    deployedCenters: IDeployedCenter[] = [];
    versionedSystems: IVersionedSystem[] = [];
    operations: IOperation[] = [];
    milestones: IMilestone[] = [];
    bcsNgListOptions: IBcsNgListOption[] = [];

    constructor(protected bmcsBaseService: BmcsBaseService,
        private excelService: ExcelService,
        private filterWithAccents: FilterWithAccents) { }

    public convertExcelToBcsData(mspDataExcel: any[] | [], bcsNgListOptions: IBcsNgListOption[]): [IOperation[], IMilestone[]] {
        const jsonString = JSON.parse(JSON.stringify(mspDataExcel));
        this.deployedCenters = this.bmcsBaseService.getDeployedCenters;
        this.versionedSystems = this.bmcsBaseService.getVersionedSystems;
        this.bcsNgListOptions = bcsNgListOptions;
        let currentSystem, currentSV, versionBMS, version, operation;

        for (let i = 0; i < jsonString.length; i++) {
            const dataType = jsonString[i]["Type pour BCS NG"];
            const taskName = jsonString[i]["Nom tâche"]
            if (taskName && !this.compareStrings(taskName, 'Barre de fond') && !this.compareStrings(taskName, 'Début FDR')) {
                if (dataType == 'Système') {
                    currentSystem = taskName;
                    operation = null;
                }
                if (dataType == 'Version') {
                    versionBMS = jsonString[i]["Version pour BMS"] ?? taskName;
                    version = taskName
                    currentSV = this.findVersionedSystem(currentSystem + ' ' + versionBMS);
                }
                if (dataType == 'Opération' && jsonString[i]["Niveau hierarchique"] === 4) {
                    const deployedCenter = this.findDeployedCenter(jsonString[i]["Centre déployé"]);
                    let name = taskName;
                    if (versionBMS != version) name += ' - ' + version;
                    operation = {
                        ...new Operation(),
                        id: Utils.createUUID(),
                        manual_id: 'O-' + jsonString[i]["ID_TPI"],
                        shortName: jsonString[i]["Abreviation"],
                        number: jsonString[i]["ID_TPI"],
                        name: name,
                        typeOperation: this.findOperationType(jsonString[i]["Type dopération"] ?? taskName, jsonString[i]["Centre déployé"] ?? null),
                        isBMS: jsonString[i]["Inclus BMS"] && jsonString[i]["Inclus BMS"] == 'OUI' ? true : false,
                        contractDGA: jsonString[i]["Contrat DGA en cours"] && jsonString[i]["Contrat DGA en cours"] != 'NA' ? jsonString[i]["Contrat DGA en cours"] : null,
                        program: this.findProgram(jsonString[i]["Programme"], jsonString[i]["Etape du programme"]),
                        description: this.createDescription(jsonString[i]["Objet de l'opération"], jsonString[i]["Commentaires"]),
                        lastModifiedDate: jsonString[i]["Dernière modification"] && jsonString[i]["Dernière modification"] != 'NC' ? moment(new Date(this.excelService.parseDateExcel(jsonString[i]["Dernière modification"])), "DD/MM/YYYY") : null,
                        createdDate: jsonString[i]["Date de création"] && jsonString[i]["Date de création"] != 'NC' ? moment(new Date(this.excelService.parseDateExcel(jsonString[i]["Date de création"])), "DD/MM/YYYY") : null,
                        deployedCenterId: deployedCenter?.id,
                        deployedCenter: deployedCenter,
                        versionedSystemId: currentSV?.id,
                        versionedSystem: currentSV,
                        bmsComplement: jsonString[i]["Complément BMS"]
                    };
                    this.operations.push(operation);
                }
                if (dataType == 'Jalon') {
                    let idMilestone;
                    if (this.milestones.find(m => m.number === jsonString[i]["ID_TPI"])) {
                        //if the milestone has already been created when adding the dependencies : recover id & delete futur duplicate
                        let index = this.milestones.findIndex(m => m.number === jsonString[i]["ID_TPI"]);
                        idMilestone = this.milestones[index].id;
                        this.milestones.splice(index, 1);
                    } else idMilestone = Utils.createUUID();
                    let predecessors = this.findPredecessors(idMilestone, this.findDigitRoadMap(jsonString[i]["ID_TPI"]), jsonString, jsonString[i]["Prédécésseurs"], operation);
                    let milestone = {
                        ...new Milestone(),
                        id: idMilestone,
                        startDate: jsonString[i]["Début"] && jsonString[i]["Début"] != 'NC' ? moment(new Date(this.excelService.parseDateExcel(jsonString[i]["Début"])), "DD/MM/YYYY") : null,
                        endDate: jsonString[i]["fin"] && jsonString[i]["fin"] != 'NC' ? moment(new Date(this.excelService.parseDateExcel(jsonString[i]["fin"])), "DD/MM/YYYY") : null,
                        contractDate: jsonString[i]["Date contractuelle"] && jsonString[i]["Date contractuelle"] != 'NC' ? moment(new Date(this.excelService.parseDateExcel(jsonString[i]["Date contractuelle"])), "DD/MM/YYYY") : null,
                        shortName: jsonString[i]["Abreviation"],
                        number: jsonString[i]["ID_TPI"],
                        name: taskName,
                        comment: jsonString[i]["Commentaires"],
                        lastModifiedDate: jsonString[i]["Dernière modification"] && jsonString[i]["Dernière modification"] != 'NC' ? moment(new Date(this.excelService.parseDateExcel(jsonString[i]["Dernière modification"])), "DD/MM/YYYY") : null,
                        createdDate: jsonString[i]["Date de création"] && jsonString[i]["Date de création"] != 'NC' ? moment(new Date(this.excelService.parseDateExcel(jsonString[i]["Date de création"])), "DD/MM/YYYY") : null,
                        source: this.findSource(jsonString[i]["Source"]),
                        typeMilestone: this.findMilestoneType(taskName),
                        operation: operation,
                        previousRPS: jsonString[i]["Date RPS n-1"] && jsonString[i]["Date RPS n-1"] != 'NC' ? moment(new Date(this.excelService.parseDateExcel(jsonString[i]["Date RPS n-1"])), "DD/MM/YYYY") : null,
                        predecessorDependencies: predecessors,
                    };
                    this.milestones.push(milestone);
                }
            }
        }
        console.log(this.operations, this.milestones);
        return [this.operations, this.milestones];
    }

    public findDeployedCenter(name: string) {
        return name ? this.deployedCenters.find(cd => this.compareStrings(cd.name, name)) : null;
    }

    public findVersionedSystem(name: string) {
        return name ? this.versionedSystems.find(sv => this.filterWithAccents.equalsStrings(sv.name, name)) : null;
    }

    public findSource(name: string): IBcsNgListOption {
        const milestoneSource = this.bcsNgListOptions.filter(o => o.type == 'MILESTONE_SOURCE');
        if (this.compareStrings(name, 'Validé DGA')) return milestoneSource.find(s => s.label == 'MOSS_COLLEC');
        else if (this.compareStrings(name, 'Recueil Industriel')) return milestoneSource.find(s => s.label == 'MOSS_COLLEC');
        else if (this.compareStrings(name, 'Recueil Administration')) return milestoneSource.find(s => s.label == 'DGA_COLLEC');
        else if (this.compareStrings(name, 'Contrat MOSS')) return milestoneSource.find(s => s.label == 'MOSS_COLLEC');
        else if (this.compareStrings(name, 'Estimé MOSS')) return milestoneSource.find(s => s.label == 'ESTIMATION');
        else if (this.compareStrings(name, 'Gestion de configuration')) return milestoneSource.find(s => s.label == 'MOSS_COLLEC');
        else if (this.compareStrings(name, 'Arbitraire')) return milestoneSource.find(s => s.label == 'ESTIMATION');
        else return milestoneSource.find(s => s.label == 'UNKNOWN');
    }

    public findOperationType(name: string, cd: string) {
        const operationType = this.bcsNgListOptions.filter(o => o.type == 'OPERATION_TYPE');
        if (this.compareStrings(name, 'Réalisation') || name == 'Réalisation') return operationType.find(t => t.label == 'REALISATION');
        else if (this.compareStrings(name, 'Déploiement') || cd) return operationType.find(t => t.label == 'DEPLOIEMENT');
        else return operationType.find(t => t.label == 'UNKNOWN');
    }

    public findMilestoneType(name: string) {
        const milestoneType = this.bcsNgListOptions.filter(o => o.type == 'MILESTONE_TYPE');
        if (this.compareStrings(name, "CGC")) return milestoneType.find(t => t.label == "CGC");
        else if (this.compareStrings(name, "GFE")) return milestoneType.find(t => t.label == "GFE");
        else if (this.compareStrings(name, "Lancement procédure contractuelle") || this.compareStrings(name, "LPC")) return milestoneType.find(t => t.label == "LPC");
        else if (this.compareStrings(name, "Date de notification") || this.compareStrings(name, "NOTIF")) return milestoneType.find(t => t.label == "NOTIF");
        else if (this.compareStrings(name, "Interface préliminaire") || this.compareStrings(name, "IFP")) return milestoneType.find(t => t.label == "IFP");
        else if (this.compareStrings(name, "Revue de conception") || this.compareStrings(name, "RCD")) return milestoneType.find(t => t.label == "RCD");
        else if (this.compareStrings(name, "Définition des interfaces figée") || this.compareStrings(name, "DIFF")) return milestoneType.find(t => t.label == "DIFF");
        else if (this.compareStrings(name, "Présentation aux Opérations de Vérification") || this.compareStrings(name, "POV")) return milestoneType.find(t => t.label == "POV");
        else if (this.compareStrings(name, "Date de début d'indisponibilité") || this.compareStrings(name, "DDI")) return milestoneType.find(t => t.label == "DDI");
        else if (this.compareStrings(name, "Date de fin d'indisponibilité") || this.compareStrings(name, "DFI")) return milestoneType.find(t => t.label == "DFI");
        else if (this.compareStrings(name, "Réception") || this.compareStrings(name, "REC")) return milestoneType.find(t => t.label == "REC");
        else if (this.compareStrings(name, "Prononcé de qualification DGA") || this.compareStrings(name, "PQE")) return milestoneType.find(t => t.label == "PQE");
        else if (this.compareStrings(name, "Mise en service opérationnel") || this.compareStrings(name, "MSO")) return milestoneType.find(t => t.label == "MSO");
        else if (this.compareStrings(name, "Date de retrait de service") || this.compareStrings(name, "DRS")) return milestoneType.find(t => t.label == "DRS");
        else if (this.compareStrings(name, "Jalon infra") || this.compareStrings(name, "INFRA")) return milestoneType.find(t => t.label == "INFRA");
        else if (this.compareStrings(name, "Jalon ext") || this.compareStrings(name, "JEXT")) return milestoneType.find(t => t.label == "JEXT");
        else if (this.compareStrings(name, "Obsolescence") || this.compareStrings(name, "OBSO")) return milestoneType.find(t => t.label == "OBSO");
        else if (this.compareStrings(name, "Revue de Définition Préliminaire") || this.compareStrings(name, "RDP")) return milestoneType.find(t => t.label == "RDP");
    }

    findProgram(program: string, programStep: string) {
        if (!program) return undefined;
        if (programStep) program += ' ' + programStep;
        return program;
    }

    createDescription(object: string, comments: string) {
        let description = "";
        if (object) description += object;
        if (object && comments) description += "<br>"
        if (comments) description += comments;
        return description;
    }

    public compareStrings(source: string, target: string): boolean {
        return this.filterWithAccents.compareStrings(source, target, true);
    }

    public findDigitRoadMap(number: number): number {
        let digit = number.toString()[0] + '00000';
        return Number(digit);
    }

    public findPredecessors(idMilestone: string, digitRoadMap: number, json: any[], predecessorsIds: string, operation: IOperation): Dependency[] {
        let result: Dependency[] = [];
        if (!predecessorsIds) return null;
        let listPredecessors = predecessorsIds.toString().split(';');
        listPredecessors = listPredecessors.filter(s => !s.startsWith('Planning'));
        if (!listPredecessors && listPredecessors?.length == 0) return null;
        listPredecessors.forEach(predecessorString => {
            let predecessorIdAndTimeGap = predecessorString?.split('FD+');
            let predecessor = this.milestones.find(m => m.number === digitRoadMap + Number(predecessorIdAndTimeGap[0]));
            if (!predecessor) {
                json.forEach(j => {
                    if (j["Type pour BCS NG"] === 'Jalon' && (j["ID_TPI"] === digitRoadMap + Number(predecessorIdAndTimeGap[0]))) {
                        let newMilestone = {
                            ... new Milestone(),
                            id: Utils.createUUID(),
                            number: j["ID_TPI"],
                            name: j["Nom tâche"],
                            typeMilestone: this.findMilestoneType(j["Nom tâche"]),
                        }
                        predecessor = newMilestone;
                        this.milestones.push(newMilestone);
                    }
                });
            }
            if (predecessor)
                result.push(this.getDependencies(predecessor, predecessorIdAndTimeGap, idMilestone, operation));
        });
        return result;
    }

    public getDependencies(predecessor: IMilestone, predecessorIdAndTimeGap: string[], idMilestone: string, operation: IOperation): Dependency {
        let timeGapDuration = 0;
        let timeGapUnit = TimeUnit.MONTHS;
        let typeDependancy = predecessor?.operation?.id === operation?.id ? DependencyType.INTRA : DependencyType.INTER;
        if (predecessorIdAndTimeGap[1]) {
            let tempGapDuration = predecessorIdAndTimeGap[1]?.split(' ');
            timeGapDuration = Number(tempGapDuration[0]);
            if (tempGapDuration[1] === 'jrs') timeGapUnit = TimeUnit.DAYS;
            if (tempGapDuration[1] === 'sem.') {
                timeGapDuration = timeGapDuration * 7;
                timeGapUnit = TimeUnit.DAYS;
            }
        }
        return {
            ... new Dependency(),
            predecessor: { id: predecessor?.id, name: predecessor?.name, number: predecessor?.number, typeMilestone: predecessor?.typeMilestone },
            successor: { id: idMilestone, name: null },
            timeGapDuration: timeGapDuration,
            timeGapUnit: timeGapUnit,
            type: typeDependancy
        };
    }
}