import { Injectable } from '@angular/core';
import * as moment from 'moment';
import { GenericTask, IGenericTask } from '../../model/task/generic-task.model';
import { GenericTaskCrud, IGenericTaskCrud } from './model/generic-task-crud.dto';
import { GenericTaskExcel, IGenericTaskExcel } from './model/generic-task-excel.model';

@Injectable()
export class GenericTaskDto {
    public convertGenericTasksDataToExcel(genericTasks: IGenericTask[] | null): IGenericTaskExcel[] {
        return genericTasks.map(genericTask => {
            return new GenericTaskExcel(
                genericTask.name,
                genericTask.shortName,
                genericTask.startDate ? moment(genericTask.startDate).format('DD/MM/YYYY') : '',
                genericTask.endDate ? moment(genericTask.endDate).format('DD/MM/YYYY') : '',
                genericTask.comment,
            )
        })
    }

    convertToCrud(genericTasks: IGenericTask[]): IGenericTaskCrud[] {
        return genericTasks?.map(genericTask => {
            const globalText: string = genericTask.id + genericTask.name + genericTask.shortName;
            return new GenericTaskCrud(
                genericTask.id,
                genericTask,
                genericTask.deleted,
                globalText,
                genericTask.name,
                genericTask.shortName,
                genericTask.startDate,
                genericTask.endDate,
                genericTask.comment,
            );
        })
    }

    convertFromCrud(genericTasksCrud: IGenericTaskCrud[]): IGenericTask[] {
        return genericTasksCrud.map(genericTaskCrud => {
            const genericTask = new GenericTask();
            genericTask.id = genericTaskCrud.id;
            genericTask.deleted = genericTaskCrud.deleted;
            genericTask.name = genericTaskCrud.name;
            genericTask.shortName = genericTaskCrud.shortName;
            genericTask.startDate = genericTaskCrud.startDate;
            genericTask.endDate = genericTaskCrud.endDate;
            genericTask.comment = genericTaskCrud.comment;
            return genericTask;
        })
    }
}