import { Injectable } from '@angular/core';
import * as moment from 'moment';
import { IMilestone, Milestone } from 'src/app/bcs_ng/model/milestone.model';
import { IDeployedCenter } from 'src/app/bmcs/model/deployed-center.model';
import { IVersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { Cruddomaine, ICruddomaine } from 'src/app/shared/model/entity/cruddomaine.model';
import { Operation } from '../../model/operation.model';

import { IMilestoneCrud, MilestoneCrud } from './model/milestone-crud.dto';
import { IMilestoneExcel, MilestoneExcel } from './model/milestone-excel.model';

@Injectable()
export class MilestoneDTO {

    public convertMilestonesDataToExcel(milestones: IMilestone[] | null): IMilestoneExcel[] {
        return milestones.map(milestone => {
            return new MilestoneExcel(
                milestone?.operation?.versionedSystem?.system?.domaine?.domaine,
                milestone?.operation?.versionedSystem?.system?.name,
                milestone?.operation?.versionedSystem?.name,
                milestone?.operation?.deployedCenter?.name,
                milestone?.operation?.deployedCenter?.site?.name,
                milestone?.operation?.name,
                milestone?.name,
                milestone?.shortName,
                milestone.comment,
                milestone?.startDate ? moment(milestone.startDate).format('DD/MM/YYYY') : '',
                milestone?.contractDate ? moment(milestone.contractDate).format('DD/MM/YYYY') : '',
                milestone.typeMilestone?.name,
                milestone.source?.name,
                milestone?.predecessorDependencies?.reduce((total, dep) => total + dep.predecessor.name + ', ', ''),
                milestone?.successorDependencies?.reduce((total, dep) => total + dep.successor.name + ', ', ''),

            );
        });
    }

    convertToCrud(milestones: IMilestone[], versionedSystems: IVersionedSystem[], deployedCenters: IDeployedCenter[], crudDomaines: [Cruddomaine[]], isBaseline: boolean = false): IMilestoneCrud[] {
        const crud: ICruddomaine[] = [];
        crudDomaines?.forEach(cruddomaineList => {
            cruddomaineList.forEach(cruddomaine => {
                const index = crud.findIndex(cr => cr.domaine.id === cruddomaine.domaine.id);
                if (index === -1)
                    crud.push(cruddomaine);
                else {
                    crud[index].create = cruddomaine.create;
                    crud[index].update = cruddomaine.update;
                    crud[index].deleted = cruddomaine.deleted;
                    crud[index].erase = cruddomaine.erase;
                    crud[index].is_referent = cruddomaine.is_referent;
                }
            });
        });
        return milestones?.map(milestone => {
            const verrsionedSystem = isBaseline ? milestone?.operation?.versionedSystem : versionedSystems.find(dvs => dvs.id === milestone?.operation?.versionedSystemId?.toUpperCase());
            const deployedCenter = isBaseline ? milestone?.operation?.deployedCenter : deployedCenters.find(cd => cd.id === milestone?.operation?.deployedCenterId?.toUpperCase());
            const domaine = verrsionedSystem?.system?.domaine;
            const globalText: string = milestone.name + milestone.shortName + milestone.typeMilestone?.name + milestone.source?.name + milestone?.operation?.name +
                verrsionedSystem?.name + verrsionedSystem?.system?.name + domaine?.domaine + deployedCenter?.name + deployedCenter?.site?.name;

            return new MilestoneCrud(
                milestone?.id,
                milestone?.deleted,
                milestone,
                globalText,
                milestone?.name,
                milestone?.number,
                milestone?.shortName,
                milestone?.comment,
                milestone?.startDate,
                milestone?.contractDate,
                milestone?.typeMilestone,
                milestone?.source,
                milestone?.predecessorDependencies,
                milestone?.successorDependencies,
                milestone?.operation?.name,
                verrsionedSystem,
                verrsionedSystem?.system,
                deployedCenter,
                deployedCenter?.site,
                domaine,
                crud?.find(c => c?.domaine?.id === domaine?.id)
            );
        });
    }

    convertFromCrud(milestonesCrud: IMilestoneCrud[]): IMilestone[] {
        return milestonesCrud.map(milestoneCrud => {
            const milestone = new Milestone();
            milestone.id = milestoneCrud.id;
            milestone.deleted = milestoneCrud.deleted;
            milestone.name = milestoneCrud.name;
            milestone.shortName = milestoneCrud.shortName;
            milestone.comment = milestoneCrud.comment;
            milestone.startDate = milestoneCrud.date,
                milestone.endDate = milestoneCrud.date,
                milestone.contractDate = milestoneCrud.contractDate;
            milestone.typeMilestone = milestoneCrud.typeMilestone;
            milestone.source = milestoneCrud.source;
            milestone.predecessorDependencies = milestoneCrud.predecessorDependencies;
            milestone.successorDependencies = milestoneCrud.successorDependencies;
            milestone.operation = { ...new Operation, name: milestoneCrud.operation };
            if (milestoneCrud.versionedSystem) {
                milestone.operation.versionedSystem = milestoneCrud.versionedSystem;
                milestone.operation.versionedSystem.system = milestoneCrud.system;
                milestone.operation.versionedSystem.system.domaine = milestoneCrud.domaine;
            } if (milestoneCrud.deployedCenter) {
                milestone.operation.deployedCenter = milestoneCrud.deployedCenter;
                milestone.operation.deployedCenter.site = milestoneCrud.site;
            }
            return milestone;
        });
    }
}