import { Injectable } from "@angular/core";
import * as moment from 'moment';
import { IDeployedCenter } from 'src/app/bmcs/model/deployed-center.model';
import { IVersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { Cruddomaine, ICruddomaine } from 'src/app/shared/model/entity/cruddomaine.model';
import { IOperation, Operation } from '../../model/operation.model';
import { IOperationCrud, OperationCrud } from './model/operation-crud.dto';
import { IOperationExcel, OperationExcel } from './model/operation-excel.model';

@Injectable()
export class OperationDto {
    public convertOperationsDataToExcel(operations: IOperation[] | null): IOperationExcel[] {
        return operations.map(operation => {
            const emptyOperation = new Operation();
            emptyOperation.milestones = operation.milestones;
            const operationDates = emptyOperation.getDates;

            return new OperationExcel(
                operation.manual_id,
                operation?.versionedSystem?.system?.domaine?.domaine,
                operation?.versionedSystem?.system?.name,
                operation?.versionedSystem?.name,
                operation?.deployedCenter?.name,
                operation?.deployedCenter?.site?.name,
                operation?.bmsComplement,
                operation.name,
                operation.shortName,
                operationDates[0] ? moment(operationDates[0]).format('DD/MM/YYYY') : '',
                operationDates[1] ? moment(operationDates[1]).format('DD/MM/YYYY') : '',
                operation.number,
                operation.isBMS ? 'Oui' : 'Non',
                operation.contractDGA,
                operation.program,
                operation.typeOperation.name,
                operation?.milestones?.reduce((total, mil) => total + mil.name + ', ', ''),
            )
        })
    }

    convertToCrud(operations: IOperation[], versionedSystems: IVersionedSystem[], deployedCenters: IDeployedCenter[], crudDomaines: [Cruddomaine[]], isBaseline: boolean): IOperationCrud[] {
        const crud: ICruddomaine[] = [];
        crudDomaines?.forEach(cruddomaineList => {
            cruddomaineList.forEach(cruddomaine => {
                const index = crud.findIndex(cr => cr.domaine.id === cruddomaine.domaine.id);
                if (index === -1)
                    crud.push(cruddomaine);
                else {
                    crud[index].create = cruddomaine.create;
                    crud[index].update = cruddomaine.update;
                    crud[index].deleted = cruddomaine.deleted;
                    crud[index].erase = cruddomaine.erase;
                    crud[index].is_referent = cruddomaine.is_referent;
                }
            });
        });
        return operations?.map(operation => {
            const emptyOperation = new Operation();
            emptyOperation.milestones = operation.milestones;
            const operationDates = emptyOperation.getDates;
            const versionedSystem = isBaseline ? operation?.versionedSystem : versionedSystems.find(dvs => dvs.id === operation?.versionedSystemId?.toUpperCase());
            const deployedCenter = isBaseline ? operation?.deployedCenter : deployedCenters.find(cd => cd.id === operation?.deployedCenterId?.toUpperCase());
            const domaine = versionedSystem?.system?.domaine;
            const globalText: string = operation.manual_id + operation.name + operation.shortName
                + operation.number + operation.contractDGA + operation.program + operation.typeOperation?.name +
                versionedSystem?.name + versionedSystem?.system?.name + domaine?.domaine + deployedCenter?.name + deployedCenter?.site?.name;
            operation.versionedSystem = versionedSystem;
            return new OperationCrud(
                operation.id,
                operation.manual_id,
                operation,
                operation.deleted,
                globalText,
                operation.name,
                operation.shortName,
                operationDates[0],
                operationDates[1],
                operation.number,
                operation.isBMS,
                operation.contractDGA,
                operation.program,
                operation.typeOperation,
                operation.milestones,
                versionedSystem,
                versionedSystem?.system,
                deployedCenter,
                deployedCenter?.site,
                operation?.bmsComplement,
                domaine,
                crud?.find(c => c?.domaine?.id === domaine?.id)
            );
        })
    }

    convertFromCrud(operationsCrud: IOperationCrud[]): IOperation[] {
        return operationsCrud.map(operationCrud => {
            const operation = new Operation();
            operation.id = operationCrud.id;
            operation.manual_id = operationCrud.manual_id;
            operation.deleted = operationCrud.deleted;
            operation.name = operationCrud.name;
            operation.shortName = operationCrud.shortName;
            operation.number = operationCrud.number;
            operation.isBMS = operationCrud.isBMS;
            operation.contractDGA = operationCrud.contractDGA;
            operation.program = operationCrud.program;
            operation.typeOperation = operationCrud.typeOperation;
            operation.milestones = operationCrud.milestones;
            operation.bmsComplement = operationCrud.bmsComplement;
            if (operationCrud.versionedSystem) {
                operation.versionedSystem = operationCrud.versionedSystem;
                operation.versionedSystem.system = operationCrud.system;
                operation.versionedSystem.system.domaine = operationCrud.domaine;
            } if (operationCrud.deployedCenter) {
                operation.deployedCenter = operationCrud.deployedCenter;
                operation.deployedCenter.site = operationCrud.site;
            }
            return operation;
        })
    }
}