import { Component, OnInit } from '@angular/core';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { JhiEventManager } from 'ng-jhipster';

import * as XLSX from 'xlsx';
import { FormBuilder, Validators } from '@angular/forms';
import { IOperation } from '../../model/operation.model';
import { IMilestone } from '../../model/milestone.model';
import { BcsDataDto } from '../dto/data-bcs.dto';
import { BcsNgListOptionService } from '../../entity/bcs-ng-list-option/bcs-ng-list-option.service';
import { IBcsNgListOption } from '../../model/bcs-ng-list-option.model';
import { OperationService } from '../../entity/table/operation/operation.service';
import { MilestoneService } from '../../entity/table/milestone/milestone.service';
import { VersionedSystem } from 'src/app/bmcs/model/versioned-system.model';
import { DeployedCenter } from 'src/app/bmcs/model/deployed-center.model';
import { DeployedVersionedSystemService } from 'src/app/bmcs/entity/deployed-versioned-system/deployed-versioned-system.service';


@Component({
  templateUrl: './import-dialog.component.html',
  styleUrls: ['./import-dialog.scss']
})
export class ImportBcsDialogComponent implements OnInit {
  uploadedFile?: File;
  fileName = '';
  isCompleted = false;
  arrayBuffer: any;
  file?: File;

  bcsNgListOptions?: IBcsNgListOption[] = [];

  listErrors: string[] = [];
  messageErrors: string[] = [];
  lineErrors: number[] = [];

  imported = false
  load = false;

  editForm = this.fb.group({
    file: ['', Validators.required],
  });

  constructor(
    public bcsNgListOptionService: BcsNgListOptionService,
    public operationService: OperationService,
    public milestoneService: MilestoneService,
    public activeModal: NgbActiveModal,
    protected eventManager: JhiEventManager,
    private bcsDataDto: BcsDataDto,
    protected deployedVersionedSystemService: DeployedVersionedSystemService,
    protected fb: FormBuilder) { }

  ngOnInit(): void {
    this.bcsNgListOptionService.query().subscribe(res => (this.bcsNgListOptions = res.body || []));
  }

  cancel(): void {
    this.eventManager.broadcast('esccoapvgApp.cleanError');
    this.activeModal.dismiss();
    this.eventManager.broadcast('actionListModification');
  }

  confirmDelete(): void {
    return;
  }

  fileChange(element: any): void {
    this.uploadedFile = element.target.files[0];
    this.messageErrors.length = 0;
    this.messageErrors = [];
    this.listErrors = [];
  }

  onSubmit(): void {
    if (confirm('Voulez-vous importer ce fichier ?')) {
      if (this.uploadedFile === undefined) return;
      this.load = true;
      this.imported = false;
      this.isCompleted = true;
      const reader: FileReader = new FileReader();

      reader.readAsBinaryString(this.uploadedFile);
      reader.onload = (e: any) => {
        const wb: XLSX.WorkBook = XLSX.read(e.target.result, { type: 'binary', bookVBA: true });

        /* selected the first sheet */
        const wsname: string = wb.SheetNames[0];
        const ws: XLSX.WorkSheet = wb.Sheets[wsname];
        /* save data */
        const jsonData = XLSX.utils.sheet_to_json(ws);
        const dataToImport: [IOperation[], IMilestone[]] = this.bcsDataDto.convertExcelToBcsData(jsonData, this.bcsNgListOptions);

        const operations = dataToImport[0];
        const milestones = dataToImport[1];
        let validOperations = [];
        let validMilestones = [];

        operations.forEach(operation => {
          if (!this.isOperationValid(operation)) {
            this.listErrors.push(operation.name);
            this.lineErrors.push(this.findLineError(jsonData, operation.number));
            this.messageErrors.push(this.typeErrorOperation(operation));
            //this.load = false;
          } else validOperations.push(operation);
        })
        milestones.forEach(milestone => {
          if (!this.isMilestoneValid(milestone)) {
            this.listErrors.push(milestone.name);
            this.lineErrors.push(this.findLineError(jsonData, milestone.number))
            this.messageErrors.push(this.typeErrorMilestone(milestone));
            //this.load = false;
          } else validMilestones.push(milestone)
        })


        //if (this.messageErrors.length === 0) 
        this.operationService.migrate(validOperations).subscribe(resOperation => {
          this.milestoneService.migrate(validMilestones).subscribe((resMilestone) => {
            this.imported = true;
            this.load = false;
            this.eventManager.broadcast('operationListModification');
            this.eventManager.broadcast('milestoneListModification');
          },
            () => {
              this.load = false;
            });
          this.setOperationIdOnSVD(resOperation?.body)
        });
      }
    }
  }

  private findLineError(jsonData: any[], idTache: number) {
    return jsonData.findIndex(d => d["ID_TPI"] == idTache.toString()) + 2;
  }

  private isOperationValid(operation: IOperation): boolean {
    return operation.name !== undefined &&
      operation.manual_id !== undefined &&
      operation.number !== undefined &&
      operation.typeOperation !== undefined &&
      operation.versionedSystem !== undefined;
  }

  private typeErrorOperation(operation: IOperation): string {
    let caseError = "Op: Autre cause";
    switch (true) {
      case operation.name === undefined:
        caseError = "Op: Nom non défini"
        break;
      case operation.manual_id === undefined:
        caseError = "Op: ID non défini"
        break;
      case operation.number === undefined:
        caseError = "Op: ID BCS non défini"
        break;
      case operation.typeOperation === undefined:
        caseError = "Op: Type non défini"
        break;
      case operation.versionedSystem === undefined:
        caseError = "Op: Système versionné non défini"
        break;
    }
    return caseError;
  }

  private isMilestoneValid(milestone: IMilestone): boolean {
    return milestone.name !== undefined &&
      milestone.startDate !== undefined &&
      milestone.number !== undefined &&
      milestone.source !== undefined &&
      milestone.typeMilestone !== undefined &&
      milestone.operation !== undefined;
  }

  private typeErrorMilestone(milestone: IMilestone): string {
    let caseError = "Jal: Autre cause";
    switch (true) {
      case milestone.name === undefined:
        caseError = "Jal: Nom non défini"
        break;
      case milestone.startDate === undefined:
        caseError = "Jal: Date non définie"
        break;
      case milestone.startDate.format('DD/MM/YYYY') !== milestone.endDate.format('DD/MM/YYYY'):
        caseError = "Jal: Date de début et de fin non identiques"
        break;
      case milestone.number === undefined:
        caseError = "Jal: ID BCS non défni"
        break;
      case milestone.source === undefined:
        caseError = "Jal: Source non défnie"
        break;
      case milestone.typeMilestone === undefined:
        caseError = "Jal: Type de jalon non défni"
        break;
      case milestone.operation === undefined:
        caseError = "Jal: Opération non défnie"
        break;
    }
    return caseError;
  }


  setOperationIdOnSVD(operationsCreated: IOperation[]) {
    for (let opeCreated of operationsCreated) {
      if (opeCreated?.versionedSystemId && opeCreated?.deployedCenterId) {
        this.deployedVersionedSystemService.updateOperation({
          idOperation: opeCreated.id,
          sv: { ...new VersionedSystem(), id: opeCreated?.versionedSystemId },
          cd: { ...new DeployedCenter(), id: opeCreated?.deployedCenterId },
          bmsComplement: opeCreated.bmsComplement
        }).subscribe(res => { });
      }
    }
  }

}
