import { AbstractControl, ValidatorFn } from '@angular/forms';
import { IBryntumEvent } from '../../model/bryntum/bryntum-event.model';
import { IMilestone } from '../../model/milestone.model';

export class DisplayTaskDateValidators {
    static dateLessThan(
        dateField1: string,
        dateField2: string,
        defaultDate1: Date,
        defaultDate2: Date,
        validatorField: { [key: string]: boolean }
    ): ValidatorFn {
        return (c: AbstractControl): { [key: string]: boolean } | null => {
            const date1 = c.get(dateField1).value ?? defaultDate1;
            const date2 = c.get(dateField2).value ?? defaultDate2;
            if ((date1 !== null && date2 !== null) && date1 > date2) return validatorField;
            return null;
        };
    }

    static afterPredecessors(dateField: string, defaultDate: Date, selectedPredecessors: () => (IBryntumEvent|IMilestone)[], validatorField: { [key: string]: boolean }): ValidatorFn {
        return (c: AbstractControl): { [key: string]: boolean } | null => {
            const date = c.get(dateField).value ?? defaultDate;
            date?.setHours(0, 0, 0, 0);
            const predecessors = selectedPredecessors();
            if (date !== null && predecessors.some(pred => {
                const predDate = new Date(pred?.startDate?.toString());
                predDate?.setHours(0, 0, 0, 0);
                return predDate > date;
            })) {
                return validatorField;
            }
            return null;
        }
    }

    static beforeSuccessors(dateField: string, defaultDate: Date, successors: IBryntumEvent[], validatorField: { [key: string]: boolean }) {
        return (c: AbstractControl): { [key: string]: boolean } | null => {
            const date = c.get(dateField).value ?? defaultDate;
            date?.setHours(0, 0, 0, 0);
            if (date !== null && successors.some(successor => {
                const successorDate = new Date(successor.startDate?.toString());
                successorDate?.setHours(0, 0, 0, 0);
                return successorDate < date;
            })) {
                return validatorField;
            }
            return null;
        }
    }
}