import { AbstractControl, ValidatorFn } from '@angular/forms';
import { IMilestone } from 'src/app/bcs_ng/model/milestone.model';

export class GenericTaskDateValidators {
    static dateLessThan(dateField1: string, dateField2: string, validatorField: { [key: string]: boolean }): ValidatorFn {
        return (c: AbstractControl): { [key: string]: boolean } | null => {
            const date1 = c.get(dateField1).value;
            const date2 = c.get(dateField2).value;
            if ((date1 !== null && date2 !== null) && date1 > date2) {
                return validatorField;
            }
            return null;
        };
    }

    static afterPredecessors(dateField: string, selectedPredecessors: () => IMilestone[], validatorField: { [key: string]: boolean }): ValidatorFn {
        return (c: AbstractControl): { [key: string]: boolean } | null => {
            const date = c.get(dateField).value;
            date?.setHours(0, 0, 0, 0);
            const predecessors = selectedPredecessors();
            if (date !== null && predecessors.some(pred => {
                const predDate = new Date(pred?.startDate?.toString());
                predDate?.setHours(0, 0, 0, 0);
                return predDate > date;
            })) {
                return validatorField;
            }
            return null;
        }
    }

    static beforeSuccessors(dateField: string, successors: IMilestone[], validatorField: { [key: string]: boolean }) {
        return (c: AbstractControl): { [key: string]: boolean } | null => {
            const date = c.get(dateField).value;
            date?.setHours(0, 0, 0, 0);
            if (date !== null && successors.some(successor => {
                const successorDate = new Date(successor.startDate?.toString());
                successorDate?.setHours(0, 0, 0, 0);
                return successorDate < date;
            })) {
                return validatorField;
            }
            return null;
        }
    }
}