import { Injectable } from '@angular/core';
import { Scheduler } from '@bryntum/scheduler';
import { TimeRangeMode } from '../model/bryntum/display-configuration/display-conf-road-map-properties';

@Injectable()
export class BryntumUtils {
    scheduler?: Scheduler;

    static getLevelsOfNode(resourceStore): number[] {
        let level: number[] = [];
        resourceStore.allRecords.forEach(resource => {
            if (!level.includes(resource.childLevel)) level.push(resource.childLevel);
        });
        return level;
    }

    zoomIn() {
        this.scheduler.zoomIn();
    }

    zoomOut() {
        this.scheduler.zoomOut();
    }

    zoomAuto() {
        this.scheduler.zoomToFit();
        const totalTimeSpan: any = this.scheduler.eventStore.getTotalTimeSpan();
        this.scheduler.startDate = totalTimeSpan.startDate;
        this.scheduler.endDate = totalTimeSpan.endDate;
    }

    increaseRowHeight(): void {
        this.scheduler.rowHeight += 10;
        this.scheduler.barMargin = this.scheduler.rowHeight / 20;
        this.scheduler.eventStore.traverse((event) => {
            event.set('rowHeight', this.scheduler.rowHeight);
        });
    }

    reduceRowHeight(): void {
        if (this.scheduler.rowHeight > 20) {
            this.scheduler.rowHeight -= 10;
            this.scheduler.barMargin = this.scheduler.rowHeight / 20;
            this.scheduler.eventStore.traverse((event) => {
                event.set('rowHeight', this.scheduler.rowHeight);
            });
        };
        
    }

    expandAllLevel(expand: boolean = true) {
        this.scheduler.resourceStore.traverse(resource => {
            this.scheduler.resourceStore.toggleCollapse(resource?.data?.id, !expand);
        })
    }

    expandLevel(level: number) {
        if (level == 0) this.expandAllLevel(false);
        else {
            this.scheduler.resourceStore.traverse(resource => {
                if (resource.childLevel < level) {
                    this.scheduler.resourceStore.toggleCollapse(resource?.data?.id, false);
                } else {
                    this.scheduler.resourceStore.toggleCollapse(resource?.data?.id, true);
                }
            })
        }
    }

    buildTimeRange(timeRangeStartDate: Date, timeRangeMode: TimeRangeMode): { startDate: Date, name: string } {
        const date = timeRangeMode === TimeRangeMode.TODAY ? new Date() : timeRangeStartDate;
        const name = date ? date.toLocaleDateString() : '';
        return {
          startDate: date,
          name,
        }
    }
}