import * as moment from 'moment';
import 'moment-precise-range-plugin';
import { Moment } from 'moment';
import { TimeUnit } from '../model/enumeration/time-unit.enum';


export class Utils {

    static calculateMomentDiff(moment1: Moment, moment2: Moment): number {
        return Math.floor(moment2.diff(moment1) / (1000 * 3600 * 24));
    }

    static calculateMomentPreciseDiff(moment1: Moment, moment2: Moment): { years: number, months: number, days: number } {
        const preciseDiff = moment.preciseDiff(moment1, moment2, true);
        return { years: preciseDiff.years, months: preciseDiff.months, days: preciseDiff.days };
    }

    static addTimegapToDate(initialDate: Date, timeGapDuration: number, timeGapUnit: TimeUnit): Date {
        const result = new Date(initialDate);
        if (timeGapUnit === TimeUnit.DAYS) {
            result.setDate(result.getDate() + timeGapDuration);
        } else if (timeGapUnit === TimeUnit.MONTHS) {
            result.setMonth(result.getMonth() + timeGapDuration);
        } else if (timeGapUnit === TimeUnit.YEARS) {
            result.setFullYear(result.getFullYear() + timeGapDuration);
        }
        return result;
    }

    static addTimeGapToMoment(initialMoment: Moment, timeGapDuration: number, timeGapUnit: TimeUnit): Moment {
        let dateUnit: 'years' | 'months' | 'days' = 'months';
        if (timeGapUnit === TimeUnit.YEARS) dateUnit = 'years';
        if (timeGapUnit === TimeUnit.DAYS) dateUnit = 'days';
        return initialMoment.clone().add(timeGapDuration, dateUnit);
    }

    static createUUID() {
        let nbr = 0, randStr = "";
        do {
            randStr += (nbr = Math.random()).toString(16).substr(3, 6);
        } while (randStr.length < 30);
        return (
            randStr.substr(0, 8) + "-" +
            randStr.substr(8, 4) + "-4" +
            randStr.substr(12, 3) + "-" +
            ((nbr * 4 | 0) + 8).toString(16) +
            randStr.substr(15, 3) + "-" +
            randStr.substr(18, 12)
        );
    }

    static isDateValid(date: Date) {
        return moment(date).isValid();
    }
}
